@@ -1,61 +1,21 @@
 /*
- * The Apache Software License, Version 1.1
- *
- * Copyright (c) 1999-2002 The Apache Software Foundation.  All rights
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xerces" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written
- *    permission, please contact apache\@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation, and was
- * originally based on software copyright (c) 1999, International
- * Business Machines, Inc., http://www.ibm.com .  For more information
- * on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Copyright 1999-2002,2004 The Apache Software Foundation.
+ * 
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 /*
- * $Id: DOMParser.hpp,v 1.16 2003/01/09 19:07:44 tng Exp $
+ * $Id: DOMParser.hpp,v 1.25 2004/09/08 13:55:42 peiyongz Exp $
  *
  */
 
@@ -66,7 +26,6 @@
 #include <xercesc/framework/XMLErrorReporter.hpp>
 #include <xercesc/framework/XMLEntityHandler.hpp>
 #include <xercesc/util/ValueStackOf.hpp>
-
 #include <xercesc/validators/DTD/DocTypeHandler.hpp>
 #include <xercesc/validators/DTD/DTDElementDecl.hpp>
 #include "DOM_Document.hpp"
@@ -82,6 +41,9 @@ class XMLScanner;
 class XMLValidator;
 class Grammar;
 class GrammarResolver;
+class XMLGrammarPool;
+class XMLEntityResolver;
+class PSVIHandler;
 
 /**
   * This class implements the Document Object Model (DOM) interface.
@@ -93,12 +55,13 @@ class GrammarResolver;
   * <p>It can be used to instantiate a validating or non-validating
   * parser, by setting a member flag.</p>
   */
-class PARSERS_EXPORT DOMParser :
+class DEPRECATED_DOM_EXPORT DOMParser :
 
     public XMLDocumentHandler
     , public XMLErrorReporter
     , public XMLEntityHandler
     , public DocTypeHandler
+    , public XMemory
 {
 public :
     // -----------------------------------------------------------------------
@@ -127,7 +90,12 @@ public :
       * @param valToAdopt Pointer to the validator instance to use. The
       *                   parser is responsible for freeing the memory.
       */
-    DOMParser(XMLValidator* const valToAdopt = 0);
+    DOMParser
+    (
+          XMLValidator* const   valToAdopt = 0
+        , MemoryManager* const  manager = XMLPlatformUtils::fgMemoryManager
+        , XMLGrammarPool* const gramPool = 0
+    );
 
     /**
       * Destructor
@@ -180,6 +148,24 @@ public :
       */
     const ErrorHandler* getErrorHandler() const;
 
+    /** Get a pointer to the PSVI handler
+      *
+      * This method returns the installed PSVI handler. If no handler
+      * has been installed, then it will be a zero pointer.
+      *
+      * @return The pointer to the installed PSVI handler object.
+      */
+    PSVIHandler* getPSVIHandler();
+
+    /** Get a const pointer to the error handler
+      *
+      * This method returns the installed error handler.  If no handler
+      * has been installed, then it will be a zero pointer.
+      *
+      * @return A const pointer to the installed error handler object.
+      */
+    const PSVIHandler* getPSVIHandler() const;
+
     /** Get a pointer to the entity resolver
       *
       * This method returns the installed entity resolver.  If no resolver
@@ -198,6 +184,24 @@ public :
       */
     const EntityResolver* getEntityResolver() const;
 
+    /** Get a pointer to the entity resolver
+      *
+      * This method returns the installed entity resolver.  If no resolver
+      * has been installed, then it will be a zero pointer.
+      *
+      * @return The pointer to the installed entity resolver object.
+      */
+    XMLEntityResolver* getXMLEntityResolver();
+
+    /** Get a const pointer to the entity resolver
+      *
+      * This method returns the installed entity resolver. If no resolver
+      * has been installed, then it will be a zero pointer.
+      *
+      * @return A const pointer to the installed entity resolver object.
+      */
+    const XMLEntityResolver* getXMLEntityResolver() const;
+
     /** Get a const reference to the underlying scanner
       *
       * This method returns a reference to the underlying scanner object.
@@ -249,6 +253,8 @@ public :
       */
     bool getValidationSchemaFullChecking() const;
 
+    bool getIdentityConstraintChecking() const;
+
     /** Get error count from the last parse operation.
       *
       * This method returns the error count from the last parse
@@ -480,6 +486,21 @@ public :
       */
     void setErrorHandler(ErrorHandler* const handler);
 
+    /** Set the PSVI handler
+      *
+      * This method allows applications to install their own PSVI handler.
+      *
+      * <i>Any previously set handler is merely dropped, since the parser
+      * does not own them.</i>
+      *
+      * @param handler  A const pointer to the user supplied PSVI
+      *                 handler.
+      *
+      * @see #getPSVIHandler
+      */
+    void setPSVIHandler(PSVIHandler* const handler);
+
+
     /** Set the entity resolver
       *
       * This method allows applications to install their own entity
@@ -487,8 +508,9 @@ public :
       * can trap and potentially redirect references to external
       * entities.
       *
-      * <i>Any previously set resolver is merely dropped, since the parser
-      * does not own them.</i>
+      * <i>Any previously set entity resolver is merely dropped, since the parser
+      * does not own them.  If both setEntityResolver and setXMLEntityResolver
+      * are called, then the last one is used.</i>
       *
       * @param handler  A const pointer to the user supplied entity
       *                 resolver.
@@ -497,6 +519,24 @@ public :
       */
     void setEntityResolver(EntityResolver* const handler);
 
+    /** Set the entity resolver
+      *
+      * This method allows applications to install their own entity
+      * resolver. By installing an entity resolver, the applications
+      * can trap and potentially redirect references to external
+      * entities.
+      *
+      * <i>Any previously set entity resolver is merely dropped, since the parser
+      * does not own them.  If both setEntityResolver and setXMLEntityResolver
+      * are called, then the last one is used.</i>
+      *
+      * @param handler  A const pointer to the user supplied entity
+      *                 resolver.
+      *
+      * @see #getXMLEntityResolver
+      */
+    void setXMLEntityResolver(XMLEntityResolver* const handler);
+
     /** Set the 'do namespaces' flag
       *
       * This method allows users to enable or disable the parser's
@@ -648,6 +688,8 @@ public :
       */
     void setValidationSchemaFullChecking(const bool schemaFullChecking);
 
+    void setIdentityConstraintChecking(const bool identityConstraintChecking);
+
     /**
       * This method allows users to set the toCreateXMLDeclTypeNode flag
       * by this parser. By setting it to 'true' user can have XMLDecl type
@@ -1223,6 +1265,8 @@ public :
       * implement 'redirection' via this callback. This method is also
       * borrowed from the SAX specification.
       *
+      * @deprecated This method is no longer called (the other resolveEntity one is).
+      *
       * @param publicId A const pointer to a Unicode string representing the
       *                 public id of the entity just parsed.
       * @param systemId A const pointer to a Unicode string representing the
@@ -1244,6 +1288,27 @@ public :
         , const XMLCh* const    baseURI = 0
     );
 
+    /** Resolve a public/system id
+      *
+      * This method allows a user installed entity handler to further
+      * process any pointers to external entities. The applications can
+      * implement 'redirection' via this callback.  
+      *
+      * @param resourceIdentifier An object containing the type of
+      *        resource to be resolved and the associated data members
+      *        corresponding to this type.
+      * @return The value returned by the user installed resolveEntity
+      *         method or NULL otherwise to indicate no processing was done.
+      *         The returned InputSource is owned by the parser which is
+      *         responsible to clean up the memory.
+      * @see XMLEntityHandler
+      * @see XMLEntityResolver
+      */
+    virtual InputSource* resolveEntity
+    (
+        XMLResourceIdentifier* resourceIdentifier
+    );
+
     /** Handle a 'start input source' event
       *
       * This method is used to indicate the start of parsing an external
@@ -1684,6 +1749,10 @@ private :
     void initialize();
     void cleanUp();
 
+    // unimplemented
+    DOMParser ( const DOMParser& toCopy);
+    DOMParser& operator= (const DOMParser& other);
+
     // -----------------------------------------------------------------------
     //  Private data members
     //
@@ -1732,6 +1801,10 @@ private :
     //      A flag to create a DOM_XMLDecl node in the ODM tree if it exists
     //      This is an extension to xerces implementation
     //
+    //   fGrammarPool
+    //      The grammar pool passed from external application (through derivatives).
+    //      which could be 0, not owned.
+    //
     // -----------------------------------------------------------------------
     bool                    fToCreateXMLDeclTypeNode;
     bool                    fCreateEntityReferenceNodes;
@@ -1742,13 +1815,17 @@ private :
     DOM_Node                fCurrentNode;
     DOM_Document            fDocument;
     EntityResolver*         fEntityResolver;
+    XMLEntityResolver*      fXMLEntityResolver;
     ErrorHandler*           fErrorHandler;
+    PSVIHandler*            fPSVIHandler;
     ValueStackOf<DOM_Node>* fNodeStack;
     XMLScanner*             fScanner;
     DocumentTypeImpl*       fDocumentType;
     GrammarResolver*        fGrammarResolver;
     XMLStringPool*          fURIStringPool;
     XMLValidator*           fValidator;
+    MemoryManager*          fMemoryManager;
+    XMLGrammarPool*         fGrammarPool;
 };
 
 
@@ -1795,11 +1872,31 @@ inline const ErrorHandler* DOMParser::getErrorHandler() const
     return fErrorHandler;
 }
 
+inline PSVIHandler* DOMParser::getPSVIHandler()
+{
+    return fPSVIHandler;
+}
+
+inline const PSVIHandler* DOMParser::getPSVIHandler() const
+{
+    return fPSVIHandler;
+}
+
 inline EntityResolver* DOMParser::getEntityResolver()
 {
     return fEntityResolver;
 }
 
+inline XMLEntityResolver* DOMParser::getXMLEntityResolver()
+{
+    return fXMLEntityResolver;
+}
+
+inline const XMLEntityResolver* DOMParser::getXMLEntityResolver() const
+{
+    return fXMLEntityResolver;
+}
+
 inline const EntityResolver* DOMParser::getEntityResolver() const
 {
     return fEntityResolver;
