@@ -2,63 +2,23 @@
 #define DocumentImpl_HEADER_GUARD_
 
 /*
- * The Apache Software License, Version 1.1
- *
- * Copyright (c) 1999-2002 The Apache Software Foundation.  All rights
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xerces" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written
- *    permission, please contact apache\@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation, and was
- * originally based on software copyright (c) 1999, International
- * Business Machines, Inc., http://www.ibm.com .  For more information
- * on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Copyright 1999-2002,2004 The Apache Software Foundation.
+ * 
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 /*
- * $Id: DocumentImpl.hpp,v 1.4 2002/12/20 22:10:57 tng Exp $
+ * $Id: DocumentImpl.hpp,v 1.9 2004/09/08 13:55:43 peiyongz Exp $
  */
 
 //
@@ -109,7 +69,7 @@ typedef RefVectorOf<TreeWalkerImpl> TreeWalkers;
 typedef RefVectorOf<RangeImpl> RangeImpls;
 
 
-class CDOM_EXPORT DocumentImpl: public ParentNode {
+class DEPRECATED_DOM_EXPORT DocumentImpl: public ParentNode {
 private:
     // -----------------------------------------------------------------------
     //  Private data types
@@ -166,15 +126,19 @@ private:
     /** Bypass error checking. */
     bool errorChecking;
 
+    MemoryManager               *fMemoryManager;
+
     friend class NodeIteratorImpl;
     friend class TreeWalkerImpl;
     friend class RangeImpl;
    	friend class DOMParser;
 
 public:
-    DocumentImpl();
+    DocumentImpl(MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager);
     DocumentImpl(const DOMString &namespaceURI,	    //DOM Level 2
-	const DOMString &qualifiedName, DocumentTypeImpl *doctype);
+	             const DOMString &qualifiedName,
+                 DocumentTypeImpl *doctype,
+                 MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager);
     virtual ~DocumentImpl();
     virtual bool isDocumentImpl();   // RTTI replacement function
 
@@ -205,8 +169,10 @@ public:
     virtual void                referenced();
     virtual NodeImpl            *removeChild(NodeImpl *oldChild);
     virtual void                unreferenced();
-    static  NodeIteratorImpl*   createNodeIterator(DOM_Node root, unsigned long whatToShow, DOM_NodeFilter* filter, bool entityReferenceExpansion);
-    static  TreeWalkerImpl*     createTreeWalker(DOM_Node root, unsigned long whatToShow, DOM_NodeFilter* filter, bool entityReferenceExpansion);
+    static  NodeIteratorImpl*   createNodeIterator(DOM_Node root, unsigned long whatToShow, DOM_NodeFilter* filter, bool entityReferenceExpansion,
+                                                   MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager);
+    static  TreeWalkerImpl*     createTreeWalker(DOM_Node root, unsigned long whatToShow, DOM_NodeFilter* filter, bool entityReferenceExpansion,
+                                                 MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager);
     virtual XMLDeclImpl*        createXMLDecl(const DOMString& version, const DOMString& encoding, const DOMString& standalone);
     virtual void*				getUserData();
     virtual void				setUserData(void* value);
@@ -268,6 +234,10 @@ public:
         return errorChecking;
     }
 
+    inline MemoryManager* getMemoryManager() const {
+        return fMemoryManager;
+    }
+
     // -----------------------------------------------------------------------
     //  Notification that lazy data has been deleted
     // -----------------------------------------------------------------------
