@@ -1,61 +1,21 @@
 /*
- * The Apache Software License, Version 1.1
- *
- * Copyright (c) 2001-2002 The Apache Software Foundation.  All rights
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xerces" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written
- *    permission, please contact apache\@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation, and was
- * originally based on software copyright (c) 2001, International
- * Business Machines, Inc., http://www.ibm.com .  For more information
- * on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Copyright 2001-2004 The Apache Software Foundation.
+ * 
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 /*
- * $Id: DOMElementImpl.cpp,v 1.19 2003/01/29 19:43:08 gareth Exp $
+ * $Id: DOMElementImpl.cpp,v 1.28 2004/09/20 15:00:50 amassari Exp $
  */
 
 #include "DOMElementImpl.hpp"
@@ -70,22 +30,19 @@
 #include "DOMAttrImpl.hpp"
 #include "DOMDocumentImpl.hpp"
 #include "DOMParentNode.hpp"
-#include "DOMStringPool.hpp"
 #include "DOMCasts.hpp"
 #include "DOMElementNSImpl.hpp"
 #include "DOMTypeInfoImpl.hpp"
 
-#include "DOMDeepNodeListImpl.hpp"
 #include "DOMDocumentTypeImpl.hpp"
-#include "DOMNamedNodeMapImpl.hpp"
+#include <xercesc/util/OutOfMemoryException.hpp>
 
 XERCES_CPP_NAMESPACE_BEGIN
 
-
 class DOMAttr;
 
 DOMElementImpl::DOMElementImpl(DOMDocument *ownerDoc, const XMLCh *eName)
-    : fNode(ownerDoc), fParent(ownerDoc), fAttributes(0), fDefaultAttributes(0), fSchemaType(0)
+    : fNode(ownerDoc), fParent(ownerDoc), fAttributes(0), fDefaultAttributes(0)
 {
     DOMDocumentImpl *docImpl = (DOMDocumentImpl *)ownerDoc;
     fName = docImpl->getPooledString(eName);
@@ -97,13 +54,12 @@ DOMElementImpl::DOMElementImpl(DOMDocument *ownerDoc, const XMLCh *eName)
     else {
         fAttributes = new (getOwnerDocument()) DOMAttrMapImpl(this, fDefaultAttributes);
     }
-};
+}
 
 
 DOMElementImpl::DOMElementImpl(const DOMElementImpl &other, bool deep)
     : fNode(other.getOwnerDocument()),
       fParent(other.getOwnerDocument()),
-      fSchemaType(other.fSchemaType),
       fAttributes(0),
       fDefaultAttributes(0)
 {
@@ -137,12 +93,12 @@ DOMElementImpl::DOMElementImpl(const DOMElementImpl &other, bool deep)
         }
     }
 
-};
+}
 
 
 DOMElementImpl::~DOMElementImpl()
 {
-};
+}
 
 
 DOMNode *DOMElementImpl::cloneNode(bool deep) const
@@ -150,48 +106,43 @@ DOMNode *DOMElementImpl::cloneNode(bool deep) const
     DOMNode* newNode = new (getOwnerDocument(), DOMDocumentImpl::ELEMENT_OBJECT) DOMElementImpl(*this, deep);
     fNode.callUserDataHandlers(DOMUserDataHandler::NODE_CLONED, this, newNode);
     return newNode;
-};
+}
 
 
 
 
 const XMLCh * DOMElementImpl::getNodeName() const {
     return fName;
-};
+}
 
 
 short DOMElementImpl::getNodeType() const {
     return DOMNode::ELEMENT_NODE;
-};
+}
 
 
 const XMLCh * DOMElementImpl::getAttribute(const XMLCh *nam) const
 {
-    static const XMLCh emptyString[]  = {0};
-    DOMNode * attr=0;
+    DOMNode * attr = fAttributes->getNamedItem(nam);
+    if (attr)
+        return attr->getNodeValue();
 
-    attr=fAttributes->getNamedItem(nam);
-
-    const XMLCh *retString = emptyString;
-    if (attr != 0)
-        retString = attr->getNodeValue();
-
-    return retString;
-};
+    return XMLUni::fgZeroLenString;
+}
 
 
 
 DOMAttr *DOMElementImpl::getAttributeNode(const XMLCh *nam) const
 {
     return  (DOMAttr *)fAttributes->getNamedItem(nam);
-};
+}
 
 
 DOMNamedNodeMap *DOMElementImpl::getAttributes() const
 {
     DOMElementImpl *ncThis = (DOMElementImpl *)this;   // cast off const
     return ncThis->fAttributes;
-};
+}
 
 
 
@@ -199,7 +150,7 @@ DOMNodeList *DOMElementImpl::getElementsByTagName(const XMLCh *tagname) const
 {
     DOMDocumentImpl *docImpl = (DOMDocumentImpl *)getOwnerDocument();
     return docImpl->getDeepNodeList(this,tagname);
-};
+}
 
 
 const XMLCh * DOMElementImpl::getTagName() const
@@ -212,7 +163,7 @@ void DOMElementImpl::removeAttribute(const XMLCh *nam)
 {
     if (fNode.isReadOnly())
         throw DOMException(
-             DOMException::NO_MODIFICATION_ALLOWED_ERR, 0);
+             DOMException::NO_MODIFICATION_ALLOWED_ERR, 0, GetDOMNodeMemoryManager);
 
     XMLSSize_t i = fAttributes->findNamePoint(nam);
     if (i >= 0)
@@ -221,7 +172,7 @@ void DOMElementImpl::removeAttribute(const XMLCh *nam)
         ((DOMAttrImpl *)att)->removeAttrFromIDNodeMap();
         att->release();
     }
-};
+}
 
 
 
@@ -229,7 +180,7 @@ DOMAttr *DOMElementImpl::removeAttributeNode(DOMAttr *oldAttr)
 {
     if (fNode.isReadOnly())
         throw DOMException(
-        DOMException::NO_MODIFICATION_ALLOWED_ERR, 0);
+        DOMException::NO_MODIFICATION_ALLOWED_ERR, 0, GetDOMNodeMemoryManager);
 
     DOMNode* found = 0;
 
@@ -249,14 +200,14 @@ DOMAttr *DOMElementImpl::removeAttributeNode(DOMAttr *oldAttr)
             ((DOMAttrImpl *)oldAttr)->removeAttrFromIDNodeMap();
         }
         else
-            throw DOMException(DOMException::NOT_FOUND_ERR, 0);
+            throw DOMException(DOMException::NOT_FOUND_ERR, 0, GetDOMNodeMemoryManager);
 
     }
     else
-        throw DOMException(DOMException::NOT_FOUND_ERR, 0);
+        throw DOMException(DOMException::NOT_FOUND_ERR, 0, GetDOMNodeMemoryManager);
 
    return (DOMAttr *)found;
-};
+}
 
 
 
@@ -264,7 +215,7 @@ void DOMElementImpl::setAttribute(const XMLCh *nam, const XMLCh *val)
 {
     if (fNode.isReadOnly())
         throw DOMException(
-        DOMException::NO_MODIFICATION_ALLOWED_ERR, 0);
+        DOMException::NO_MODIFICATION_ALLOWED_ERR, 0, GetDOMNodeMemoryManager);
 
     DOMAttr* newAttr = getAttributeNode(nam);
     if (!newAttr)
@@ -274,43 +225,43 @@ void DOMElementImpl::setAttribute(const XMLCh *nam, const XMLCh *val)
     }
 
     newAttr->setNodeValue(val);
-};
+}
 
 void DOMElementImpl::setIdAttribute(const XMLCh* name)
 {
     if (fNode.isReadOnly())
         throw DOMException(
-        DOMException::NO_MODIFICATION_ALLOWED_ERR, 0);
+        DOMException::NO_MODIFICATION_ALLOWED_ERR, 0, GetDOMNodeMemoryManager);
 
     DOMAttr *attr = getAttributeNode(name);
 
     if (!attr) 
-        throw DOMException(DOMException::NOT_FOUND_ERR, 0);
+        throw DOMException(DOMException::NOT_FOUND_ERR, 0, GetDOMNodeMemoryManager);
 
     ((DOMAttrImpl *)attr)->addAttrToIDNodeMap();
-};
+}
 
 void DOMElementImpl::setIdAttributeNS(const XMLCh* namespaceURI, const XMLCh* localName) {
 
     if (fNode.isReadOnly())
         throw DOMException(
-        DOMException::NO_MODIFICATION_ALLOWED_ERR, 0);
+        DOMException::NO_MODIFICATION_ALLOWED_ERR, 0, GetDOMNodeMemoryManager);
 
     DOMAttr *attr = getAttributeNodeNS(namespaceURI, localName);
 
     if (!attr) 
-        throw DOMException(DOMException::NOT_FOUND_ERR, 0);
+        throw DOMException(DOMException::NOT_FOUND_ERR, 0, GetDOMNodeMemoryManager);
 
     ((DOMAttrImpl *)attr)->addAttrToIDNodeMap();
 
-};
+}
 
 
 void DOMElementImpl::setIdAttributeNode(const DOMAttr *idAttr) {
 
     if (fNode.isReadOnly())
         throw DOMException(
-        DOMException::NO_MODIFICATION_ALLOWED_ERR, 0);
+        DOMException::NO_MODIFICATION_ALLOWED_ERR, 0, GetDOMNodeMemoryManager);
 
     DOMAttr *attr;
     const XMLCh* localName = idAttr->getLocalName();
@@ -320,33 +271,33 @@ void DOMElementImpl::setIdAttributeNode(const DOMAttr *idAttr) {
         attr = getAttributeNode(idAttr->getName());
     
     if(!attr) 
-        throw DOMException(DOMException::NOT_FOUND_ERR, 0);
+        throw DOMException(DOMException::NOT_FOUND_ERR, 0, GetDOMNodeMemoryManager);
 
     ((DOMAttrImpl *)attr)->addAttrToIDNodeMap();
-};
+}
 
 
 DOMAttr * DOMElementImpl::setAttributeNode(DOMAttr *newAttr)
 {
     if (fNode.isReadOnly())
         throw DOMException(
-        DOMException::NO_MODIFICATION_ALLOWED_ERR, 0);
+        DOMException::NO_MODIFICATION_ALLOWED_ERR, 0, GetDOMNodeMemoryManager);
 
     if (newAttr->getNodeType() != DOMNode::ATTRIBUTE_NODE)
-        throw DOMException(DOMException::WRONG_DOCUMENT_ERR, 0);
+        throw DOMException(DOMException::WRONG_DOCUMENT_ERR, 0, GetDOMNodeMemoryManager);
         // revisit.  Exception doesn't match test.
 
     // This will throw INUSE if necessary
     DOMAttr *oldAttr = (DOMAttr *) fAttributes->setNamedItem(newAttr);
 
     return oldAttr;
-};
+}
 
 
 void DOMElementImpl::setNodeValue(const XMLCh *x)
 {
     fNode.setNodeValue(x);
-};
+}
 
 
 
@@ -354,7 +305,7 @@ void DOMElementImpl::setReadOnly(bool readOnl, bool deep)
 {
     fNode.setReadOnly(readOnl,deep);
     fAttributes->setReadOnly(readOnl,true);
-};
+}
 
 
 //Introduced in DOM Level 2
@@ -372,7 +323,7 @@ void DOMElementImpl::setAttributeNS(const XMLCh *fNamespaceURI,
 {
     if (fNode.isReadOnly())
         throw DOMException(
-        DOMException::NO_MODIFICATION_ALLOWED_ERR, 0);
+        DOMException::NO_MODIFICATION_ALLOWED_ERR, 0, GetDOMNodeMemoryManager);
 
     DOMAttr* newAttr = getAttributeNodeNS(fNamespaceURI, qualifiedName);
 
@@ -391,7 +342,7 @@ void DOMElementImpl::removeAttributeNS(const XMLCh *fNamespaceURI,
 {
     if (fNode.isReadOnly())
         throw DOMException(
-        DOMException::NO_MODIFICATION_ALLOWED_ERR, 0);
+        DOMException::NO_MODIFICATION_ALLOWED_ERR, 0, GetDOMNodeMemoryManager);
 
     XMLSSize_t i = fAttributes->findNamePoint(fNamespaceURI, fLocalName);
     if (i >= 0)
@@ -413,10 +364,10 @@ DOMAttr *DOMElementImpl::setAttributeNodeNS(DOMAttr *newAttr)
 {
     if (fNode.isReadOnly())
         throw DOMException(
-            DOMException::NO_MODIFICATION_ALLOWED_ERR, 0);
+            DOMException::NO_MODIFICATION_ALLOWED_ERR, 0, GetDOMNodeMemoryManager);
 
     if (newAttr -> getOwnerDocument() != this -> getOwnerDocument())
-        throw DOMException(DOMException::WRONG_DOCUMENT_ERR, 0);
+        throw DOMException(DOMException::WRONG_DOCUMENT_ERR, 0, GetDOMNodeMemoryManager);
 
     // This will throw INUSE if necessary
     DOMAttr *oldAttr = (DOMAttr *) fAttributes->setNamedItemNS(newAttr);
@@ -428,27 +379,27 @@ DOMAttr *DOMElementImpl::setAttributeNodeNS(DOMAttr *newAttr)
 DOMNodeList *DOMElementImpl::getElementsByTagNameNS(const XMLCh *namespaceURI,
     const XMLCh *localName) const
 {
-    DOMDocumentImpl *docImpl = (DOMDocumentImpl *)getOwnerDocument();;
+    DOMDocumentImpl *docImpl = (DOMDocumentImpl *)getOwnerDocument();
     return docImpl->getDeepNodeList(this, namespaceURI, localName);
 }
 
 bool DOMElementImpl::hasAttributes() const
 {
     return (fAttributes != 0 && fAttributes->getLength() != 0);
-};
+}
 
 
 bool DOMElementImpl::hasAttribute(const XMLCh *name) const
 {
     return (getAttributeNode(name) != 0);
-};
+}
 
 
 bool DOMElementImpl::hasAttributeNS(const XMLCh *namespaceURI,
     const XMLCh *localName) const
 {
     return (getAttributeNodeNS(namespaceURI, localName) != 0);
-};
+}
 
 
 // util functions for default attributes
@@ -476,10 +427,10 @@ DOMAttr * DOMElementImpl::setDefaultAttributeNode(DOMAttr *newAttr)
 {
     if (fNode.isReadOnly())
         throw DOMException(
-        DOMException::NO_MODIFICATION_ALLOWED_ERR, 0);
+        DOMException::NO_MODIFICATION_ALLOWED_ERR, 0, GetDOMNodeMemoryManager);
 
     if (newAttr->getNodeType() != DOMNode::ATTRIBUTE_NODE)
-        throw DOMException(DOMException::WRONG_DOCUMENT_ERR, 0);
+        throw DOMException(DOMException::WRONG_DOCUMENT_ERR, 0, GetDOMNodeMemoryManager);
         // revisit.  Exception doesn't match test.
 
     // This will throw INUSE if necessary
@@ -487,17 +438,17 @@ DOMAttr * DOMElementImpl::setDefaultAttributeNode(DOMAttr *newAttr)
     fAttributes->hasDefaults(true);
 
     return oldAttr;
-};
+}
 
 
 DOMAttr *DOMElementImpl::setDefaultAttributeNodeNS(DOMAttr *newAttr)
 {
     if (fNode.isReadOnly())
         throw DOMException(
-            DOMException::NO_MODIFICATION_ALLOWED_ERR, 0);
+            DOMException::NO_MODIFICATION_ALLOWED_ERR, 0, GetDOMNodeMemoryManager);
 
     if (newAttr -> getOwnerDocument() != this -> getOwnerDocument())
-        throw DOMException(DOMException::WRONG_DOCUMENT_ERR, 0);
+        throw DOMException(DOMException::WRONG_DOCUMENT_ERR, 0, GetDOMNodeMemoryManager);
 
     // This will throw INUSE if necessary
     DOMAttr *oldAttr = (DOMAttr *) fDefaultAttributes->setNamedItemNS(newAttr);
@@ -509,7 +460,7 @@ DOMAttr *DOMElementImpl::setDefaultAttributeNodeNS(DOMAttr *newAttr)
 void DOMElementImpl::release()
 {
     if (fNode.isOwned() && !fNode.isToBeReleased())
-        throw DOMException(DOMException::INVALID_ACCESS_ERR,0);
+        throw DOMException(DOMException::INVALID_ACCESS_ERR,0, GetDOMNodeMemoryManager);
 
     DOMDocumentImpl* doc = (DOMDocumentImpl*) getOwnerDocument();
     if (doc) {
@@ -519,7 +470,7 @@ void DOMElementImpl::release()
     }
     else {
         // shouldn't reach here
-        throw DOMException(DOMException::INVALID_ACCESS_ERR,0);
+        throw DOMException(DOMException::INVALID_ACCESS_ERR,0, GetDOMNodeMemoryManager);
     }
 }
 
@@ -536,10 +487,14 @@ const XMLCh* DOMElementImpl::getBaseURI() const
             const XMLCh* uri =  attrNode->getNodeValue();
             if (uri && *uri) {// attribute value is always empty string
                 try {
-                    XMLUri temp(baseURI);
-                    XMLUri temp2(&temp, uri);
+                    XMLUri temp(baseURI, ((DOMDocumentImpl *)this->getOwnerDocument())->getMemoryManager());
+                    XMLUri temp2(&temp, uri, ((DOMDocumentImpl *)this->getOwnerDocument())->getMemoryManager());
                     uri = ((DOMDocumentImpl *)this->getOwnerDocument())->cloneString(temp2.getUriText());
                 }
+                catch(const OutOfMemoryException&)
+                {
+                    throw;
+                }
                 catch (...){
                     // REVISIT: what should happen in this case?
                     return 0;
@@ -556,39 +511,39 @@ const XMLCh* DOMElementImpl::getBaseURI() const
 //
 //   Functions inherited from Node
 //
-           DOMNode*         DOMElementImpl::appendChild(DOMNode *newChild)          {return fParent.appendChild (newChild); };
-           DOMNodeList*     DOMElementImpl::getChildNodes() const                   {return fParent.getChildNodes (); };
-           DOMNode*         DOMElementImpl::getFirstChild() const                   {return fParent.getFirstChild (); };
-           DOMNode*         DOMElementImpl::getLastChild() const                    {return fParent.getLastChild (); };
-     const XMLCh*           DOMElementImpl::getLocalName() const                    {return fNode.getLocalName (); };
-     const XMLCh*           DOMElementImpl::getNamespaceURI() const                 {return fNode.getNamespaceURI (); };
-           DOMNode*         DOMElementImpl::getNextSibling() const                  {return fChild.getNextSibling (); };
-     const XMLCh*           DOMElementImpl::getNodeValue() const                    {return fNode.getNodeValue (); };
-           DOMDocument*     DOMElementImpl::getOwnerDocument() const                {return fParent.fOwnerDocument; };
-     const XMLCh*           DOMElementImpl::getPrefix() const                       {return fNode.getPrefix (); };
-           DOMNode*         DOMElementImpl::getParentNode() const                   {return fChild.getParentNode (this); };
-           DOMNode*         DOMElementImpl::getPreviousSibling() const              {return fChild.getPreviousSibling (this); };
-           bool             DOMElementImpl::hasChildNodes() const                   {return fParent.hasChildNodes (); };
+           DOMNode*         DOMElementImpl::appendChild(DOMNode *newChild)          {return fParent.appendChild (newChild); }
+           DOMNodeList*     DOMElementImpl::getChildNodes() const                   {return fParent.getChildNodes (); }
+           DOMNode*         DOMElementImpl::getFirstChild() const                   {return fParent.getFirstChild (); }
+           DOMNode*         DOMElementImpl::getLastChild() const                    {return fParent.getLastChild (); }
+     const XMLCh*           DOMElementImpl::getLocalName() const                    {return fNode.getLocalName (); }
+     const XMLCh*           DOMElementImpl::getNamespaceURI() const                 {return fNode.getNamespaceURI (); }
+           DOMNode*         DOMElementImpl::getNextSibling() const                  {return fChild.getNextSibling (); }
+     const XMLCh*           DOMElementImpl::getNodeValue() const                    {return fNode.getNodeValue (); }
+           DOMDocument*     DOMElementImpl::getOwnerDocument() const                {return fParent.fOwnerDocument; }
+     const XMLCh*           DOMElementImpl::getPrefix() const                       {return fNode.getPrefix (); }
+           DOMNode*         DOMElementImpl::getParentNode() const                   {return fChild.getParentNode (this); }
+           DOMNode*         DOMElementImpl::getPreviousSibling() const              {return fChild.getPreviousSibling (this); }
+           bool             DOMElementImpl::hasChildNodes() const                   {return fParent.hasChildNodes (); }
            DOMNode*         DOMElementImpl::insertBefore(DOMNode *newChild, DOMNode *refChild)
-                                                                                    {return fParent.insertBefore (newChild, refChild); };
-           void             DOMElementImpl::normalize()                             {fParent.normalize (); };
-           DOMNode*         DOMElementImpl::removeChild(DOMNode *oldChild)          {return fParent.removeChild (oldChild); };
+                                                                                    {return fParent.insertBefore (newChild, refChild); }
+           void             DOMElementImpl::normalize()                             {fParent.normalize (); }
+           DOMNode*         DOMElementImpl::removeChild(DOMNode *oldChild)          {return fParent.removeChild (oldChild); }
            DOMNode*         DOMElementImpl::replaceChild(DOMNode *newChild, DOMNode *oldChild)
-                                                                                    {return fParent.replaceChild (newChild, oldChild); };
+                                                                                    {return fParent.replaceChild (newChild, oldChild); }
            bool             DOMElementImpl::isSupported(const XMLCh *feature, const XMLCh *version) const
-                                                                                    {return fNode.isSupported (feature, version); };
-           void             DOMElementImpl::setPrefix(const XMLCh  *prefix)         {fNode.setPrefix(prefix); };
-           bool             DOMElementImpl::isSameNode(const DOMNode* other) const  {return fNode.isSameNode(other); };
+                                                                                    {return fNode.isSupported (feature, version); }
+           void             DOMElementImpl::setPrefix(const XMLCh  *prefix)         {fNode.setPrefix(prefix); }
+           bool             DOMElementImpl::isSameNode(const DOMNode* other) const  {return fNode.isSameNode(other); }
            void*            DOMElementImpl::setUserData(const XMLCh* key, void* data, DOMUserDataHandler* handler)
-                                                                                    {return fNode.setUserData(key, data, handler); };
-           void*            DOMElementImpl::getUserData(const XMLCh* key) const     {return fNode.getUserData(key); };
-           short            DOMElementImpl::compareTreePosition(const DOMNode* other) const {return fNode.compareTreePosition(other); };
-           const XMLCh*     DOMElementImpl::getTextContent() const                  {return fNode.getTextContent(); };
-           void             DOMElementImpl::setTextContent(const XMLCh* textContent){fNode.setTextContent(textContent); };
-           const XMLCh*     DOMElementImpl::lookupNamespacePrefix(const XMLCh* namespaceURI, bool useDefault) const  {return fNode.lookupNamespacePrefix(namespaceURI, useDefault); };
-           bool             DOMElementImpl::isDefaultNamespace(const XMLCh* namespaceURI) const {return fNode.isDefaultNamespace(namespaceURI); };
-           const XMLCh*     DOMElementImpl::lookupNamespaceURI(const XMLCh* prefix) const  {return fNode.lookupNamespaceURI(prefix); };
-           DOMNode*         DOMElementImpl::getInterface(const XMLCh* feature)      {return fNode.getInterface(feature); };
+                                                                                    {return fNode.setUserData(key, data, handler); }
+           void*            DOMElementImpl::getUserData(const XMLCh* key) const     {return fNode.getUserData(key); }
+           short            DOMElementImpl::compareTreePosition(const DOMNode* other) const {return fNode.compareTreePosition(other); }
+           const XMLCh*     DOMElementImpl::getTextContent() const                  {return fNode.getTextContent(); }
+           void             DOMElementImpl::setTextContent(const XMLCh* textContent){fNode.setTextContent(textContent); }
+           const XMLCh*     DOMElementImpl::lookupNamespacePrefix(const XMLCh* namespaceURI, bool useDefault) const  {return fNode.lookupNamespacePrefix(namespaceURI, useDefault); }
+           bool             DOMElementImpl::isDefaultNamespace(const XMLCh* namespaceURI) const {return fNode.isDefaultNamespace(namespaceURI); }
+           const XMLCh*     DOMElementImpl::lookupNamespaceURI(const XMLCh* prefix) const  {return fNode.lookupNamespaceURI(prefix); }
+           DOMNode*         DOMElementImpl::getInterface(const XMLCh* feature)      {return fNode.getInterface(feature); }
 
 
 
@@ -635,7 +590,7 @@ bool DOMElementImpl::isEqualNode(const DOMNode* arg) const
     }
 
     return fParent.isEqualNode(arg);
-};
+}
 
 DOMNode* DOMElementImpl::rename(const XMLCh* namespaceURI, const XMLCh* name)
 {
@@ -687,15 +642,7 @@ DOMNode* DOMElementImpl::rename(const XMLCh* namespaceURI, const XMLCh* name)
 
 const DOMTypeInfo *DOMElementImpl::getTypeInfo() const
 {
-    if(!fSchemaType) 
-        ((DOMElementImpl *)(this))->fSchemaType = new (getOwnerDocument()) DOMTypeInfoImpl(0, 0, (DOMDocumentImpl *)getOwnerDocument());
-    return fSchemaType;
-}
-
-
-void DOMElementImpl::setTypeInfo(const XMLCh* typeName, const XMLCh* typeURI) 
-{
-    fSchemaType = new (getOwnerDocument()) DOMTypeInfoImpl(typeName, typeURI, (DOMDocumentImpl *)getOwnerDocument());
+    return &DOMTypeInfoImpl::g_DtdValidatedElement;
 }
 
 XERCES_CPP_NAMESPACE_END
