@@ -1,61 +1,21 @@
 /*
- * The Apache Software License, Version 1.1
- *
- * Copyright (c) 2001-2002 The Apache Software Foundation.  All rights
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xerces" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written
- *    permission, please contact apache\@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation, and was
- * originally based on software copyright (c) 2001, International
- * Business Machines, Inc., http://www.ibm.com .  For more information
- * on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Copyright 2001-2002,2004 The Apache Software Foundation.
+ * 
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 /*
- * $Id: DOMNodeIDMap.cpp,v 1.4 2002/11/04 15:07:34 tng Exp $
+ * $Id: DOMNodeIDMap.cpp,v 1.7 2004/09/08 13:55:51 peiyongz Exp $
  */
 
 #include "DOMAttrImpl.hpp"
@@ -75,9 +35,9 @@ static const float gMaxFill = 0.8f;   // The maximum fraction of the total
                                     // table entries to consume before exanding.
 
 DOMNodeIDMap::DOMNodeIDMap(int initialSize, DOMDocument *doc)
+: fNumEntries(0)
+, fDoc(doc)
 {
-    fDoc = doc;
-
     for (fSizeIndex = 0; gPrimes[fSizeIndex] < initialSize; fSizeIndex++)
     {
         if (gPrimes[fSizeIndex] == 0)
@@ -85,12 +45,11 @@ DOMNodeIDMap::DOMNodeIDMap(int initialSize, DOMDocument *doc)
             // We need a bigger size than the largest available one.
             //   Big trouble.
             fSizeIndex--;
-            ThrowXML(RuntimeException, XMLExcepts::NodeIDMap_GrowErr);
+            ThrowXMLwithMemMgr(RuntimeException, XMLExcepts::NodeIDMap_GrowErr, ((DOMDocumentImpl *)fDoc)->getMemoryManager());
         }
     }
 
-    fSize = gPrimes[fSizeIndex];
-    fNumEntries = 0;
+    fSize = gPrimes[fSizeIndex];    
     fMaxEntries = (XMLSize_t)(float(fSize) * gMaxFill);
 
     //fTable = new (fDoc) DOMAttr*[fSize];
@@ -98,14 +57,14 @@ DOMNodeIDMap::DOMNodeIDMap(int initialSize, DOMDocument *doc)
     XMLSize_t i;
     for (i=0; i<fSize; i++)
         fTable[i] = 0;
-};
+}
 
 
 DOMNodeIDMap::~DOMNodeIDMap()
 {
     // don't delete - the document owns the storage.
     fTable = 0;
-};
+}
 
 
 
@@ -126,7 +85,7 @@ void DOMNodeIDMap::add(DOMAttr *attr)
 	//      An initial hash of zero would cause the rehash to fail.
 	//
 	const XMLCh *id=attr->getValue();
-    XMLSize_t initalHash = XMLString::hash(id, fSize-1);
+    XMLSize_t initalHash = XMLString::hash(id, fSize-1, ((DOMDocumentImpl *)fDoc)->getMemoryManager());
 	initalHash++;
 	XMLSize_t currentHash = initalHash;
 
@@ -152,7 +111,7 @@ void DOMNodeIDMap::add(DOMAttr *attr)
     //
     fTable[currentHash] = attr;
 
-};
+}
 
 
 void DOMNodeIDMap::remove(DOMAttr *attr)
@@ -163,7 +122,7 @@ void DOMNodeIDMap::remove(DOMAttr *attr)
 	//      An initial hash of zero would cause the rehash to fail.
 	//
 	const XMLCh *id=attr->getValue();
-    XMLSize_t initalHash = XMLString::hash(id, fSize-1);
+    XMLSize_t initalHash = XMLString::hash(id, fSize-1, ((DOMDocumentImpl *)fDoc)->getMemoryManager());
 	initalHash++;
 	XMLSize_t currentHash = initalHash;
 
@@ -194,7 +153,7 @@ void DOMNodeIDMap::remove(DOMAttr *attr)
             currentHash = currentHash % fSize;
     }
 
-};
+}
 
 
 DOMAttr *DOMNodeIDMap::find(const XMLCh *id)
@@ -202,7 +161,7 @@ DOMAttr *DOMNodeIDMap::find(const XMLCh *id)
     //
     //  Get the hashcode for the supplied string.
     //
-	XMLSize_t initalHash = XMLString::hash(id, fSize-1);
+	XMLSize_t initalHash = XMLString::hash(id, fSize-1, ((DOMDocumentImpl *)fDoc)->getMemoryManager());
 	initalHash++;
 	XMLSize_t currentHash = initalHash;
 
@@ -227,7 +186,7 @@ DOMAttr *DOMNodeIDMap::find(const XMLCh *id)
             currentHash = currentHash % fSize;
     }
     return 0;  // Never gets here, but keeps some compilers happy.
-};
+}
 
 
 //
@@ -253,7 +212,7 @@ void DOMNodeIDMap::growTable()
         // We need to grow bigger than the largest available size.
         //   Big trouble.
         fSizeIndex--;
-        ThrowXML(RuntimeException, XMLExcepts::NodeIDMap_GrowErr);
+        ThrowXMLwithMemMgr(RuntimeException, XMLExcepts::NodeIDMap_GrowErr, ((DOMDocumentImpl *)fDoc)->getMemoryManager());
     }
 
     //
@@ -279,7 +238,7 @@ void DOMNodeIDMap::growTable()
     // delete [] oldTable;   (The document owns the storage.  The old table will just
     //                        need to leak until until the document is discarded.)
 
-};
+}
 
 
 XERCES_CPP_NAMESPACE_END
