@@ -1,61 +1,39 @@
 /*
- * The Apache Software License, Version 1.1
- *
- * Copyright (c) 1999-2000 The Apache Software Foundation.  All rights
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xerces" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written
- *    permission, please contact apache\@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation, and was
- * originally based on software copyright (c) 1999, International
- * Business Machines, Inc., http://www.ibm.com .  For more information
- * on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Copyright 1999-2000,2004 The Apache Software Foundation.
+ * 
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 /*
  * $Log: MemBufInputSource.hpp,v $
+ * Revision 1.9  2004/09/28 02:14:13  cargilld
+ * Add support for validating annotations.
+ *
+ * Revision 1.8  2004/09/08 13:55:57  peiyongz
+ * Apache License Version 2.0
+ *
+ * Revision 1.7  2004/01/29 11:46:29  cargilld
+ * Code cleanup changes to get rid of various compiler diagnostic messages.
+ *
+ * Revision 1.6  2003/12/01 23:23:25  neilg
+ * fix for bug 25118; thanks to Jeroen Witmond
+ *
+ * Revision 1.5  2003/07/30 14:48:38  peiyongz
+ * Allow default value
+ *
+ * Revision 1.4  2003/05/16 21:36:55  knoaman
+ * Memory manager implementation: Modify constructors to pass in the memory manager.
+ *
  * Revision 1.3  2002/11/04 15:00:21  tng
  * C++ Namespace Support.
  *
@@ -159,6 +137,8 @@ public :
       * @param  adoptBuffer     Indicates whether this object should adopt
       *                         the buffer (i.e. make a copy of it) or just
       *                         use it in place.
+      * @param  manager         Pointer to the memory manager to be used to
+      *                         allocate objects.
       */
     MemBufInputSource
     (
@@ -166,6 +146,7 @@ public :
         , const unsigned int    byteCount
         , const XMLCh* const    bufId
         , const bool            adoptBuffer = false
+        , MemoryManager* const  manager = XMLPlatformUtils::fgMemoryManager
     );
 
     /**
@@ -178,6 +159,7 @@ public :
         , const unsigned int    byteCount
         , const char* const     bufId
         , const bool            adoptBuffer = false
+        , MemoryManager* const  manager = XMLPlatformUtils::fgMemoryManager
     );
     //@}
 
@@ -238,10 +220,27 @@ public :
       */
     void setCopyBufToStream(const bool newState);
 
+    /**
+      * This methods allows the MemBufInputSource to be used for more than
+      * one input source, instead of destructing/constructing another
+      * MemBufInputSource.
+      *
+      * @param  srcDocBytes     The actual data buffer to be parsed from.
+      * @param  byteCount       The count of bytes (not characters, bytes!)
+      *                         in the buffer.     
+      */
+    void resetMemBufInputSource(const   XMLByte* const  srcDocBytes
+                                , const unsigned int    byteCount);
     //@}
 
 
 private :
+    // -----------------------------------------------------------------------
+    //  Unimplemented constructors and operators
+    // -----------------------------------------------------------------------
+    MemBufInputSource(const MemBufInputSource&);
+    MemBufInputSource& operator=(const MemBufInputSource&);
+
     // -----------------------------------------------------------------------
     //  Private data members
     //
