@@ -1,61 +1,33 @@
 /*
- * The Apache Software License, Version 1.1
- *
- * Copyright (c) 1999-2000 The Apache Software Foundation.  All rights
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xerces" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written
- *    permission, please contact apache\@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation, and was
- * originally based on software copyright (c) 1999, International
- * Business Machines, Inc., http://www.ibm.com .  For more information
- * on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Copyright 1999-2000,2004 The Apache Software Foundation.
+ * 
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 /*
  * $Log: URLInputSource.hpp,v $
+ * Revision 1.6  2004/09/08 13:55:58  peiyongz
+ * Apache License Version 2.0
+ *
+ * Revision 1.5  2004/01/29 11:46:29  cargilld
+ * Code cleanup changes to get rid of various compiler diagnostic messages.
+ *
+ * Revision 1.4  2003/12/01 23:23:25  neilg
+ * fix for bug 25118; thanks to Jeroen Witmond
+ *
+ * Revision 1.3  2003/05/16 21:36:55  knoaman
+ * Memory manager implementation: Modify constructors to pass in the memory manager.
+ *
  * Revision 1.2  2002/11/04 15:00:21  tng
  * C++ Namespace Support.
  *
@@ -143,8 +115,14 @@ public :
       *
       * @param  urlId   The URL which holds the system id of the entity
       *                 to parse.
+      * @param  manager Pointer to the memory manager to be used to
+      *                 allocate objects.
       */
-    URLInputSource(const XMLURL& urlId);
+    URLInputSource
+    (
+        const XMLURL& urlId
+        , MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager
+    );
 
 
     /**
@@ -158,11 +136,14 @@ public :
      *
      *  @param  systemId    The possibly relative system id URL. If its relative
      *                      its based on baseId, else its taken as is.
+     *  @param  manager     Pointer to the memory manager to be used to
+     *                      allocate objects.
      */
     URLInputSource
     (
         const   XMLCh* const    baseId
         , const XMLCh* const    systemId
+        , MemoryManager* const  manager = XMLPlatformUtils::fgMemoryManager
     );
 
     /**
@@ -177,12 +158,16 @@ public :
      *
      *  @param  publicId    The optional public id to set. This is just passed
      *                      on to the parent class for storage.
+     *
+     * @param  manager      Pointer to the memory manager to be used to
+     *                      allocate objects.
      */
     URLInputSource
     (
         const   XMLCh* const    baseId
         , const XMLCh* const    systemId
         , const XMLCh* const    publicId
+        , MemoryManager* const  manager = XMLPlatformUtils::fgMemoryManager
     );
 
 
@@ -196,11 +181,15 @@ public :
      *
      *  @param  systemId    The possibly relative system id URL. If its relative
      *                      its based on baseId, else its taken as is.
+     *
+     *  @param  manager     Pointer to the memory manager to be used to
+     *                      allocate objects.
      */
     URLInputSource
     (
         const   XMLCh* const    baseId
         , const char* const     systemId
+        , MemoryManager* const  manager = XMLPlatformUtils::fgMemoryManager
     );
 
     /**
@@ -217,12 +206,16 @@ public :
      *  @param  publicId    The optional public id to set. This is just passed
      *                      on to the parent class for storage.
      *                      on to the parent class for storage.
+     *
+     *  @param  manager     Pointer to the memory manager to be used to
+     *                      allocate objects.
      */
     URLInputSource
     (
         const   XMLCh* const    baseId
         , const char* const     systemId
         , const char* const     publicId
+        , MemoryManager* const  manager = XMLPlatformUtils::fgMemoryManager
     );
 
     //@}
@@ -270,6 +263,12 @@ public :
 
 
 private :
+    // -----------------------------------------------------------------------
+    //  Unimplemented constructors and operators
+    // -----------------------------------------------------------------------    
+    URLInputSource(const URLInputSource&);
+    URLInputSource& operator=(const URLInputSource&);
+
     // -----------------------------------------------------------------------
     //  Private data members
     //
