@@ -1,61 +1,36 @@
 /*
- * The Apache Software License, Version 1.1
- *
- * Copyright (c) 1999-2000 The Apache Software Foundation.  All rights
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xerces" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written
- *    permission, please contact apache\@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation, and was
- * originally based on software copyright (c) 1999, International
- * Business Machines, Inc., http://www.ibm.com .  For more information
- * on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Copyright 1999-2004 The Apache Software Foundation.
+ * 
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 /*
  * $Log: XMLBuffer.hpp,v $
+ * Revision 1.8  2004/09/29 18:59:18  peiyongz
+ * [jira1207] --patch from Dan Rosen
+ *
+ * Revision 1.7  2004/09/08 13:55:58  peiyongz
+ * Apache License Version 2.0
+ *
+ * Revision 1.6  2004/01/29 11:46:29  cargilld
+ * Code cleanup changes to get rid of various compiler diagnostic messages.
+ *
+ * Revision 1.5  2003/05/16 21:36:55  knoaman
+ * Memory manager implementation: Modify constructors to pass in the memory manager.
+ *
+ * Revision 1.4  2003/05/15 18:26:07  knoaman
+ * Partial implementation of the configurable memory manager.
+ *
  * Revision 1.3  2002/11/04 15:00:21  tng
  * C++ Namespace Support.
  *
@@ -94,10 +69,14 @@
 #if !defined(XMLBUFFER_HPP)
 #define XMLBUFFER_HPP
 
-#include <xercesc/util/XercesDefs.hpp>
+#include <xercesc/util/XMemory.hpp>
+#include <xercesc/util/PlatformUtils.hpp>
+#include <xercesc/framework/MemoryManager.hpp>
 
 XERCES_CPP_NAMESPACE_BEGIN
 
+class XMLBufferFullHandler;
+
 /**
  *  XMLBuffer is a lightweight, expandable Unicode text buffer. Since XML is
  *  inherently theoretically unbounded in terms of the sizes of things, we
@@ -108,7 +87,7 @@ XERCES_CPP_NAMESPACE_BEGIN
  *  The buffer is not nul terminated until some asks to see the raw buffer
  *  contents. This also avoids overhead during append operations.
  */
-class XMLPARSER_EXPORT XMLBuffer
+ class XMLPARSER_EXPORT XMLBuffer : public XMemory
 {
 public :
     // -----------------------------------------------------------------------
@@ -117,15 +96,19 @@ public :
 
     /** @name Constructor */
     //@{
-    XMLBuffer(int capacity = 1023) :
+    XMLBuffer(const unsigned int capacity = 1023
+              , MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager) :
 
-        fBuffer(0)
+        fUsed(false)
         , fIndex(0)
         , fCapacity(capacity)
-        , fUsed(false)
+        , fMemoryManager(manager)
+        , fFullHandler(0)
+        , fFullSize(0)
+        , fBuffer(0)
     {
         // Buffer is one larger than capacity, to allow for zero term
-        fBuffer = new XMLCh[fCapacity+1];
+        fBuffer = (XMLCh*) manager->allocate((capacity+1) * sizeof(XMLCh)); //new XMLCh[fCapacity+1];
 
         // Keep it null terminated
         fBuffer[0] = XMLCh(0);
@@ -136,27 +119,37 @@ public :
     //@{
     ~XMLBuffer()
     {
-        delete [] fBuffer;
+        fMemoryManager->deallocate(fBuffer); //delete [] fBuffer;
     }
     //@}
 
+    // -----------------------------------------------------------------------
+    //  Buffer Full Handler Management
+    // -----------------------------------------------------------------------
+    void setFullHandler(XMLBufferFullHandler* handler, const unsigned int fullSize)
+    {
+        fFullHandler = handler;
+        fFullSize = fullSize;
+    }
+
     // -----------------------------------------------------------------------
     //  Buffer Management
     // -----------------------------------------------------------------------
     void append(const XMLCh toAppend)
     {
-        if (fIndex == fCapacity)
-            expand();
-
         // Put in char and bump the index
+        if (fIndex == fCapacity)
+            insureCapacity(1);
         fBuffer[fIndex++] = toAppend;
     }
 
-    void append
-    (
-        const   XMLCh* const    chars
-        , const unsigned int    count = 0
-    );
+    void append (const XMLCh* const chars, const unsigned int count = 0);
+
+    void set (const XMLCh* const chars, const unsigned int count = 0)
+    {
+        fIndex = 0;
+        append(chars, count);
+    }
 
     const XMLCh* getRawBuffer() const
     {
@@ -176,13 +169,6 @@ public :
         fBuffer[0] = 0;
     }
 
-    void set
-    (
-        const   XMLCh* const    chars
-        , const unsigned int    count = 0
-    );
-
-
     // -----------------------------------------------------------------------
     //  Getters
     // -----------------------------------------------------------------------
@@ -201,7 +187,6 @@ public :
         return (fIndex == 0);
     }
 
-
     // -----------------------------------------------------------------------
     //  Setters
     // -----------------------------------------------------------------------
@@ -210,18 +195,21 @@ public :
         fUsed = newValue;
     }
 
-
 private :
+    // -----------------------------------------------------------------------
+    //  Unimplemented constructors and operators
+    // -----------------------------------------------------------------------
+    XMLBuffer(const XMLBuffer&);
+    XMLBuffer& operator=(const XMLBuffer&);
+
     // -----------------------------------------------------------------------
     //  Declare our friends
     // -----------------------------------------------------------------------
     friend class XMLBufBid;
 
-
     // -----------------------------------------------------------------------
     //  Private helpers
     // -----------------------------------------------------------------------
-    void expand();
     void insureCapacity(const unsigned int extraNeeded);
 
 
@@ -242,11 +230,41 @@ private :
     //
     //  fUsed
     //      Indicates whether this buffer is in use or not.
+    //
+    //  fFullHandler, fFullSize
+    //      If fFullHandler is non-null, the buffer has a maximum size
+    //      indicated by fFullSize. If writing to the buffer would exceed the
+    //      buffer's maximum size, fFullHandler's bufferFull callback is
+    //      invoked, to empty the buffer.
     // -----------------------------------------------------------------------
-    XMLCh*          fBuffer;
+    bool            fUsed;
     unsigned int    fIndex;
     unsigned int    fCapacity;
-    bool            fUsed;
+    MemoryManager* const        fMemoryManager;
+    XMLBufferFullHandler*       fFullHandler;
+    unsigned int                fFullSize;
+    XMLCh*          fBuffer;
+};
+
+/**
+ *  XMLBufferFullHandler is a callback interface for clients of
+ *  XMLBuffers that impose a size restriction (e.g. XMLScanner).
+ *  Note that this is intended solely as a mix-in for internal
+ *  use, and therefore does not derive from XMemory (to avoid
+ *  the ambiguous base class problem).
+ */
+class XMLPARSER_EXPORT XMLBufferFullHandler
+{
+public :
+
+    /**
+     * Callback method, intended to allow clients of an XMLBuffer which has
+     * become full to empty it appropriately.
+     * @return true if the handler was able to empty the buffer (either
+     * partially or completely), otherwise false to indicate an error.
+     */
+    virtual bool bufferFull(XMLBuffer&) = 0;
+
 };
 
 XERCES_CPP_NAMESPACE_END
