@@ -1,61 +1,48 @@
 /*
- * The Apache Software License, Version 1.1
- *
- * Copyright (c) 1999-2001 The Apache Software Foundation.  All rights
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xerces" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written
- *    permission, please contact apache\@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation, and was
- * originally based on software copyright (c) 1999, International
- * Business Machines, Inc., http://www.ibm.com .  For more information
- * on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Copyright 1999-2001,2004 The Apache Software Foundation.
+ * 
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 /*
  * $Log: XMLReader.hpp,v $
+ * Revision 1.20  2004/09/29 00:24:01  knoaman
+ * Performance: improve src offset calculation. Patch by Anthony O'Dowd.
+ *
+ * Revision 1.19  2004/09/08 13:56:14  peiyongz
+ * Apache License Version 2.0
+ *
+ * Revision 1.18  2004/06/15 15:51:21  peiyongz
+ * Compilation error on (AIX, Solaris) solved
+ *
+ * Revision 1.17  2004/06/14 15:18:53  peiyongz
+ * Consolidated End Of Line Handling
+ *
+ * Revision 1.16  2004/06/03 15:38:27  peiyongz
+ * XML1.1:  The characters #x85 and #x2028 cannot be reliably recognized
+ * and translated until an entity's encoding declaration (if present) has been
+ * read. Therefore, it is a fatal error to use them within the XML declaration or
+ * text declaration.
+ *
+ * Revision 1.15  2004/01/29 11:46:30  cargilld
+ * Code cleanup changes to get rid of various compiler diagnostic messages.
+ *
+ * Revision 1.14  2003/05/16 21:36:58  knoaman
+ * Memory manager implementation: Modify constructors to pass in the memory manager.
+ *
+ * Revision 1.13  2003/05/15 18:26:29  knoaman
+ * Partial implementation of the configurable memory manager.
+ *
  * Revision 1.12  2003/01/27 16:50:27  knoaman
  * some cleanup.
  *
@@ -165,17 +152,16 @@
 #if !defined(XMLREADER_HPP)
 #define XMLREADER_HPP
 
-#include <xercesc/util/XMLUniDefs.hpp>
 #include <xercesc/util/XMLChar.hpp>
 #include <xercesc/framework/XMLRecognizer.hpp>
 #include <xercesc/framework/XMLBuffer.hpp>
+#include <xercesc/util/TranscodingException.hpp>
 
 XERCES_CPP_NAMESPACE_BEGIN
 
 class InputSource;
 class BinInputStream;
 class ReaderMgr;
-class XMLBuffer;
 class XMLScanner;
 class XMLTranscoder;
 
@@ -194,7 +180,7 @@ class XMLTranscoder;
 //  This is NOT to be derived from.
 //
 // ---------------------------------------------------------------------------
-class XMLPARSER_EXPORT XMLReader
+class XMLPARSER_EXPORT XMLReader : public XMemory
 {
 public:
     // -----------------------------------------------------------------------
@@ -257,43 +243,46 @@ public:
     // -----------------------------------------------------------------------
     XMLReader
     (
-        const   XMLCh* const                pubId
-        , const XMLCh* const                sysId
-        ,       BinInputStream* const       streamToAdopt
-        , const RefFrom                     from
-        , const Types                       type
-        , const Sources                     source
-        , const bool                        throwAtEnd = false
-        , const bool                        calculateSrcOfs = true
-        , const XMLVersion                  xmlVersion = XMLV1_0
+        const   XMLCh* const          pubId
+        , const XMLCh* const          sysId
+        ,       BinInputStream* const streamToAdopt
+        , const RefFrom               from
+        , const Types                 type
+        , const Sources               source
+        , const bool                  throwAtEnd = false
+        , const bool                  calculateSrcOfs = true
+        , const XMLVersion            xmlVersion = XMLV1_0
+        ,       MemoryManager* const  manager = XMLPlatformUtils::fgMemoryManager
     );
 
     XMLReader
     (
-        const   XMLCh* const                pubId
-        , const XMLCh* const                sysId
-        ,       BinInputStream* const       streamToAdopt
-        , const XMLCh* const                encodingStr
-        , const RefFrom                     from
-        , const Types                       type
-        , const Sources                     source
-        , const bool                        throwAtEnd = false
-        , const bool                        calculateSrcOfs = true
-        , const XMLVersion                  xmlVersion = XMLV1_0
+        const   XMLCh* const          pubId
+        , const XMLCh* const          sysId
+        ,       BinInputStream* const streamToAdopt
+        , const XMLCh* const          encodingStr
+        , const RefFrom               from
+        , const Types                 type
+        , const Sources               source
+        , const bool                  throwAtEnd = false
+        , const bool                  calculateSrcOfs = true
+        , const XMLVersion            xmlVersion = XMLV1_0
+        ,       MemoryManager* const  manager = XMLPlatformUtils::fgMemoryManager
     );
 
     XMLReader
     (
-        const   XMLCh* const                pubId
-        , const XMLCh* const                sysId
-        ,       BinInputStream* const       streamToAdopt
-        , XMLRecognizer::Encodings          encodingEnum
-        , const RefFrom                     from
-        , const Types                       type
-        , const Sources                     source
-        , const bool                        throwAtEnd = false
-        , const bool                        calculateSrcOfs = true
-        , const XMLVersion                  xmlVersion = XMLV1_0
+        const   XMLCh* const          pubId
+        , const XMLCh* const          sysId
+        ,       BinInputStream* const streamToAdopt
+        , XMLRecognizer::Encodings    encodingEnum
+        , const RefFrom               from
+        , const Types                 type
+        , const Sources               source
+        , const bool                  throwAtEnd = false
+        , const bool                  calculateSrcOfs = true
+        , const XMLVersion            xmlVersion = XMLV1_0
+        ,       MemoryManager* const  manager = XMLPlatformUtils::fgMemoryManager
     );
 
     ~XMLReader();
@@ -317,7 +306,7 @@ public:
     bool getUpToCharOrWS(XMLBuffer& toFill, const XMLCh toCheck);
     bool peekNextChar(XMLCh& chGotten);
     bool skipIfQuote(XMLCh& chGotten);
-    bool skipSpaces(bool& skippedSomething);
+    bool skipSpaces(bool& skippedSomething, bool inDecl = false);
     bool skippedChar(const XMLCh toSkip);
     bool skippedSpace();
     bool skippedString(const XMLCh* const toSkip);
@@ -354,6 +343,12 @@ public:
 
 
 private:
+    // -----------------------------------------------------------------------
+    //  Unimplemented constructors and operators
+    // -----------------------------------------------------------------------
+    XMLReader(const XMLReader&);
+    XMLReader& operator=(const XMLReader&);
+
     // ---------------------------------------------------------------------------
     //  Class Constants
     //
@@ -405,6 +400,11 @@ private:
         , const unsigned int            maxChars
     );
 
+    inline void handleEOL
+    (
+              XMLCh&   curCh
+            , bool     inDecl = false
+    );
 
     // -----------------------------------------------------------------------
     //  Data members
@@ -427,6 +427,11 @@ private:
     //      to make the internalized char fCharBuf[x]. This only contains
     //      useful data if fSrcOfsSupported is true.
     //
+    //  fCharOfsBuf
+    //      This buffer is an array that contains the offset in the
+    //      fRawByteBuf buffer of each char in the fCharBuf buffer. It
+    //      only contains useful data if fSrcOfsSupported is true.
+    //
     //  fCurCol
     //  fCurLine
     //      The current line and column that we are in within this reader's
@@ -551,6 +556,7 @@ private:
     XMLCh                       fCharBuf[kCharBufSize];
     unsigned int                fCharsAvail;
     unsigned char               fCharSizeBuf[kCharBufSize];
+    unsigned int                fCharOfsBuf[kCharBufSize];
     XMLSSize_t                  fCurCol;
     XMLSSize_t                  fCurLine;
     XMLRecognizer::Encodings    fEncoding;
@@ -577,6 +583,7 @@ private:
     XMLByte*                    fgCharCharsTable;
     bool                        fNEL;
     XMLVersion                  fXMLVersion;
+    MemoryManager*              fMemoryManager;
 };
 
 
@@ -777,62 +784,8 @@ inline bool XMLReader::getNextCharIfNot(const XMLCh chNotToGet, XMLCh& chGotten)
     chGotten = fCharBuf[fCharIndex++];
 
     // Handle end of line normalization and line/col member maintenance.
-    if (chGotten == chCR)
-    {
-        //
-        //  Do the normalization. We return chLF regardless of which was
-        //  found. We also eat a chCR followed by an chLF.
-        //
-        //  We only do this if the content being spooled is not already
-        //  internalized.
-        //
-        if (fSource == Source_External)
-        {
-            //
-            //  See if we have another char left. If not, don't bother.
-            //  Else, see if its an chLF to eat. If it is, bump the
-            //  index again.
-            //
-            if (fCharIndex < fCharsAvail)
-            {
-                if (fCharBuf[fCharIndex] == chLF
-                    || ((fCharBuf[fCharIndex] == chNEL) && fNEL))
-                    fCharIndex++;
-            }
-             else
-            {
-                if (refreshCharBuffer())
-                {
-                    if (fCharBuf[fCharIndex] == chLF
-                        || ((fCharBuf[fCharIndex] == chNEL) && fNEL))
-                        fCharIndex++;
-                }
-            }
+    handleEOL(chGotten, false);
 
-            // And return just an chLF
-            chGotten = chLF;
-        }
-
-        // And handle the line/col stuff
-        fCurCol = 1;
-        fCurLine++;
-    }
-     else if (chGotten == chLF
-              || ((chGotten == chNEL || chGotten == chLineSeparator) && fNEL))
-    {
-        chGotten = chLF;
-        fCurLine++;
-        fCurCol = 1;
-    }
-     else if (chGotten)
-    {
-        //
-        //  Only do this is not a null char. Null chars are not part of the
-        //  real content. They are just marker characters inserted into
-        //  the stream.
-        //
-        fCurCol++;
-    }
     return true;
 }
 
@@ -859,53 +812,8 @@ inline bool XMLReader::getNextChar(XMLCh& chGotten)
     chGotten = fCharBuf[fCharIndex++];
 
     // Handle end of line normalization and line/col member maintenance.
-    if (chGotten == chCR)
-    {
-        //
-        //  Do the normalization. We return chLF regardless of which was
-        //  found. We also eat a chCR followed by an chLF.
-        //
-        //  We only do this if the content being spooled is not already
-        //  internalized.
-        //
-        if (fSource == Source_External)
-        {
-            //
-            //  See if we have another char left. If not, don't bother.
-            //  Else, see if its an chLF to eat. If it is, bump the
-            //  index again.
-            //
-            if ((fCharIndex < fCharsAvail) || refreshCharBuffer())
-            {
-                if (fCharBuf[fCharIndex] == chLF
-                    || ((fCharBuf[fCharIndex] == chNEL) && fNEL))
-                    fCharIndex++;
-            }
+    handleEOL(chGotten, false);
 
-            // And return just an chLF
-            chGotten = chLF;
-        }
-
-        // And handle the line/col stuff
-        fCurCol = 1;
-        fCurLine++;
-    }
-     else if (chGotten == chLF
-              || ((chGotten == chNEL || chGotten == chLineSeparator) && fNEL))
-    {
-        chGotten = chLF;
-        fCurLine++;
-        fCurCol = 1;
-    }
-     else if (chGotten)
-    {
-        //
-        //  Only do this is not a null char. Null chars are not part of the
-        //  real content. They are just marker characters inserted into
-        //  the stream.
-        //
-        fCurCol++;
-    }
     return true;
 }
 
@@ -943,6 +851,102 @@ inline bool XMLReader::peekNextChar(XMLCh& chGotten)
     return true;
 }
 
+/***
+ *
+ * XML1.1
+ *
+ * 2.11 End-of-Line Handling
+ *
+ *    XML parsed entities are often stored in computer files which, for editing 
+ *    convenience, are organized into lines. These lines are typically separated 
+ *    by some combination of the characters CARRIAGE RETURN (#xD) and LINE FEED (#xA).
+ *
+ *    To simplify the tasks of applications, the XML processor MUST behave as if 
+ *    it normalized all line breaks in external parsed entities (including the document 
+ *    entity) on input, before parsing, by translating all of the following to a single 
+ *    #xA character:
+ *
+ *  1. the two-character sequence #xD #xA
+ *  2. the two-character sequence #xD #x85
+ *  3. the single character #x85
+ *  4. the single character #x2028
+ *  5. any #xD character that is not immediately followed by #xA or #x85.
+ *
+ *
+ ***/
+inline void XMLReader::handleEOL(XMLCh& curCh, bool inDecl)
+{
+    // 1. the two-character sequence #xD #xA
+    // 2. the two-character sequence #xD #x85
+    // 5. any #xD character that is not immediately followed by #xA or #x85.
+    if (curCh == chCR)
+    {
+        fCurCol = 1;
+        fCurLine++;
+
+        //
+        //  If not already internalized, then convert it to an
+        //  LF and eat any following LF.
+        //
+        if (fSource == Source_External)
+        {
+            if ((fCharIndex < fCharsAvail) || refreshCharBuffer())
+            {
+                if ( fCharBuf[fCharIndex] == chLF              || 
+                    ((fCharBuf[fCharIndex] == chNEL) && fNEL)  )
+                {
+                    fCharIndex++;
+                }
+            }
+            curCh = chLF;
+        }
+    }
+    else if (curCh == chLF)                   
+    {
+        fCurCol = 1;
+        fCurLine++;
+    }
+    // 3. the single character #x85
+    // 4. the single character #x2028
+    else if (curCh == chNEL || curCh == chLineSeparator)
+    {
+        if (inDecl && fXMLVersion == XMLV1_1)
+        {
+
+        /***
+         * XML1.1
+         *
+         * 2.11 End-of-Line Handling
+         *  ...
+         *   The characters #x85 and #x2028 cannot be reliably recognized and translated 
+         *   until an entity's encoding declaration (if present) has been read. 
+         *   Therefore, it is a fatal error to use them within the XML declaration or 
+         *   text declaration. 
+         *
+         ***/
+            ThrowXMLwithMemMgr1
+                (
+                TranscodingException
+                , XMLExcepts::Reader_NelLsepinDecl
+                , fSystemId
+                , fMemoryManager
+                );
+        }
+
+        if (fNEL && fSource == Source_External)
+        {
+            fCurCol = 1;
+            fCurLine++;
+            curCh = chLF;
+        }
+    }
+    else
+    {
+        fCurCol++;
+    }
+
+    return;
+}
 
 XERCES_CPP_NAMESPACE_END
 
