@@ -1,66 +1,34 @@
 /*
- * The Apache Software License, Version 1.1
- *
- * Copyright (c) 2003 The Apache Software Foundation.  All rights
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xerces" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written
- *    permission, please contact apache\@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation, and was
- * originally based on software copyright (c) 1999, International
- * Business Machines, Inc., http://www.ibm.com .  For more information
- * on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Copyright 2003,2004 The Apache Software Foundation.
+ * 
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
-/*
 /*
  * $Log: XTemplateComparator.cpp,v $
+ * Revision 1.4  2004/09/08 13:56:14  peiyongz
+ * Apache License Version 2.0
+ *
+ * Revision 1.3  2003/12/19 23:02:43  cargilld
+ * Fix compiler messages on OS390.
+ *
+ * Revision 1.2  2003/12/17 00:18:34  cargilld
+ * Update to memory management so that the static memory manager (one used to call Initialize) is only for static data.
+ *
  * Revision 1.1  2003/10/29 16:14:15  peiyongz
  * XObjectComparator/XTemplateComparator
  *
- * $Id: XTemplateComparator.cpp,v 1.1 2003/10/29 16:14:15 peiyongz Exp $
+ * $Id: XTemplateComparator.cpp,v 1.4 2004/09/08 13:56:14 peiyongz Exp $
  *
  */
 
@@ -401,7 +369,7 @@ bool XTemplateComparator::isEquivalent(RefHashTableOf<KVStringPair>* const lValu
 {
     IS_EQUIVALENT(lValue, rValue);
 
-    RefHashTableOfEnumerator<KVStringPair> lEnum(lValue);
+    RefHashTableOfEnumerator<KVStringPair> lEnum(lValue, false, lValue->getMemoryManager());
     int lItemNumber = 0;        
     while (lEnum.hasMoreElements())
     {
@@ -409,7 +377,7 @@ bool XTemplateComparator::isEquivalent(RefHashTableOf<KVStringPair>* const lValu
         lItemNumber++;
     }
 
-    RefHashTableOfEnumerator<KVStringPair> rEnum(lValue);
+    RefHashTableOfEnumerator<KVStringPair> rEnum(rValue, false, rValue->getMemoryManager());
     int rItemNumber = 0;        
     while (rEnum.hasMoreElements())
     {
@@ -442,7 +410,7 @@ bool XTemplateComparator::isEquivalent(RefHashTableOf<XMLAttDef>* const lValue
 {
     IS_EQUIVALENT(lValue, rValue);
 
-    RefHashTableOfEnumerator<XMLAttDef> lEnum(lValue);
+    RefHashTableOfEnumerator<XMLAttDef> lEnum(lValue, false, lValue->getMemoryManager());
     int lItemNumber = 0;        
     while (lEnum.hasMoreElements())
     {
@@ -450,7 +418,7 @@ bool XTemplateComparator::isEquivalent(RefHashTableOf<XMLAttDef>* const lValue
         lItemNumber++;
     }
 
-    RefHashTableOfEnumerator<XMLAttDef> rEnum(lValue);
+    RefHashTableOfEnumerator<XMLAttDef> rEnum(rValue, false, rValue->getMemoryManager());
     int rItemNumber = 0;        
     while (rEnum.hasMoreElements())
     {
@@ -485,7 +453,7 @@ bool XTemplateComparator::isEquivalent(RefHashTableOf<DTDAttDef>* const lValue
 {
     IS_EQUIVALENT(lValue, rValue);
 
-    RefHashTableOfEnumerator<DTDAttDef> lEnum(lValue);
+    RefHashTableOfEnumerator<DTDAttDef> lEnum(lValue, false, lValue->getMemoryManager());
     int lItemNumber = 0;        
     while (lEnum.hasMoreElements())
     {
@@ -493,7 +461,7 @@ bool XTemplateComparator::isEquivalent(RefHashTableOf<DTDAttDef>* const lValue
         lItemNumber++;
     }
 
-    RefHashTableOfEnumerator<DTDAttDef> rEnum(lValue);
+    RefHashTableOfEnumerator<DTDAttDef> rEnum(rValue, false, rValue->getMemoryManager());
     int rItemNumber = 0;        
     while (rEnum.hasMoreElements())
     {
@@ -527,7 +495,7 @@ bool XTemplateComparator::isEquivalent(RefHashTableOf<ComplexTypeInfo>* const lV
 {
     IS_EQUIVALENT(lValue, rValue);
 
-    RefHashTableOfEnumerator<ComplexTypeInfo> lEnum(lValue);
+    RefHashTableOfEnumerator<ComplexTypeInfo> lEnum(lValue, false, lValue->getMemoryManager());
     int lItemNumber = 0;        
     while (lEnum.hasMoreElements())
     {
@@ -535,7 +503,7 @@ bool XTemplateComparator::isEquivalent(RefHashTableOf<ComplexTypeInfo>* const lV
         lItemNumber++;
     }
 
-    RefHashTableOfEnumerator<ComplexTypeInfo> rEnum(lValue);
+    RefHashTableOfEnumerator<ComplexTypeInfo> rEnum(rValue, false, rValue->getMemoryManager());
     int rItemNumber = 0;        
     while (rEnum.hasMoreElements())
     {
@@ -568,7 +536,7 @@ bool XTemplateComparator::isEquivalent(RefHashTableOf<XercesGroupInfo>* const lV
 {
     IS_EQUIVALENT(lValue, rValue);
 
-    RefHashTableOfEnumerator<XercesGroupInfo> lEnum(lValue);
+    RefHashTableOfEnumerator<XercesGroupInfo> lEnum(lValue, false, lValue->getMemoryManager());
     int lItemNumber = 0;        
     while (lEnum.hasMoreElements())
     {
@@ -576,7 +544,7 @@ bool XTemplateComparator::isEquivalent(RefHashTableOf<XercesGroupInfo>* const lV
         lItemNumber++;
     }
 
-    RefHashTableOfEnumerator<XercesGroupInfo> rEnum(lValue);
+    RefHashTableOfEnumerator<XercesGroupInfo> rEnum(rValue, false, rValue->getMemoryManager());
     int rItemNumber = 0;        
     while (rEnum.hasMoreElements())
     {
@@ -609,7 +577,7 @@ bool XTemplateComparator::isEquivalent(RefHashTableOf<XercesAttGroupInfo>* const
 {
     IS_EQUIVALENT(lValue, rValue);
 
-    RefHashTableOfEnumerator<XercesAttGroupInfo> lEnum(lValue);
+    RefHashTableOfEnumerator<XercesAttGroupInfo> lEnum(lValue, false, lValue->getMemoryManager());
     int lItemNumber = 0;        
     while (lEnum.hasMoreElements())
     {
@@ -617,7 +585,7 @@ bool XTemplateComparator::isEquivalent(RefHashTableOf<XercesAttGroupInfo>* const
         lItemNumber++;
     }
 
-    RefHashTableOfEnumerator<XercesAttGroupInfo> rEnum(lValue);
+    RefHashTableOfEnumerator<XercesAttGroupInfo> rEnum(rValue, false, rValue->getMemoryManager());
     int rItemNumber = 0;        
     while (rEnum.hasMoreElements())
     {
@@ -650,7 +618,7 @@ bool XTemplateComparator::isEquivalent(RefHashTableOf<XMLRefInfo>* const lValue
 {
     IS_EQUIVALENT(lValue, rValue);
 
-    RefHashTableOfEnumerator<XMLRefInfo> lEnum(lValue);
+    RefHashTableOfEnumerator<XMLRefInfo> lEnum(lValue, false, lValue->getMemoryManager());
     int lItemNumber = 0;        
     while (lEnum.hasMoreElements())
     {
@@ -658,7 +626,7 @@ bool XTemplateComparator::isEquivalent(RefHashTableOf<XMLRefInfo>* const lValue
         lItemNumber++;
     }
 
-    RefHashTableOfEnumerator<XMLRefInfo> rEnum(lValue);
+    RefHashTableOfEnumerator<XMLRefInfo> rEnum(rValue, false, rValue->getMemoryManager());
     int rItemNumber = 0;        
     while (rEnum.hasMoreElements())
     {
@@ -691,7 +659,7 @@ bool XTemplateComparator::isEquivalent(RefHashTableOf<DatatypeValidator>* const
 {
     IS_EQUIVALENT(lValue, rValue);
 
-    RefHashTableOfEnumerator<DatatypeValidator> lEnum(lValue);
+    RefHashTableOfEnumerator<DatatypeValidator> lEnum(lValue, false, lValue->getMemoryManager());
     int lItemNumber = 0;        
     while (lEnum.hasMoreElements())
     {
@@ -699,7 +667,7 @@ bool XTemplateComparator::isEquivalent(RefHashTableOf<DatatypeValidator>* const
         lItemNumber++;
     }
 
-    RefHashTableOfEnumerator<DatatypeValidator> rEnum(lValue);
+    RefHashTableOfEnumerator<DatatypeValidator> rEnum(rValue, false, rValue->getMemoryManager());
     int rItemNumber = 0;        
     while (rEnum.hasMoreElements())
     {
@@ -732,7 +700,7 @@ bool XTemplateComparator::isEquivalent(RefHashTableOf<Grammar>* const lValue
 {
     IS_EQUIVALENT(lValue, rValue);
 
-    RefHashTableOfEnumerator<Grammar> lEnum(lValue);
+    RefHashTableOfEnumerator<Grammar> lEnum(lValue, false, lValue->getMemoryManager());
     int lItemNumber = 0;        
     while (lEnum.hasMoreElements())
     {
@@ -740,7 +708,7 @@ bool XTemplateComparator::isEquivalent(RefHashTableOf<Grammar>* const lValue
         lItemNumber++;
     }
 
-    RefHashTableOfEnumerator<Grammar> rEnum(lValue);
+    RefHashTableOfEnumerator<Grammar> rEnum(rValue, false, rValue->getMemoryManager());
     int rItemNumber = 0;        
     while (rEnum.hasMoreElements())
     {
@@ -781,7 +749,7 @@ bool XTemplateComparator::isEquivalent(RefHash2KeysTableOf<SchemaAttDef>* const
 {
     IS_EQUIVALENT(lValue, rValue);
 
-    RefHash2KeysTableOfEnumerator<SchemaAttDef> lEnum(lValue);
+    RefHash2KeysTableOfEnumerator<SchemaAttDef> lEnum(lValue, false, lValue->getMemoryManager());
     int lItemNumber = 0;        
     while (lEnum.hasMoreElements())
     {
@@ -789,7 +757,7 @@ bool XTemplateComparator::isEquivalent(RefHash2KeysTableOf<SchemaAttDef>* const
         lItemNumber++;
     }
 
-    RefHash2KeysTableOfEnumerator<SchemaAttDef> rEnum(lValue);
+    RefHash2KeysTableOfEnumerator<SchemaAttDef> rEnum(rValue, false, rValue->getMemoryManager());
     int rItemNumber = 0;        
     while (rEnum.hasMoreElements())
     {
@@ -827,7 +795,7 @@ bool XTemplateComparator::isEquivalent(RefHash2KeysTableOf<ElemVector>* const lV
 
     IS_EQUIVALENT(lValue, rValue);
 
-    RefHash2KeysTableOfEnumerator<ElemVector> lEnum(lValue);
+    RefHash2KeysTableOfEnumerator<ElemVector> lEnum(lValue, false, lValue->getMemoryManager());
     int lItemNumber = 0;        
     while (lEnum.hasMoreElements())
     {
@@ -835,7 +803,7 @@ bool XTemplateComparator::isEquivalent(RefHash2KeysTableOf<ElemVector>* const lV
         lItemNumber++;
     }
 
-    RefHash2KeysTableOfEnumerator<ElemVector> rEnum(lValue);
+    RefHash2KeysTableOfEnumerator<ElemVector> rEnum(rValue, false, rValue->getMemoryManager());
     int rItemNumber = 0;        
     while (rEnum.hasMoreElements())
     {
@@ -880,8 +848,8 @@ bool XTemplateComparator::isEquivalent(RefHash3KeysIdPool<SchemaElementDecl>* co
 
     IS_EQUIVALENT(lValue, rValue)
 
-    RefHash3KeysIdPoolEnumerator<SchemaElementDecl> lEnum(lValue);
-    RefHash3KeysIdPoolEnumerator<SchemaElementDecl> rEnum(rValue);
+    RefHash3KeysIdPoolEnumerator<SchemaElementDecl> lEnum(lValue, false, lValue->getMemoryManager());
+    RefHash3KeysIdPoolEnumerator<SchemaElementDecl> rEnum(rValue, false, rValue->getMemoryManager());
 
     if (lEnum.size() != rEnum.size())
         return false;
@@ -914,8 +882,8 @@ bool XTemplateComparator::isEquivalent(NameIdPool<DTDElementDecl>* const lValue
 
     IS_EQUIVALENT(lValue, rValue)
 
-    NameIdPoolEnumerator<DTDElementDecl> lEnum(lValue);
-    NameIdPoolEnumerator<DTDElementDecl> rEnum(lValue);
+    NameIdPoolEnumerator<DTDElementDecl> lEnum(lValue, lValue->getMemoryManager());
+    NameIdPoolEnumerator<DTDElementDecl> rEnum(rValue, rValue->getMemoryManager());
 
     if (lEnum.size() != rEnum.size())
         return false;
@@ -940,8 +908,8 @@ bool XTemplateComparator::isEquivalent(NameIdPool<DTDEntityDecl>* const lValue
 
     IS_EQUIVALENT(lValue, rValue)
 
-    NameIdPoolEnumerator<DTDEntityDecl> lEnum(lValue);
-    NameIdPoolEnumerator<DTDEntityDecl> rEnum(lValue);
+    NameIdPoolEnumerator<DTDEntityDecl> lEnum(lValue, lValue->getMemoryManager());
+    NameIdPoolEnumerator<DTDEntityDecl> rEnum(rValue, rValue->getMemoryManager());
 
     if (lEnum.size() != rEnum.size())
         return false;
@@ -965,8 +933,8 @@ bool XTemplateComparator::isEquivalent(NameIdPool<XMLNotationDecl>* const lValue
 {
     IS_EQUIVALENT(lValue, rValue)
 
-    NameIdPoolEnumerator<XMLNotationDecl> lEnum(lValue);
-    NameIdPoolEnumerator<XMLNotationDecl> rEnum(lValue);
+    NameIdPoolEnumerator<XMLNotationDecl> lEnum(lValue, lValue->getMemoryManager());
+    NameIdPoolEnumerator<XMLNotationDecl> rEnum(rValue, rValue->getMemoryManager());
 
     if (lEnum.size() != rEnum.size())
         return false;
