@@ -1,61 +1,107 @@
 /*
- * The Apache Software License, Version 1.1
- *
- * Copyright (c) 1999-2001 The Apache Software Foundation.  All rights
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xerces" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written
- *    permission, please contact apache\@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation, and was
- * originally based on software copyright (c) 1999, International
- * Business Machines, Inc., http://www.ibm.com .  For more information
- * on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Copyright 1999-2004 The Apache Software Foundation.
+ * 
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 /*
  * $Log: SAXParser.cpp,v $
+ * Revision 1.36  2004/09/29 19:00:29  peiyongz
+ * [jira1207] --patch from Dan Rosen
+ *
+ * Revision 1.35  2004/09/28 02:14:14  cargilld
+ * Add support for validating annotations.
+ *
+ * Revision 1.34  2004/09/23 01:09:55  cargilld
+ * Add support for generating synthetic XSAnnotations.  When a schema component has non-schema attributes and no child attributes create a synthetic XSAnnotation (under feature control) so the non-schema attributes can be recovered under PSVI.
+ *
+ * Revision 1.33  2004/09/08 13:56:17  peiyongz
+ * Apache License Version 2.0
+ *
+ * Revision 1.32  2004/04/13 16:53:26  peiyongz
+ * get/setIdentityConstraintChecking
+ *
+ * Revision 1.31  2004/01/29 11:46:32  cargilld
+ * Code cleanup changes to get rid of various compiler diagnostic messages.
+ *
+ * Revision 1.30  2003/12/17 00:18:35  cargilld
+ * Update to memory management so that the static memory manager (one used to call Initialize) is only for static data.
+ *
+ * Revision 1.29  2003/11/21 22:38:50  neilg
+ * Enable grammar pools and grammar resolvers to manufacture
+ * XSModels.  This also cleans up handling in the
+ * parser classes by eliminating the need to tell
+ * the grammar pool that schema compoments need to be produced.
+ * Thanks to David Cargill.
+ *
+ * Revision 1.28  2003/11/06 15:30:07  neilg
+ * first part of PSVI/schema component model implementation, thanks to David Cargill.  This covers setting the PSVIHandler on parser objects, as well as implementing XSNotation, XSSimpleTypeDefinition, XSIDCDefinition, and most of XSWildcard, XSComplexTypeDefinition, XSElementDeclaration, XSAttributeDeclaration and XSAttributeUse.
+ *
+ * Revision 1.27  2003/10/30 21:37:31  knoaman
+ * Enhanced Entity Resolver Support. Thanks to David Cargill.
+ *
+ * Revision 1.26  2003/10/01 16:32:38  neilg
+ * improve handling of out of memory conditions, bug #23415.  Thanks to David Cargill.
+ *
+ * Revision 1.25  2003/09/16 18:30:54  neilg
+ * make Grammar pool be responsible for creating and owning URI string pools.  This is one more step towards having grammars be independent of the parsers involved in their creation
+ *
+ * Revision 1.24  2003/08/13 15:43:24  knoaman
+ * Use memory manager when creating SAX exceptions.
+ *
+ * Revision 1.23  2003/07/31 17:05:48  peiyongz
+ * using getGrammar(URI)
+ *
+ * Revision 1.22  2003/07/10 19:48:24  peiyongz
+ * Stateless Grammar: Initialize scanner with grammarResolver,
+ *
+ * Revision 1.21  2003/06/25 22:36:46  peiyongz
+ * to use new GrammarResolver::getGrammar()
+ *
+ * Revision 1.20  2003/06/20 18:55:54  peiyongz
+ * Stateless Grammar Pool :: Part I
+ *
+ * Revision 1.19  2003/05/18 14:02:05  knoaman
+ * Memory manager implementation: pass per instance manager.
+ *
+ * Revision 1.18  2003/05/16 21:36:59  knoaman
+ * Memory manager implementation: Modify constructors to pass in the memory manager.
+ *
+ * Revision 1.17  2003/05/15 18:26:50  knoaman
+ * Partial implementation of the configurable memory manager.
+ *
+ * Revision 1.16  2003/04/17 21:58:50  neilg
+ * Adding a new property,
+ * http://apache.org/xml/properties/security-manager, with
+ * appropriate getSecurityManager/setSecurityManager methods on DOM
+ * and SAX parsers.  Also adding a new SecurityManager class.
+ *
+ * The purpose of these modifications is to permit applications a
+ * means to have the parser reject documents whose processing would
+ * otherwise consume large amounts of system resources.  Malicious
+ * use of such documents could be used to launch a denial-of-service
+ * attack against a system running the parser.  Initially, the
+ * SecurityManager only knows about attacks that can result from
+ * exponential entity expansion; this is the only known attack that
+ * involves processing a single XML document.  Other, simlar attacks
+ * can be launched if arbitrary schemas may be parsed; there already
+ * exist means (via use of the EntityResolver interface) by which
+ * applications can deny processing of untrusted schemas.  In future,
+ * the SecurityManager will be expanded to take these other exploits
+ * into account.
+ *
+ * add security manager
+ * 
  * Revision 1.15  2003/02/04 19:27:43  knoaman
  * Performance: use global buffer to eliminate repetitive memory creation/deletion.
  *
@@ -212,6 +258,11 @@
 #include <xercesc/sax/EntityResolver.hpp>
 #include <xercesc/sax/SAXParseException.hpp>
 #include <xercesc/validators/common/GrammarResolver.hpp>
+#include <xercesc/framework/XMLGrammarPool.hpp>
+#include <xercesc/framework/XMLSchemaDescription.hpp>
+#include <xercesc/util/Janitor.hpp>
+#include <xercesc/util/OutOfMemoryException.hpp>
+#include <xercesc/util/XMLEntityResolver.hpp>
 #include <string.h>
 
 XERCES_CPP_NAMESPACE_BEGIN
@@ -220,7 +271,9 @@ XERCES_CPP_NAMESPACE_BEGIN
 // ---------------------------------------------------------------------------
 //  SAXParser: Constructors and Destructor
 // ---------------------------------------------------------------------------
-SAXParser::SAXParser(XMLValidator* const valToAdopt) :
+SAXParser::SAXParser( XMLValidator* const   valToAdopt
+                    , MemoryManager* const  manager
+                    , XMLGrammarPool* const gramPool):
 
     fParseInProgress(false)
     , fElemDepth(0)
@@ -229,17 +282,26 @@ SAXParser::SAXParser(XMLValidator* const valToAdopt) :
     , fDocHandler(0)
     , fDTDHandler(0)
     , fEntityResolver(0)
+    , fXMLEntityResolver(0)
     , fErrorHandler(0)
+    , fPSVIHandler(0)
     , fAdvDHList(0)
     , fScanner(0)
     , fGrammarResolver(0)
     , fURIStringPool(0)
     , fValidator(valToAdopt)
+    , fMemoryManager(manager)
+    , fGrammarPool(gramPool)
+    , fElemQNameBuf(1023, manager)
 {
     try
     {
         initialize();
     }
+    catch(const OutOfMemoryException&)
+    {
+        throw;
+    }
     catch(...)
     {
         cleanUp();
@@ -259,25 +321,28 @@ SAXParser::~SAXParser()
 void SAXParser::initialize()
 {
     // Create grammar resolver and string pool to pass to scanner
-    fGrammarResolver = new GrammarResolver();
-    fURIStringPool = new XMLStringPool();
+    fGrammarResolver = new (fMemoryManager) GrammarResolver(fGrammarPool, fMemoryManager);
+    fURIStringPool = fGrammarResolver->getStringPool();
 
     // Create our scanner and tell it what validator to use
-    fScanner = XMLScannerResolver::getDefaultScanner(fValidator);
-    fScanner->setGrammarResolver(fGrammarResolver);
+    fScanner = XMLScannerResolver::getDefaultScanner(fValidator, fGrammarResolver, fMemoryManager);
     fScanner->setURIStringPool(fURIStringPool);
 
     // Create the initial advanced handler list array and zero it out
-    fAdvDHList = new XMLDocumentHandler*[fAdvDHListSize];
+    fAdvDHList = (XMLDocumentHandler**) fMemoryManager->allocate
+    (
+        fAdvDHListSize * sizeof(XMLDocumentHandler*)
+    );//new XMLDocumentHandler*[fAdvDHListSize];
     memset(fAdvDHList, 0, sizeof(void*) * fAdvDHListSize);
 }
 
 void SAXParser::cleanUp()
 {
-    delete [] fAdvDHList;
+    fMemoryManager->deallocate(fAdvDHList);//delete [] fAdvDHList;
     delete fScanner;
     delete fGrammarResolver;
-    delete fURIStringPool;
+    // grammar pool must do this
+    //delete fURIStringPool;
 
     if (fValidator)
         delete fValidator;
@@ -294,7 +359,10 @@ void SAXParser::installAdvDocHandler(XMLDocumentHandler* const toInstall)
     {
         // Calc a new size and allocate the new temp buffer
         const unsigned int newSize = (unsigned int)(fAdvDHListSize * 1.5);
-        XMLDocumentHandler** newList = new XMLDocumentHandler*[newSize];
+        XMLDocumentHandler** newList = (XMLDocumentHandler**) fMemoryManager->allocate
+        (
+            newSize * sizeof(XMLDocumentHandler*)
+        );//new XMLDocumentHandler*[newSize];
 
         // Copy over the old data to the new list and zero out the rest
         memcpy(newList, fAdvDHList, sizeof(void*) * fAdvDHListSize);
@@ -306,7 +374,7 @@ void SAXParser::installAdvDocHandler(XMLDocumentHandler* const toInstall)
         );
 
         // And now clean up the old array and store the new stuff
-        delete [] fAdvDHList;
+        fMemoryManager->deallocate(fAdvDHList);//delete [] fAdvDHList;
         fAdvDHList = newList;
         fAdvDHListSize = newSize;
     }
@@ -387,6 +455,15 @@ bool SAXParser::getDoNamespaces() const
     return fScanner->getDoNamespaces();
 }
 
+bool SAXParser::getGenerateSyntheticAnnotations() const
+{
+    return fScanner->getGenerateSyntheticAnnotations();
+}
+
+bool SAXParser::getValidateAnnotations() const
+{
+    return fScanner->getValidateAnnotations();
+}
 
 bool SAXParser::getExitOnFirstFatalError() const
 {
@@ -421,6 +498,11 @@ bool SAXParser::getValidationSchemaFullChecking() const
     return fScanner->getValidationSchemaFullChecking();
 }
 
+bool SAXParser::getIdentityConstraintChecking() const
+{
+    return fScanner->getIdentityConstraintChecking();
+}
+
 int SAXParser::getErrorCount() const
 {
     return fScanner->getErrorCount();
@@ -436,6 +518,11 @@ XMLCh* SAXParser::getExternalNoNamespaceSchemaLocation() const
     return fScanner->getExternalNoNamespaceSchemaLocation();
 }
 
+SecurityManager* SAXParser::getSecurityManager() const
+{
+    return fScanner->getSecurityManager();
+}
+
 bool SAXParser::getLoadExternalDTD() const
 {
     return fScanner->getLoadExternalDTD();
@@ -489,6 +576,15 @@ void SAXParser::setDoNamespaces(const bool newState)
     fScanner->setDoNamespaces(newState);
 }
 
+void SAXParser::setGenerateSyntheticAnnotations(const bool newState)
+{
+    fScanner->setGenerateSyntheticAnnotations(newState);
+}
+
+void SAXParser::setValidateAnnotations(const bool newState)
+{
+    fScanner->setValidateAnnotations(newState);
+}
 
 void SAXParser::setExitOnFirstFatalError(const bool newState)
 {
@@ -522,6 +618,11 @@ void SAXParser::setValidationSchemaFullChecking(const bool schemaFullChecking)
     fScanner->setValidationSchemaFullChecking(schemaFullChecking);
 }
 
+void SAXParser::setIdentityConstraintChecking(const bool identityConstraintChecking)
+{
+    fScanner->setIdentityConstraintChecking(identityConstraintChecking);
+}
+
 void SAXParser::setExternalSchemaLocation(const XMLCh* const schemaLocation)
 {
     fScanner->setExternalSchemaLocation(schemaLocation);
@@ -540,6 +641,16 @@ void SAXParser::setExternalNoNamespaceSchemaLocation(const char* const noNamespa
     fScanner->setExternalNoNamespaceSchemaLocation(noNamespaceSchemaLocation);
 }
 
+void SAXParser::setSecurityManager(SecurityManager* const securityManager)
+{
+    // since this could impact various components, don't permit it to change
+    // during a parse
+    if (fParseInProgress)
+        ThrowXMLwithMemMgr(IOException, XMLExcepts::Gen_ParseInProgress, fMemoryManager);
+
+    fScanner->setSecurityManager(securityManager);
+}
+
 void SAXParser::setLoadExternalDTD(const bool newState)
 {
     fScanner->setLoadExternalDTD(newState);
@@ -571,18 +682,27 @@ void SAXParser::setStandardUriConformant(const bool newState)
 
 void SAXParser::useScanner(const XMLCh* const scannerName)
 {
-    XMLScanner* tempScanner = XMLScannerResolver::resolveScanner(scannerName, fValidator);
+    XMLScanner* tempScanner = XMLScannerResolver::resolveScanner
+    (
+        scannerName
+        , fValidator
+        , fGrammarResolver
+        , fMemoryManager
+    );
 
     if (tempScanner) {
 
         tempScanner->setParseSettings(fScanner);
-        tempScanner->setGrammarResolver(fGrammarResolver);
         tempScanner->setURIStringPool(fURIStringPool);
         delete fScanner;
         fScanner = tempScanner;
     }
 }
 
+void SAXParser::setInputBufferSize(const size_t bufferSize)
+{
+    fScanner->setInputBufferSize(bufferSize);
+}
 
 // ---------------------------------------------------------------------------
 //  SAXParser: Overrides of the SAX Parser interface
@@ -591,7 +711,7 @@ void SAXParser::parse(const InputSource& source)
 {
     // Avoid multiple entrance
     if (fParseInProgress)
-        ThrowXML(IOException, XMLExcepts::Gen_ParseInProgress);
+        ThrowXMLwithMemMgr(IOException, XMLExcepts::Gen_ParseInProgress, fMemoryManager);
 
     try
     {
@@ -599,7 +719,10 @@ void SAXParser::parse(const InputSource& source)
         fScanner->scanDocument(source);
         fParseInProgress = false;
     }
-
+    catch(const OutOfMemoryException&)
+    {
+        throw;
+    }
     catch (...)
     {
         fParseInProgress = false;
@@ -611,7 +734,7 @@ void SAXParser::parse(const XMLCh* const systemId)
 {
     // Avoid multiple entrance
     if (fParseInProgress)
-        ThrowXML(IOException, XMLExcepts::Gen_ParseInProgress);
+        ThrowXMLwithMemMgr(IOException, XMLExcepts::Gen_ParseInProgress, fMemoryManager);
 
     try
     {
@@ -619,7 +742,10 @@ void SAXParser::parse(const XMLCh* const systemId)
         fScanner->scanDocument(systemId);
         fParseInProgress = false;
     }
-
+    catch(const OutOfMemoryException&)
+    {
+        throw;
+    }
     catch (...)
     {
         fParseInProgress = false;
@@ -631,7 +757,7 @@ void SAXParser::parse(const char* const systemId)
 {
     // Avoid multiple entrance
     if (fParseInProgress)
-        ThrowXML(IOException, XMLExcepts::Gen_ParseInProgress);
+        ThrowXMLwithMemMgr(IOException, XMLExcepts::Gen_ParseInProgress, fMemoryManager);
 
     try
     {
@@ -639,7 +765,10 @@ void SAXParser::parse(const char* const systemId)
         fScanner->scanDocument(systemId);
         fParseInProgress = false;
     }
-
+    catch(const OutOfMemoryException&)
+    {
+        throw;
+    }
     catch (...)
     {
         fParseInProgress = false;
@@ -699,18 +828,40 @@ void SAXParser::setErrorHandler(ErrorHandler* const handler)
 }
 
 
+void SAXParser::setPSVIHandler(PSVIHandler* const handler)
+{
+    fPSVIHandler = handler;
+    if (fPSVIHandler) {
+        fScanner->setPSVIHandler(fPSVIHandler);      
+    }
+    else {
+        fScanner->setPSVIHandler(0);       
+    }
+}
+
 void SAXParser::setEntityResolver(EntityResolver* const resolver)
 {
     fEntityResolver = resolver;
     if (fEntityResolver) {
         fScanner->setEntityHandler(this);
+        fXMLEntityResolver = 0;
     }
     else {
         fScanner->setEntityHandler(0);
     }
 }
 
-
+void SAXParser::setXMLEntityResolver(XMLEntityResolver* const resolver)
+{
+    fXMLEntityResolver = resolver;
+    if (fXMLEntityResolver) {
+        fScanner->setEntityHandler(this);
+        fEntityResolver = 0;
+    }
+    else {
+        fScanner->setEntityHandler(0);
+    }
+}
 
 // ---------------------------------------------------------------------------
 //  SAXParser: Progressive parse methods
@@ -723,7 +874,7 @@ bool SAXParser::parseFirst( const   XMLCh* const    systemId
     //  is in progress.
     //
     if (fParseInProgress)
-        ThrowXML(IOException, XMLExcepts::Gen_ParseInProgress);
+        ThrowXMLwithMemMgr(IOException, XMLExcepts::Gen_ParseInProgress, fMemoryManager);
 
     return fScanner->scanFirst(systemId, toFill);
 }
@@ -736,7 +887,7 @@ bool SAXParser::parseFirst( const   char* const     systemId
     //  is in progress.
     //
     if (fParseInProgress)
-        ThrowXML(IOException, XMLExcepts::Gen_ParseInProgress);
+        ThrowXMLwithMemMgr(IOException, XMLExcepts::Gen_ParseInProgress, fMemoryManager);
 
     return fScanner->scanFirst(systemId, toFill);
 }
@@ -749,7 +900,7 @@ bool SAXParser::parseFirst( const   InputSource&    source
     //  is in progress.
     //
     if (fParseInProgress)
-        ThrowXML(IOException, XMLExcepts::Gen_ParseInProgress);
+        ThrowXMLwithMemMgr(IOException, XMLExcepts::Gen_ParseInProgress, fMemoryManager);
 
     return fScanner->scanFirst(source, toFill);
 }
@@ -1040,9 +1191,9 @@ void SAXParser::startEntityReference(const XMLEntityDecl& entityDecl)
 // ---------------------------------------------------------------------------
 //  SAXParser: Overrides of the DocTypeHandler interface
 // ---------------------------------------------------------------------------
-void SAXParser::attDef( const   DTDElementDecl& elemDecl
-                        , const DTDAttDef&      attDef
-                        , const bool            ignoring)
+void SAXParser::attDef( const   DTDElementDecl&
+                        , const DTDAttDef&
+                        , const bool)
 {
     // Unused by SAX DTDHandler interface at this time
 }
@@ -1054,11 +1205,11 @@ void SAXParser::doctypeComment(const XMLCh* const)
 }
 
 
-void SAXParser::doctypeDecl(const   DTDElementDecl& elemDecl
-                            , const XMLCh* const    publicId
-                            , const XMLCh* const    systemId
-                            , const bool            hasIntSubset
-                            , const bool            hasExtSubset)
+void SAXParser::doctypeDecl(const   DTDElementDecl&
+                            , const XMLCh* const
+                            , const XMLCh* const
+                            , const bool
+                            , const bool)
 {
     // Unused by SAX DTDHandler interface at this time
 }
@@ -1071,8 +1222,8 @@ void SAXParser::doctypePI(  const   XMLCh* const
 }
 
 
-void SAXParser::doctypeWhitespace(  const   XMLCh* const    chars
-                                    , const unsigned int    length)
+void SAXParser::doctypeWhitespace(  const   XMLCh* const
+                                    , const unsigned int)
 {
     // Unused by SAX DTDHandler interface at this time
 }
@@ -1103,7 +1254,7 @@ void SAXParser::endExtSubset()
 
 
 void SAXParser::entityDecl( const   DTDEntityDecl&  entityDecl
-                            , const bool            isPEDecl
+                            , const bool
                             , const bool            isIgnored)
 {
     //
@@ -1184,8 +1335,8 @@ void SAXParser::resetErrors()
 }
 
 
-void SAXParser::error(  const   unsigned int                code
-                        , const XMLCh* const                msgDomain
+void SAXParser::error(  const   unsigned int
+                        , const XMLCh* const
                         , const XMLErrorReporter::ErrTypes  errType
                         , const XMLCh* const                errorText
                         , const XMLCh* const                systemId
@@ -1200,6 +1351,7 @@ void SAXParser::error(  const   unsigned int                code
         , systemId
         , lineNum
         , colNum
+        , fMemoryManager
     );
 
     if (!fErrorHandler)
@@ -1242,7 +1394,7 @@ void SAXParser::resetEntities()
 InputSource*
 SAXParser::resolveEntity(   const   XMLCh* const    publicId
                             , const XMLCh* const    systemId
-                            , const XMLCh* const    baseURI)
+                            , const XMLCh* const)
 {
     // Just map to the SAX entity resolver handler
     if (fEntityResolver)
@@ -1251,6 +1403,19 @@ SAXParser::resolveEntity(   const   XMLCh* const    publicId
 }
 
 
+InputSource*
+SAXParser::resolveEntity(  XMLResourceIdentifier* resourceIdentifier )
+{
+    // Just map to the SAX entity resolver handler
+    if (fEntityResolver)
+        return fEntityResolver->resolveEntity(resourceIdentifier->getPublicId(), 
+                                                resourceIdentifier->getSystemId());
+    if (fXMLEntityResolver)
+        return fXMLEntityResolver->resolveEntity(resourceIdentifier);
+    return 0;
+}
+
+
 void SAXParser::startInputSource(const InputSource&)
 {
     // Nothing to do for this one
@@ -1293,7 +1458,7 @@ Grammar* SAXParser::loadGrammar(const char* const systemId,
 {
     // Avoid multiple entrance
     if (fParseInProgress)
-        ThrowXML(IOException, XMLExcepts::Gen_ParseInProgress);
+        ThrowXMLwithMemMgr(IOException, XMLExcepts::Gen_ParseInProgress, fMemoryManager);
 
     Grammar* grammar = 0;
     try
@@ -1302,7 +1467,10 @@ Grammar* SAXParser::loadGrammar(const char* const systemId,
         grammar = fScanner->loadGrammar(systemId, grammarType, toCache);
         fParseInProgress = false;
     }
-
+    catch(const OutOfMemoryException&)
+    {
+        throw;
+    }
     catch(...)
     {
         fParseInProgress = false;
@@ -1318,7 +1486,7 @@ Grammar* SAXParser::loadGrammar(const XMLCh* const systemId,
 {
     // Avoid multiple entrance
     if (fParseInProgress)
-        ThrowXML(IOException, XMLExcepts::Gen_ParseInProgress);
+        ThrowXMLwithMemMgr(IOException, XMLExcepts::Gen_ParseInProgress, fMemoryManager);
 
     Grammar* grammar = 0;
     try
@@ -1327,7 +1495,10 @@ Grammar* SAXParser::loadGrammar(const XMLCh* const systemId,
         grammar = fScanner->loadGrammar(systemId, grammarType, toCache);
         fParseInProgress = false;
     }
-
+    catch(const OutOfMemoryException&)
+    {
+        throw;
+    }
     catch(...)
     {
         fParseInProgress = false;
@@ -1343,7 +1514,7 @@ Grammar* SAXParser::loadGrammar(const InputSource& source,
 {
     // Avoid multiple entrance
     if (fParseInProgress)
-        ThrowXML(IOException, XMLExcepts::Gen_ParseInProgress);
+        ThrowXMLwithMemMgr(IOException, XMLExcepts::Gen_ParseInProgress, fMemoryManager);
 
     Grammar* grammar = 0;
     try
@@ -1352,7 +1523,10 @@ Grammar* SAXParser::loadGrammar(const InputSource& source,
         grammar = fScanner->loadGrammar(source, grammarType, toCache);
         fParseInProgress = false;
     }
-
+    catch(const OutOfMemoryException&)
+    {
+        throw;
+    }
     catch(...)
     {
         fParseInProgress = false;
