@@ -1,61 +1,39 @@
 /*
- * The Apache Software License, Version 1.1
- *
- * Copyright (c) 2001 The Apache Software Foundation.  All rights
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xerces" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written
- *    permission, please contact apache\@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation, and was
- * originally based on software copyright (c) 2001, International
- * Business Machines, Inc., http://www.ibm.com .  For more information
- * on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Copyright 2001,2004 The Apache Software Foundation.
+ * 
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 /*
  * $Log: Base64.cpp,v $
+ * Revision 1.15  2004/09/08 13:56:21  peiyongz
+ * Apache License Version 2.0
+ *
+ * Revision 1.14  2004/08/17 21:10:33  peiyongz
+ * fix bug in getting CanRep from decode()
+ *
+ * Revision 1.13  2004/08/11 16:47:32  peiyongz
+ * Decoding and getCanRep
+ *
+ * Revision 1.12  2004/06/24 15:00:37  peiyongz
+ * Schema-Errata: E2-54 new specs for base64
+ *
+ * Revision 1.11  2003/12/17 00:18:35  cargilld
+ * Update to memory management so that the static memory manager (one used to call Initialize) is only for static data.
+ *
+ * Revision 1.10  2003/05/20 22:10:02  peiyongz
+ * To differentiate external/internal memory
+ *
  * Revision 1.8  2003/01/27 21:15:56  peiyongz
  * only zero or one space allowed in between B64 character.
  *
@@ -143,6 +121,41 @@ const XMLByte Base64::base64Padding = chEqual;
 
 bool Base64::isInitialized = false;
 
+/***
+ *
+ * Memory Management Issue:
+ *
+ * . For memory allocated for result returned to caller (external memory), 
+ *   the plugged memory manager is used if it is provided, otherwise global 
+ *   new used to retain the pre-memory-manager behaviour.
+ *
+ * . For memory allocated for temperary buffer (internal memory), 
+ *   XMLPlatformUtils::fgMemoryManager is used.
+ *
+ */
+
+static inline void* getExternalMemory(MemoryManager* const allocator
+                                    , unsigned int const   sizeToAllocate)
+{
+   return allocator ? allocator->allocate(sizeToAllocate)
+       : ::operator new(sizeToAllocate);
+}
+
+static inline void* getInternalMemory(unsigned int const   sizeToAllocate)
+{
+    return XMLPlatformUtils::fgMemoryManager->allocate(sizeToAllocate);
+}
+
+/***
+ * internal memory is deallocated by janitorArray
+ */ 
+static inline void returnExternalMemory(MemoryManager* const allocator
+                                      , void*                buffer)
+{
+    allocator ? allocator->deallocate(buffer)
+        : ::operator delete(buffer);
+}
+
 /**
  *     E2-9
  *
@@ -171,14 +184,15 @@ bool Base64::isInitialized = false;
 // number of quadruplets per one line ( must be >1 and <19 )
 const unsigned int Base64::quadsPerLine = 15;
 
-XMLByte* Base64::encode(const XMLByte* const inputData,
-                        const unsigned int   inputLength,
-                        unsigned int*        outputLength)
+XMLByte* Base64::encode(const XMLByte* const inputData
+                      , const unsigned int   inputLength
+                      , unsigned int*        outputLength                      
+                      , MemoryManager* const memMgr)
 {
     if (!isInitialized)
         init();
 
-    if (!inputData)
+    if (!inputData || !outputLength)
         return 0;
 
     int quadrupletCount = ( inputLength + 2 ) / 3;
@@ -195,7 +209,7 @@ XMLByte* Base64::encode(const XMLByte* const inputData,
 
     unsigned int inputIndex = 0;
     unsigned int outputIndex = 0;
-    XMLByte *encodedData = new XMLByte[ quadrupletCount*FOURBYTE + lineCount + 1 ];
+    XMLByte *encodedData = (XMLByte*) getExternalMemory(memMgr, (quadrupletCount*FOURBYTE+lineCount+1) * sizeof(XMLByte));
 
     //
     // Process all quadruplet(s) except the last
@@ -261,8 +275,7 @@ XMLByte* Base64::encode(const XMLByte* const inputData,
     // write out end of string
     encodedData[ outputIndex ] = 0;
 
-    if( outputLength != 0 )
-      (*outputLength) = outputIndex;
+    *outputLength = outputIndex;
 
     return encodedData;
 }
@@ -280,20 +293,159 @@ XMLByte* Base64::encode(const XMLByte* const inputData,
 // Since decode() has track of length of the decoded data, we
 // will get this length from decode(), instead of strLen().
 //
-int Base64::getDataLength( const XMLCh* const inputData )
+int Base64::getDataLength(const XMLCh*         const inputData
+                        ,       MemoryManager* const manager
+                        ,       Conformance          conform )
+
 {
     unsigned int    retLen = 0;
-    XMLCh* decodedData = decode(inputData, &retLen);
+    XMLCh* decodedData = decode(inputData, &retLen, manager, conform);
 
     if ( !decodedData )
         return -1;
     else
     {
-        delete[] decodedData;
+        returnExternalMemory(manager, decodedData);
         return retLen;
     }
 }
 
+XMLByte* Base64::decode(const XMLByte*       const  inputData
+                      ,       unsigned int*         decodedLength
+                      ,       MemoryManager* const  memMgr
+                      ,       Conformance           conform )
+{
+    XMLByte* canRepInByte = 0;
+    XMLByte* retStr = decode(
+                              inputData
+                            , decodedLength
+                            , canRepInByte
+                            , memMgr
+                            , conform);
+
+    /***
+     * Release the canRepData
+     */ 
+    if (retStr)
+        returnExternalMemory(memMgr, canRepInByte);
+
+    return retStr;
+}
+
+XMLCh* Base64::decode(const XMLCh*         const   inputData
+                    ,       unsigned int*          decodedLen
+                    ,       MemoryManager* const   memMgr
+                    ,       Conformance            conform )
+{
+	if (!inputData)
+		return 0;
+
+    /***
+     *  Move input data to a XMLByte buffer
+     */
+	unsigned int srcLen = XMLString::stringLen(inputData);
+    XMLByte *dataInByte = (XMLByte*) getExternalMemory(memMgr, (srcLen+1) * sizeof(XMLByte));
+    ArrayJanitor<XMLByte> janFill(dataInByte, memMgr ? memMgr : XMLPlatformUtils::fgMemoryManager);
+
+    for (unsigned int i = 0; i < srcLen; i++)
+		dataInByte[i] = (XMLByte)inputData[i];
+
+	dataInByte[srcLen] = 0;
+
+    /***
+     * Forward to the actual decoding method to do the decoding
+     */
+	*decodedLen = 0;
+	XMLByte *DecodedBuf = decode(dataInByte, decodedLen, memMgr, conform);
+
+	if (!DecodedBuf)
+		return 0;
+
+    /***
+     * Move decoded data to a XMLCh buffer to return
+     */
+    XMLCh *toRet = (XMLCh*) getExternalMemory(memMgr, (*decodedLen+1) * sizeof(XMLCh));
+               
+    for (unsigned int j = 0; j < *decodedLen; j++)
+		toRet[j] = (XMLCh)DecodedBuf[j];
+
+	toRet[*decodedLen] = 0;
+
+    /***
+     * Release the memory allocated in the actual decoding method
+     */ 
+    returnExternalMemory(memMgr, DecodedBuf);
+
+    return toRet;
+}
+
+/***
+* E2-54
+*
+* Canonical-base64Binary ::= (B64 B64 B64 B64)*((B64 B64 B16 '=')|(B64 B04 '=='))? 
+* B04                    ::= [AQgw]
+* B16                    ::= [AEIMQUYcgkosw048]
+* B64                    ::= [A-Za-z0-9+/] 
+*
+***/
+XMLCh* Base64::getCanonicalRepresentation(const XMLCh*         const   inputData
+                                        ,       MemoryManager* const   memMgr
+                                        ,       Conformance            conform)
+    
+{
+	if (!inputData || !*inputData) 
+		return 0;
+
+    /***
+     *  Move input data to a XMLByte buffer
+     */
+	unsigned int srcLen = XMLString::stringLen(inputData);
+    XMLByte *dataInByte = (XMLByte*) getExternalMemory(memMgr, (srcLen+1) * sizeof(XMLByte));
+    ArrayJanitor<XMLByte> janFill(dataInByte, memMgr ? memMgr : XMLPlatformUtils::fgMemoryManager);
+
+    for (unsigned int i = 0; i < srcLen; i++)
+		dataInByte[i] = (XMLByte)inputData[i];
+
+	dataInByte[srcLen] = 0;
+
+    /***
+     * Forward to the actual decoding method to do the decoding
+     */
+	unsigned int decodedLength = 0;
+    XMLByte*     canRepInByte = 0;
+    XMLByte*     retStr = decode(
+                              dataInByte
+                            , &decodedLength
+                            , canRepInByte
+                            , memMgr
+                            , conform);
+
+    if (!retStr)
+        return 0;
+
+    /***
+     * Move canonical representation to a XMLCh buffer to return
+     */
+    unsigned int canRepLen = XMLString::stringLen((char*)canRepInByte);
+    XMLCh *canRepData = (XMLCh*) getExternalMemory(memMgr, (canRepLen + 1) * sizeof(XMLCh));
+               
+    for (unsigned int j = 0; j < canRepLen; j++)
+		canRepData[j] = (XMLCh)canRepInByte[j];
+
+	canRepData[canRepLen] = 0;
+
+    /***
+     * Release the memory allocated in the actual decoding method
+     */ 
+    returnExternalMemory(memMgr, retStr);
+    returnExternalMemory(memMgr, canRepInByte);
+
+    return canRepData;
+}
+// -----------------------------------------------------------------------
+//  Helper methods
+// -----------------------------------------------------------------------
+
 //
 // return 0(null) if invalid data found.
 // return the buffer containning decoded data otherwise
@@ -316,45 +468,115 @@ int Base64::getDataLength( const XMLCh* const inputData )
  *     B04          ::= [AQgw]
  *     B16          ::= [AEIMQUYcgkosw048]
  *     B64          ::= [A-Za-z0-9+/]
+ *
+ *
+ *     E2-54
+ *
+ *     Base64Binary  ::=  ((B64S B64S B64S B64S)*
+ *                         ((B64S B64S B64S B64) |
+ *                          (B64S B64S B16S '=') |
+ *                          (B64S B04S '=' #x20? '=')))?
+ *
+ *     B64S         ::= B64 #x20?
+ *     B16S         ::= B16 #x20?
+ *     B04S         ::= B04 #x20?
+ *
+ *
+ *     Note that this grammar requires the number of non-whitespace characters 
+ *     in the lexical form to be a multiple of four, and for equals signs to 
+ *     appear only at the end of the lexical form; strings which do not meet these 
+ *     constraints are not legal lexical forms of base64Binary because they 
+ *     cannot successfully be decoded by base64 decoders.
+ * 
+ *     Note: 
+ *     The above definition of the lexical space is more restrictive than that given 
+ *     in [RFC 2045] as regards whitespace -- this is not an issue in practice. Any 
+ *     string compatible with the RFC can occur in an element or attribute validated 
+ *     by this type, because the whiteSpace facet of this type is fixed to collapse, 
+ *     which means that all leading and trailing whitespace will be stripped, and all 
+ *     internal whitespace collapsed to single space characters, before the above grammar 
+ *     is enforced.
+ *
 */
 
-XMLByte* Base64::decode(const XMLByte* const inputData,
-                        unsigned int*        outputLength)
+XMLByte* Base64::decode (   const XMLByte*        const   inputData
+                          ,       unsigned int*           decodedLength
+                          ,       XMLByte*&               canRepData
+                          ,       MemoryManager*  const   memMgr
+                          ,       Conformance             conform
+                        )
 {
     if (!isInitialized)
         init();
 
     if ((!inputData) || (!*inputData))
         return 0;
-
+    
     //
     // remove all XML whitespaces from the base64Data
     //
     int inputLength = XMLString::stringLen( (const char* const)inputData );
-    XMLByte* rawInputData = new XMLByte[ inputLength + 1 ];
-    ArrayJanitor<XMLByte> jan(rawInputData);
+    XMLByte* rawInputData = (XMLByte*) getExternalMemory(memMgr, (inputLength+1) * sizeof(XMLByte));
+    ArrayJanitor<XMLByte> jan(rawInputData, memMgr ? memMgr : XMLPlatformUtils::fgMemoryManager);
 
     int inputIndex = 0;
     int rawInputLength = 0;
     bool inWhiteSpace = false;
 
-    while ( inputIndex < inputLength )
+    switch (conform)
     {
-        if (!XMLChar1_0::isWhitespace(inputData[inputIndex]))
+    case Conf_RFC2045:
+        while ( inputIndex < inputLength )
         {
-            rawInputData[ rawInputLength++ ] = inputData[ inputIndex ];
-            inWhiteSpace = false;
-        }
-        else
-        {
-            if (inWhiteSpace)
-                return 0; // more than 1 whitespaces encountered
-            else
-                inWhiteSpace = true;
+            if (!XMLChar1_0::isWhitespace(inputData[inputIndex]))
+            {
+                rawInputData[ rawInputLength++ ] = inputData[ inputIndex ];
+            }
+            // RFC2045 does not explicitly forbid more than ONE whitespace 
+            // before, in between, or after base64 octects.
+            // Besides, S? allows more than ONE whitespace as specified in the production 
+            // [3]   S   ::=   (#x20 | #x9 | #xD | #xA)+
+            // therefore we do not detect multiple ws
+
+            inputIndex++;
         }
 
-        inputIndex++;
+        break;
+    case Conf_Schema:
+        // no leading #x20
+        if (chSpace == inputData[inputIndex])
+            return 0;
+
+        while ( inputIndex < inputLength )
+        {
+            if (chSpace != inputData[inputIndex])
+            {
+                rawInputData[ rawInputLength++ ] = inputData[ inputIndex ];
+                inWhiteSpace = false;
+            }
+            else
+            {
+                if (inWhiteSpace)
+                    return 0; // more than 1 #x20 encountered
+                else
+                    inWhiteSpace = true;
+            }
+
+            inputIndex++;
+        }
+
+        // no trailing #x20
+        if (inWhiteSpace)
+            return 0;
+
+        break;
+
+    default:
+        break;
     }
+
+    //now rawInputData contains canonical representation 
+    //if the data is valid Base64
     rawInputData[ rawInputLength ] = 0;
 
     // the length of raw data should be divisible by four
@@ -373,7 +595,7 @@ XMLByte* Base64::decode(const XMLByte* const inputData,
 
     int rawInputIndex  = 0;
     int outputIndex    = 0;
-    XMLByte *decodedData = new XMLByte[ quadrupletCount*3 + 1 ];
+    XMLByte *decodedData = (XMLByte*) getExternalMemory(memMgr, (quadrupletCount*3+1) * sizeof(XMLByte));
 
     //
     // Process all quadruplet(s) except the last
@@ -388,7 +610,7 @@ XMLByte* Base64::decode(const XMLByte* const inputData,
             !isData( (d4 = rawInputData[ rawInputIndex++ ]) ))
         {
             // if found "no data" just return NULL
-            delete[] decodedData;
+            returnExternalMemory(memMgr, decodedData);
             return 0;
         }
 
@@ -411,7 +633,7 @@ XMLByte* Base64::decode(const XMLByte* const inputData,
         !isData( (d2 = rawInputData[ rawInputIndex++ ]) ))
     {
         // if found "no data" just return NULL
-        delete[] decodedData;
+        returnExternalMemory(memMgr, decodedData);
         return 0;
     }
 
@@ -430,7 +652,7 @@ XMLByte* Base64::decode(const XMLByte* const inputData,
             // two PAD e.g. 3c==
             if ((b2 & 0xf) != 0) // last 4 bits should be zero
             {
-                delete[] decodedData;
+                returnExternalMemory(memMgr, decodedData);
                 return 0;
             }
 
@@ -442,7 +664,7 @@ XMLByte* Base64::decode(const XMLByte* const inputData,
             b3 = base64Inverse[ d3 ];
             if (( b3 & 0x3 ) != 0 ) // last 2 bits should be zero
             {
-                delete[] decodedData;
+                returnExternalMemory(memMgr, decodedData);
                 return 0;
             }
 
@@ -452,7 +674,7 @@ XMLByte* Base64::decode(const XMLByte* const inputData,
         else
         {
             // an error like "3c[Pad]r", "3cdX", "3cXd", "3cXX" where X is non data
-            delete[] decodedData;
+            returnExternalMemory(memMgr, decodedData);
             return 0;
         }
     }
@@ -468,49 +690,15 @@ XMLByte* Base64::decode(const XMLByte* const inputData,
 
     // write out the end of string
     decodedData[ outputIndex ] = 0;
-    *outputLength = outputIndex;
+    *decodedLength = outputIndex;
+
+    //allow the caller to have access to the canonical representation
+    jan.release(); 
+    canRepData = rawInputData;
 
     return decodedData;
 }
 
-XMLCh* Base64::decode(const XMLCh* const inputData,
-                      unsigned int*      outputLength)
-{
-	if (!inputData)
-		return 0;
-
-	unsigned int srcLen = XMLString::stringLen(inputData);
-    XMLByte *toFill = new XMLByte[srcLen+1];
-    ArrayJanitor<XMLByte> janFill(toFill);
-
-    for (unsigned int i = 0; i < srcLen; i++)
-		toFill[i] = (XMLByte)inputData[i];
-
-	toFill[srcLen] = 0;
-
-	unsigned int      decodedLen = 0;
-	XMLByte *DecodedBuf = decode(toFill, &decodedLen);
-
-	if (!DecodedBuf)
-		return 0;
-
-    XMLCh *toRet = new XMLCh[decodedLen+1];
-
-    for (unsigned int j = 0; j < decodedLen; j++)
-		toRet[j] = (XMLCh)DecodedBuf[j];
-
-	toRet[decodedLen] = 0;
-
-	*outputLength = decodedLen;
-	delete[] DecodedBuf;
-
-	return toRet;
-
-}
-
-// -----------------------------------------------------------------------
-//  Helper methods
-// -----------------------------------------------------------------------
 void Base64::init()
 {
     if (isInitialized)
