@@ -1,61 +1,66 @@
 /*
- * The Apache Software License, Version 1.1
- *
- * Copyright (c) 1999-2000 The Apache Software Foundation.  All rights
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xerces" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written
- *    permission, please contact apache\@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation, and was
- * originally based on software copyright (c) 1999, International
- * Business Machines, Inc., http://www.ibm.com .  For more information
- * on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Copyright 1999-2004 The Apache Software Foundation.
+ * 
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 /*
  * $Log: UnixWarePlatformUtils.cpp,v $
+ * Revision 1.18  2004/09/08 13:56:43  peiyongz
+ * Apache License Version 2.0
+ *
+ * Revision 1.17  2004/01/29 11:51:21  cargilld
+ * Code cleanup changes to get rid of various compiler diagnostic messages.
+ *
+ * Revision 1.16  2004/01/13 16:34:22  cargilld
+ * Misc memory management changes.
+ *
+ * Revision 1.15  2003/12/24 15:24:14  cargilld
+ * More updates to memory management so that the static memory manager.
+ *
+ * Revision 1.14  2003/12/17 19:58:05  cargilld
+ * Platform update for memory management so that the static memory manager (one
+ * used to call Initialize) is only for static data.
+ *
+ * Revision 1.13  2003/12/17 15:16:10  cargilld
+ * Platform update for memory management so that the static memory manager (one
+ * used to call Initialize) is only for static data.
+ *
+ * Revision 1.12  2003/12/17 13:58:03  cargilld
+ * Platform update for memory management so that the static memory manager (one
+ * used to call Initialize) is only for static data.
+ *
+ * Revision 1.11  2003/10/01 16:32:40  neilg
+ * improve handling of out of memory conditions, bug #23415.  Thanks to David Cargill.
+ *
+ * Revision 1.10  2003/05/15 18:37:49  knoaman
+ * Partial implementation of the configurable memory manager.
+ *
+ * Revision 1.9  2003/04/25 17:21:31  peiyongz
+ * throw exception if getcwd() fails
+ *
+ * Revision 1.8  2003/04/24 02:59:20  peiyongz
+ * Logical Path Resolution
+ *
+ * Revision 1.7  2003/04/21 04:32:10  peiyongz
+ * performance tuning
+ *
+ * Revision 1.6  2003/03/09 17:01:08  peiyongz
+ * PanicHandler
+ *
+ * Revision 1.5  2003/03/07 20:34:16  tng
+ * [Bug 17774] Unixware platform utils not implemented .  Patch from Peter Crozier.
+ *
  * Revision 1.4  2002/11/04 15:13:01  tng
  * C++ Namespace Support.
  *
@@ -131,6 +136,7 @@
 #include    <xercesc/util/XMLString.hpp>
 #include    <xercesc/util/XMLUniDefs.hpp>
 #include    <xercesc/util/XMLUni.hpp>
+#include    <xercesc/util/OutOfMemoryException.hpp>
 
 #if defined (XML_USE_ICU_TRANSCODER)
     #include <xercesc/util/Transcoders/ICU/ICUTransService.hpp>
@@ -195,9 +201,13 @@ XMLMsgLoader* XMLPlatformUtils::loadAMsgSet(const XMLCh* const msgDomain)
         retVal = new InMemMsgLoader(msgDomain);
 #endif
     }
+    catch(const OutOfMemoryException&)
+    {
+        throw;
+    }
     catch(...)
     {
-        panic(XMLPlatformUtils::Panic_CantLoadMsgDomain);
+        panic(PanicHandler::Panic_CantLoadMsgDomain);
     }
     return retVal;
 }
@@ -232,78 +242,64 @@ XMLTransService* XMLPlatformUtils::makeTransService()
 // ---------------------------------------------------------------------------
 void XMLPlatformUtils::panic(const PanicReasons reason)
 {
-	 const char* reasonStr = "Unknown reason";
-    if (reason == Panic_NoTransService)
-        reasonStr = "Could not load a transcoding service";
-    else if (reason == Panic_NoDefTranscoder)
-        reasonStr = "Could not load a local code page transcoder";
-    else if (reason == Panic_CantFindLib)
-        reasonStr = "Could not find the xerces-c DLL";
-    else if (reason == Panic_UnknownMsgDomain)
-        reasonStr = "Unknown message domain";
-    else if (reason == Panic_CantLoadMsgDomain)
-        reasonStr = "Cannot load message domain";
-    else if (reason == Panic_SynchronizationErr)
-        reasonStr = "Cannot synchronize system or mutex";
-    else if (reason == Panic_SystemInit)
-        reasonStr = "Cannot initialize the system or mutex";
-
-    fprintf(stderr, "%s\n", reasonStr);
-
-    exit(-1);
+    fgUserPanicHandler? fgUserPanicHandler->panic(reason) : fgDefaultPanicHandler->panic(reason);
 }
 
 // ---------------------------------------------------------------------------
 //  XMLPlatformUtils: File Methods
 // ---------------------------------------------------------------------------
-unsigned int XMLPlatformUtils::curFilePos(FileHandle theFile)
+unsigned int XMLPlatformUtils::curFilePos(FileHandle theFile
+                                          , MemoryManager* const manager)
 {
     // Get the current position
     int curPos = ftell((FILE*) theFile);
     if (curPos == -1)
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotGetSize);
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotGetSize, manager);
 
     return (unsigned int)curPos;
 }
 
-void XMLPlatformUtils::closeFile(FileHandle theFile)
+void XMLPlatformUtils::closeFile(FileHandle theFile
+                                 , MemoryManager* const manager)
 {
     if (fclose((FILE*) theFile))
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotCloseFile);
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotCloseFile, manager);
 }
 
-unsigned int XMLPlatformUtils::fileSize(FileHandle theFile)
+unsigned int XMLPlatformUtils::fileSize(FileHandle theFile
+                                        , MemoryManager* const manager)
 {
     // Get the current position
     long  int curPos = ftell((FILE*) theFile);
     if (curPos == -1)
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotGetCurPos);
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotGetCurPos, manager);
 
     // Seek to the end and save that value for return
 	if (fseek((FILE*) theFile, 0, SEEK_END))
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotSeekToEnd);
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotSeekToEnd, manager);
 
     long int retVal = ftell((FILE*) theFile);
     if (retVal == -1)
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotSeekToEnd);
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotSeekToEnd, manager);
 
     // And put the pointer back
     if (fseek((FILE*) theFile, curPos, SEEK_SET))
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotSeekToPos);
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotSeekToPos, manager);
 
     return (unsigned int)retVal;
 }
 
-FileHandle XMLPlatformUtils::openFile(const XMLCh* const fileName)
+FileHandle XMLPlatformUtils::openFile(const XMLCh* const fileName
+                                      , MemoryManager* const manager)
 {
-    const char* tmpFileName = XMLString::transcode(fileName);
-    ArrayJanitor<char> janText((char*)tmpFileName);
+    const char* tmpFileName = XMLString::transcode(fileName, manager);
+    ArrayJanitor<char> janText((char*)tmpFileName, manager);
     FileHandle retVal = (FILE*)fopen( tmpFileName , "rb" );
 
     if (retVal == NULL)
@@ -311,7 +307,8 @@ FileHandle XMLPlatformUtils::openFile(const XMLCh* const fileName)
     return retVal;
 }
 
-FileHandle XMLPlatformUtils::openFile(const char* const fileName)
+FileHandle XMLPlatformUtils::openFile(const char* const fileName
+                                      , MemoryManager* const manager)
 {
     FileHandle retVal = (FILE*)fopen( fileName , "rb" );
 
@@ -320,7 +317,21 @@ FileHandle XMLPlatformUtils::openFile(const char* const fileName)
     return retVal;
 }
 
-FileHandle XMLPlatformUtils::openStdInHandle()
+FileHandle XMLPlatformUtils::openFileToWrite(const XMLCh* const fileName
+                                             , MemoryManager* const manager)
+{
+    const char* tmpFileName = XMLString::transcode(fileName, manager);
+    ArrayJanitor<char> janText((char*)tmpFileName, manager);
+    return fopen( tmpFileName , "wb" );
+}
+
+FileHandle XMLPlatformUtils::openFileToWrite(const char* const fileName
+                                             , MemoryManager* const manager)
+{
+    return fopen( fileName , "wb" );
+}
+
+FileHandle XMLPlatformUtils::openStdInHandle(MemoryManager* const manager)
 {
 	return (FileHandle)fdopen(dup(0), "rb");
 }
@@ -328,57 +339,95 @@ FileHandle XMLPlatformUtils::openStdInHandle()
 unsigned int
 XMLPlatformUtils::readFileBuffer( FileHandle          theFile
                                , const unsigned int      toRead
-                               , XMLByte* const          toFill)
+                               , XMLByte* const          toFill
+                               , MemoryManager* const manager)
 {
     size_t noOfItemsRead = fread((void*) toFill, 1, toRead, (FILE*) theFile);
 
     if(ferror((FILE*) theFile))
     {
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotReadFromFile);
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotReadFromFile, manager);
     }
 
     return (unsigned int) noOfItemsRead;
 }
 
+void
+XMLPlatformUtils::writeBufferToFile( FileHandle     const  theFile
+                                   , long                  toWrite
+                                   , const XMLByte* const  toFlush
+                                   , MemoryManager* const  manager)
+{
+    if (!theFile        ||
+        (toWrite <= 0 ) ||
+        !toFlush         )
+        return;
 
-void XMLPlatformUtils::resetFile(FileHandle theFile)
+    const XMLByte* tmpFlush = (const XMLByte*) toFlush;
+    size_t bytesWritten = 0;
+
+    while (true)
+    {
+        bytesWritten=fwrite(tmpFlush, sizeof(XMLByte), toWrite, (FILE*)theFile);
+
+        if(ferror((FILE*)theFile))
+        {
+            ThrowXMLwithMemMgr(XMLPlatformUtilsException, XMLExcepts::File_CouldNotWriteToFile, manager);
+        }
+
+        if (bytesWritten < toWrite) //incomplete write
+        {
+            tmpFlush+=bytesWritten;
+            toWrite-=bytesWritten;
+            bytesWritten=0;
+        }
+        else
+            return;
+    }
+
+    return;
+}
+
+void XMLPlatformUtils::resetFile(FileHandle theFile
+                                 , MemoryManager* const manager)
 {
     // Seek to the start of the file
     if (fseek((FILE*) theFile, 0, SEEK_SET))
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotResetFile);
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotResetFile, manager);
 }
 
 
 // ---------------------------------------------------------------------------
 //  XMLPlatformUtils: File system methods
 // ---------------------------------------------------------------------------
-XMLCh* XMLPlatformUtils::getFullPath(const XMLCh* const srcPath)
+XMLCh* XMLPlatformUtils::getFullPath(const XMLCh* const srcPath,
+                                     MemoryManager* const manager)
 {
     //
     //  NOTE: The path provided has always already been opened successfully,
     //  so we know that its not some pathological freaky path. It comes in
     //  in native format, and goes out as Unicode always
     //
-    char* newSrc = XMLString::transcode(srcPath);
-    ArrayJanitor<char> janText(newSrc);
+    char* newSrc = XMLString::transcode(srcPath, manager);
+    ArrayJanitor<char> janText(newSrc, manager);
 
     // Use a local buffer that is big enough for the largest legal path
-    char *absPath = new char[PATH_MAX];
-    ArrayJanitor<char> janText2(absPath);
+    char absPath[PATH_MAX + 1];
     //get the absolute path
-    char* retPath = realpath(newSrc, absPath);
+    char* retPath = realpath(newSrc, &absPath[0]);
 
     if (!retPath)
     {
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotGetBasePathName);
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotGetBasePathName, manager);
     }
-    return XMLString::transcode(absPath);
+    return XMLString::transcode(absPath, manager);
 }
 
-bool XMLPlatformUtils::isRelative(const XMLCh* const toCheck)
+bool XMLPlatformUtils::isRelative(const XMLCh* const toCheck
+                                  , MemoryManager* const manager)
 {
     // Check for pathological case of empty path
     if (!toCheck[0])
@@ -396,112 +445,24 @@ bool XMLPlatformUtils::isRelative(const XMLCh* const toCheck)
     return true;
 }
 
-XMLCh* XMLPlatformUtils::weavePaths(const   XMLCh* const    basePath
-                                    , const XMLCh* const    relativePath)
-
+XMLCh* XMLPlatformUtils::getCurrentDirectory(MemoryManager* const manager)
 {
-    // Create a buffer as large as both parts and empty it
-    XMLCh* tmpBuf = new XMLCh[XMLString::stringLen(basePath)
-                              + XMLString::stringLen(relativePath)
-                              + 2];
-    *tmpBuf = 0;
+    char  dirBuf[PATH_MAX + 2];
+    char  *curDir = getcwd(&dirBuf[0], PATH_MAX + 1);
 
-    //
-    //  If we have no base path, then just take the relative path as
-    //  is.
-    //
-    if (!basePath)
+    if (!curDir)
     {
-        XMLString::copyString(tmpBuf, relativePath);
-        return tmpBuf;
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotGetBasePathName, manager);
     }
 
-    if (!*basePath)
-    {
-        XMLString::copyString(tmpBuf, relativePath);
-        return tmpBuf;
-    }
-
-    const XMLCh* basePtr = basePath + (XMLString::stringLen(basePath) - 1);
-    if ((*basePtr != chForwardSlash)
-    &&  (*basePtr != chBackSlash))
-    {
-        while ((basePtr >= basePath)
-        &&     ((*basePtr != chForwardSlash) && (*basePtr != chBackSlash)))
-        {
-            basePtr--;
-        }
-    }
-
-    // There is no relevant base path, so just take the relative part
-    if (basePtr < basePath)
-    {
-        XMLString::copyString(tmpBuf, relativePath);
-        return tmpBuf;
-    }
-
-    // After this, make sure the buffer gets handled if we exit early
-    ArrayJanitor<XMLCh> janBuf(tmpBuf);
-
-    //
-    //  We have some path part, so we need to check to see if we ahve to
-    //  weave any of the parts together.
-    //
-    const XMLCh* pathPtr = relativePath;
-    while (true)
-    {
-        // If it does not start with some period, then we are done
-        if (*pathPtr != chPeriod)
-            break;
-
-        unsigned int periodCount = 1;
-        pathPtr++;
-        if (*pathPtr == chPeriod)
-        {
-            pathPtr++;
-            periodCount++;
-        }
-
-        // Has to be followed by a \ or / or the null to mean anything
-        if ((*pathPtr != chForwardSlash) && (*pathPtr != chBackSlash)
-        &&  *pathPtr)
-        {
-            break;
-        }
-        if (*pathPtr)
-            pathPtr++;
-
-        // If its one period, just eat it, else move backwards in the base
-        if (periodCount == 2)
-        {
-            basePtr--;
-            while ((basePtr >= basePath)
-            &&     ((*basePtr != chForwardSlash) && (*basePtr != chBackSlash)))
-            {
-                basePtr--;
-            }
-
-            // The base cannot provide enough levels, so its in error/
-            if (basePtr < basePath)
-                ThrowXML(XMLPlatformUtilsException,
-                         XMLExcepts::File_BasePathUnderflow);
-        }
-    }
-
-    // Copy the base part up to the base pointer
-    XMLCh* bufPtr = tmpBuf;
-    const XMLCh* tmpPtr = basePath;
-    while (tmpPtr <= basePtr)
-        *bufPtr++ = *tmpPtr++;
-
-    // And then copy on the rest of our path
-    XMLString::copyString(bufPtr, pathPtr);
-
-    // Orphan the buffer and return it
-    janBuf.orphan();
-    return tmpBuf;
+    return XMLString::transcode(curDir, manager);
 }
 
+inline bool XMLPlatformUtils::isAnySlash(XMLCh c) 
+{
+    return ( chBackSlash == c || chForwardSlash == c);
+}
 
 // ---------------------------------------------------------------------------
 //  XMLPlatformUtils: Timing Methods
@@ -539,10 +500,10 @@ void XMLPlatformUtils::platformInit()
     gAtomicOpMutex = new pthread_mutex_t;
 
     if (pthread_mutex_init(gAtomicOpMutex, NULL))
-        panic( XMLPlatformUtils::Panic_SystemInit );
+        panic( PanicHandler::Panic_SystemInit );
 }
 
-class  RecursiveMutex
+class  RecursiveMutex : public XMemory
 {
 public:
     pthread_mutex_t   mutex;
@@ -551,15 +512,15 @@ public:
 
     RecursiveMutex() {
 		if (pthread_mutex_init(&mutex, NULL))
-			ThrowXML(XMLPlatformUtilsException, XMLExcepts::Mutex_CouldNotCreate);
+			ThrowXMLwithMemMgr(XMLPlatformUtilsException, XMLExcepts::Mutex_CouldNotCreate, XMLPlatformUtils::fgMemoryManager);
 		recursionCount = 0;
 		tid = 0;
-	};
+	}
 
     ~RecursiveMutex() {
 		if (pthread_mutex_destroy(&mutex))
-			ThrowXML(XMLPlatformUtilsException, XMLExcepts::Mutex_CouldNotDestroy);
-	};
+			ThrowXMLwithMemMgr(XMLPlatformUtilsException, XMLExcepts::Mutex_CouldNotDestroy, XMLPlatformUtils::fgMemoryManager);
+	}
 
 	void lock()      {
 		if (pthread_equal(tid, pthread_self()))
@@ -568,10 +529,10 @@ public:
 			return;
 		}
 		if (pthread_mutex_lock(&mutex) != 0)
-			ThrowXML(XMLPlatformUtilsException, XMLExcepts::Mutex_CouldNotLock);
+			ThrowXMLwithMemMgr(XMLPlatformUtilsException, XMLExcepts::Mutex_CouldNotLock, XMLPlatformUtils::fgMemoryManager);
 		tid = pthread_self();
 		recursionCount = 1;
-	};
+	}
 
 
 	void unlock()    {
@@ -579,15 +540,15 @@ public:
 			return;
 
 		if (pthread_mutex_unlock(&mutex) != 0)
-			ThrowXML(XMLPlatformUtilsException, XMLExcepts::Mutex_CouldNotUnlock);
+			ThrowXMLwithMemMgr(XMLPlatformUtilsException, XMLExcepts::Mutex_CouldNotUnlock, XMLPlatformUtils::fgMemoryManager);
 		tid = 0;
-	};
+	}
 };
 
 void* XMLPlatformUtils::makeMutex()
 {
     return new RecursiveMutex;
-};
+}
 
 
 void XMLPlatformUtils::closeMutex(void* const mtxHandle)
@@ -596,7 +557,7 @@ void XMLPlatformUtils::closeMutex(void* const mtxHandle)
         return;
     RecursiveMutex *rm = (RecursiveMutex *)mtxHandle;
     delete rm;
-};
+}
 
 
 void XMLPlatformUtils::lockMutex(void* const mtxHandle)
@@ -627,14 +588,14 @@ void* XMLPlatformUtils::compareAndSwap ( void**      toFill ,
                     const void* const toCompare)
 {
     if (pthread_mutex_lock( gAtomicOpMutex))
-        panic(XMLPlatformUtils::Panic_SynchronizationErr);
+        panic(PanicHandler::Panic_SynchronizationErr);
 
     void *retVal = *toFill;
     if (*toFill == toCompare)
               *toFill = (void *)newValue;
 
     if (pthread_mutex_unlock( gAtomicOpMutex))
-        panic(XMLPlatformUtils::Panic_SynchronizationErr);
+        panic(PanicHandler::Panic_SynchronizationErr);
 
 
     return retVal;
@@ -643,12 +604,12 @@ void* XMLPlatformUtils::compareAndSwap ( void**      toFill ,
 int XMLPlatformUtils::atomicIncrement(int &location)
 {
     if (pthread_mutex_lock( gAtomicOpMutex))
-        panic(XMLPlatformUtils::Panic_SynchronizationErr);
+        panic(PanicHandler::Panic_SynchronizationErr);
 
     int tmp = ++location;
 
     if (pthread_mutex_unlock( gAtomicOpMutex))
-        panic(XMLPlatformUtils::Panic_SynchronizationErr);
+        panic(PanicHandler::Panic_SynchronizationErr);
 
     return tmp;
 }
@@ -656,12 +617,12 @@ int XMLPlatformUtils::atomicIncrement(int &location)
 int XMLPlatformUtils::atomicDecrement(int &location)
 {
     if (pthread_mutex_lock( gAtomicOpMutex))
-        panic(XMLPlatformUtils::Panic_SynchronizationErr);
+        panic(PanicHandler::Panic_SynchronizationErr);
 
     int tmp = --location;
 
     if (pthread_mutex_unlock( gAtomicOpMutex))
-        panic(XMLPlatformUtils::Panic_SynchronizationErr);
+        panic(PanicHandler::Panic_SynchronizationErr);
 
     return tmp;
 }
@@ -723,4 +684,6 @@ void XMLPlatformUtils::platformTerm()
 #endif
 }
 
+#include <xercesc/util/LogicalPath.c>
+
 XERCES_CPP_NAMESPACE_END
