@@ -1,61 +1,37 @@
 /*
- * The Apache Software License, Version 1.1
- *
- * Copyright (c) 2001-2003 The Apache Software Foundation.  All rights
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xerces" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written
- *    permission, please contact apache\@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation, and was
- * originally based on software copyright (c) 2001, International
- * Business Machines, Inc., http://www.ibm.com .  For more information
- * on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Copyright 2001-2004 The Apache Software Foundation.
+ * 
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 /*
  * $Log: RefHash2KeysTableOf.hpp,v $
+ * Revision 1.12  2004/09/08 13:56:22  peiyongz
+ * Apache License Version 2.0
+ *
+ * Revision 1.11  2004/08/30 15:18:35  amassari
+ * - Added transferElement API
+ * - The iterator class now can iterate over the items having the same primary key
+ *
+ * Revision 1.10  2004/03/01 15:03:08  peiyongz
+ * new getter: getHashModulus
+ *
+ * Revision 1.9  2004/01/29 11:48:46  cargilld
+ * Code cleanup changes to get rid of various compiler diagnostic messages.
+ *
+ * Revision 1.8  2003/12/17 00:18:35  cargilld
+ * Update to memory management so that the static memory manager (one used to call Initialize) is only for static data.
+ *
  * Revision 1.7  2003/10/20 11:45:06  gareth
  * Made enumerators inherit from XMemory.
  *
@@ -131,11 +107,19 @@ template <class TVal> struct RefHash2KeysTableBucketElem : public XMemory
 		: fData(value), fNext(next), fKey1(key1), fKey2(key2)
         {
         }
+    ~RefHash2KeysTableBucketElem() {};
 
     TVal*                           fData;
     RefHash2KeysTableBucketElem<TVal>*   fNext;
 	void*							fKey1;
 	int							fKey2;
+
+private:
+    // -----------------------------------------------------------------------
+    //  Unimplemented constructors and operators
+    // -----------------------------------------------------------------------
+    RefHash2KeysTableBucketElem(const RefHash2KeysTableBucketElem<TVal>&);
+    RefHash2KeysTableBucketElem<TVal>& operator=(const RefHash2KeysTableBucketElem<TVal>&);
 };
 
 
@@ -178,7 +162,7 @@ public:
     bool containsKey(const void* const key1, const int key2) const;
     void removeKey(const void* const key1, const int key2);
     void removeAll();
-
+    void transferElement(const void* const key1, void* key2);
 
     // -----------------------------------------------------------------------
     //  Getters
@@ -186,20 +170,27 @@ public:
     TVal* get(const void* const key1, const int key2);
     const TVal* get(const void* const key1, const int key2) const;
 
+    MemoryManager* getMemoryManager() const;
+    unsigned int   getHashModulus()   const;
 
     // -----------------------------------------------------------------------
     //  Putters
     // -----------------------------------------------------------------------
 	void put(void* key1, int key2, TVal* const valueToAdopt);
 
-
 private :
     // -----------------------------------------------------------------------
     //  Declare our friends
     // -----------------------------------------------------------------------
     friend class RefHash2KeysTableOfEnumerator<TVal>;
 
+    
 private:
+    // -----------------------------------------------------------------------
+    //  Unimplemented constructors and operators
+    // -----------------------------------------------------------------------
+    RefHash2KeysTableOf(const RefHash2KeysTableOf<TVal>&);
+    RefHash2KeysTableOf<TVal>& operator=(const RefHash2KeysTableOf<TVal>&);
 
     // -----------------------------------------------------------------------
     //  Private methods
@@ -248,7 +239,9 @@ public :
     // -----------------------------------------------------------------------
     //  Constructors and Destructor
     // -----------------------------------------------------------------------
-    RefHash2KeysTableOfEnumerator(RefHash2KeysTableOf<TVal>* const toEnum, const bool adopt = false);
+    RefHash2KeysTableOfEnumerator(RefHash2KeysTableOf<TVal>* const toEnum
+        , const bool adopt = false
+        , MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager);
     virtual ~RefHash2KeysTableOfEnumerator();
 
 
@@ -263,8 +256,15 @@ public :
     //  New interface 
     // -----------------------------------------------------------------------
     void nextElementKey(void*&, int&);
+    void setPrimaryKey(const void* key);
 
 private :
+    // -----------------------------------------------------------------------
+    //  Unimplemented constructors and operators
+    // -----------------------------------------------------------------------
+    RefHash2KeysTableOfEnumerator(const RefHash2KeysTableOfEnumerator<TVal>&);
+    RefHash2KeysTableOfEnumerator<TVal>& operator=(const RefHash2KeysTableOfEnumerator<TVal>&);
+
     // -----------------------------------------------------------------------
     //  Private methods
     // -----------------------------------------------------------------------
@@ -288,11 +288,18 @@ private :
     //
     //  fToEnum
     //      The value array being enumerated.
+    //
+    //  fLockPrimaryKey
+    //      Indicates that we are requested to iterate over the secondary keys
+    //      associated with the given primary key
+    //
     // -----------------------------------------------------------------------
-    bool                                  fAdopted;
-    RefHash2KeysTableBucketElem<TVal>*         fCurElem;
-    unsigned int                          fCurHash;
-    RefHash2KeysTableOf<TVal>*                 fToEnum;
+    bool                                    fAdopted;
+    RefHash2KeysTableBucketElem<TVal>*      fCurElem;
+    unsigned int                            fCurHash;
+    RefHash2KeysTableOf<TVal>*              fToEnum;
+    MemoryManager* const                    fMemoryManager;
+    const void*                             fLockPrimaryKey;
 };
 
 XERCES_CPP_NAMESPACE_END
