@@ -1,61 +1,48 @@
 /*
- * The Apache Software License, Version 1.1
- *
- * Copyright (c) 1999-2000 The Apache Software Foundation.  All rights
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xerces" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written
- *    permission, please contact apache\@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation, and was
- * originally based on software copyright (c) 1999, International
- * Business Machines, Inc., http://www.ibm.com .  For more information
- * on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Copyright 1999-2004 The Apache Software Foundation.
+ * 
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 /*
  * $Log: RefHashTableOf.hpp,v $
+ * Revision 1.15  2004/09/08 13:56:23  peiyongz
+ * Apache License Version 2.0
+ *
+ * Revision 1.14  2004/03/01 15:03:08  peiyongz
+ * new getter: getHashModulus
+ *
+ * Revision 1.13  2004/01/29 11:48:46  cargilld
+ * Code cleanup changes to get rid of various compiler diagnostic messages.
+ *
+ * Revision 1.12  2003/12/17 00:18:35  cargilld
+ * Update to memory management so that the static memory manager (one used to call Initialize) is only for static data.
+ *
+ * Revision 1.11  2003/10/20 11:45:06  gareth
+ * Made enumerators inherit from XMemory.
+ *
+ * Revision 1.10  2003/05/18 14:02:05  knoaman
+ * Memory manager implementation: pass per instance manager.
+ *
+ * Revision 1.9  2003/05/16 21:36:59  knoaman
+ * Memory manager implementation: Modify constructors to pass in the memory manager.
+ *
+ * Revision 1.8  2003/05/15 19:04:35  knoaman
+ * Partial implementation of the configurable memory manager.
+ *
+ * Revision 1.7  2003/05/15 10:37:08  gareth
+ * Optimization. We now resize the hash when appropriate. Patch by Nathan Codding.
+ *
  * Revision 1.6  2002/11/04 15:22:04  tng
  * C++ Namespace Support.
  *
@@ -120,16 +107,14 @@
 #define REFHASHTABLEOF_HPP
 
 
-#include <xercesc/util/XercesDefs.hpp>
 #include <xercesc/util/HashBase.hpp>
 #include <xercesc/util/IllegalArgumentException.hpp>
 #include <xercesc/util/NoSuchElementException.hpp>
 #include <xercesc/util/RuntimeException.hpp>
-#include <xercesc/util/XMLExceptMsgs.hpp>
-#include <xercesc/util/XMLEnumerator.hpp>
 #include <xercesc/util/XMLString.hpp>
-#include <xercesc/util/HashBase.hpp>
 #include <xercesc/util/HashXMLCh.hpp>
+#include <xercesc/util/PlatformUtils.hpp>
+#include <xercesc/framework/MemoryManager.hpp>
 
 XERCES_CPP_NAMESPACE_BEGIN
 
@@ -145,33 +130,57 @@ template <class TVal> struct RefHashTableBucketElem;
 //  This should really be a nested class, but some of the compilers we
 //  have to support cannot deal with that!
 //
-template <class TVal> struct RefHashTableBucketElem
+template <class TVal> struct RefHashTableBucketElem : public XMemory
 {
     RefHashTableBucketElem(void* key, TVal* const value, RefHashTableBucketElem<TVal>* next)
 		: fData(value), fNext(next), fKey(key)
         {
         }
 
+    RefHashTableBucketElem(){};
+
     TVal*                           fData;
     RefHashTableBucketElem<TVal>*   fNext;
 	void*							fKey;
+
+private:
+    // -----------------------------------------------------------------------
+    //  Unimplemented constructors and operators
+    // -----------------------------------------------------------------------
+    RefHashTableBucketElem(const RefHashTableBucketElem<TVal>&);
+    RefHashTableBucketElem<TVal>& operator=(const RefHashTableBucketElem<TVal>&);
 };
 
 
-template <class TVal> class RefHashTableOf
+template <class TVal> class RefHashTableOf : public XMemory
 {
 public:
     // -----------------------------------------------------------------------
     //  Constructors and Destructor
     // -----------------------------------------------------------------------
 	// backwards compatability - default hasher is HashXMLCh
-    RefHashTableOf(const unsigned int modulus);
+    RefHashTableOf
+    (
+        const unsigned int modulus
+        , MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager
+    );
 	// backwards compatability - default hasher is HashXMLCh
-    RefHashTableOf(const unsigned int modulus, const bool adoptElems);
+    RefHashTableOf
+    (
+        const unsigned int modulus
+        , const bool adoptElems
+        , MemoryManager* const manager =  XMLPlatformUtils::fgMemoryManager
+    );
 	// if a hash function is passed in, it will be deleted when the hashtable is deleted.
 	// use a new instance of the hasher class for each hashtable, otherwise one hashtable
 	// may delete the hasher of a different hashtable if both use the same hasher.
-    RefHashTableOf(const unsigned int modulus, const bool adoptElems, HashBase* hashBase);
+    RefHashTableOf
+    (
+        const unsigned int modulus
+        , const bool adoptElems
+        , HashBase* hashBase
+        , MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager
+    );
     ~RefHashTableOf();
 
 
@@ -192,6 +201,8 @@ public:
     // -----------------------------------------------------------------------
     TVal* get(const void* const key);
     const TVal* get(const void* const key) const;
+    MemoryManager* getMemoryManager() const;   
+    unsigned int   getHashModulus()   const;
 
     // -----------------------------------------------------------------------
     //  Setters
@@ -212,6 +223,11 @@ private :
     friend class RefHashTableOfEnumerator<TVal>;
 
 private:
+    // -----------------------------------------------------------------------
+    //  Unimplemented constructors and operators
+    // -----------------------------------------------------------------------
+    RefHashTableOf(const RefHashTableOf<TVal>&);
+    RefHashTableOf<TVal>& operator=(const RefHashTableOf<TVal>&);
 
     // -----------------------------------------------------------------------
     //  Private methods
@@ -220,6 +236,7 @@ private:
     const RefHashTableBucketElem<TVal>* findBucketElem(const void* const key, unsigned int& hashVal) const;
     void removeBucketElem(const void* const key, unsigned int& hashVal);
     void initialize(const unsigned int modulus);
+    void rehash();
 
 
     // -----------------------------------------------------------------------
@@ -241,10 +258,13 @@ private:
 	//  fHash
 	//      The hasher for the key data type.
     // -----------------------------------------------------------------------
-    bool                                fAdoptedElems;
-    RefHashTableBucketElem<TVal>**      fBucketList;
-    unsigned int                        fHashModulus;
-	HashBase*							fHash;
+    MemoryManager*                 fMemoryManager;
+    bool                           fAdoptedElems;
+    RefHashTableBucketElem<TVal>** fBucketList;
+    unsigned int                   fHashModulus;
+    unsigned int                   fInitialModulus;
+    unsigned int                   fCount;
+    HashBase*                      fHash;
 };
 
 
@@ -253,16 +273,18 @@ private:
 //  An enumerator for a value array. It derives from the basic enumerator
 //  class, so that value vectors can be generically enumerated.
 //
-template <class TVal> class RefHashTableOfEnumerator : public XMLEnumerator<TVal>
+template <class TVal> class RefHashTableOfEnumerator : public XMLEnumerator<TVal>, public XMemory
 {
 public :
     // -----------------------------------------------------------------------
     //  Constructors and Destructor
     // -----------------------------------------------------------------------
-    RefHashTableOfEnumerator(RefHashTableOf<TVal>* const toEnum, const bool adopt = false);
+    RefHashTableOfEnumerator(RefHashTableOf<TVal>* const toEnum
+        , const bool adopt = false
+        , MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager);
     virtual ~RefHashTableOfEnumerator();
 
-
+    RefHashTableOfEnumerator(const RefHashTableOfEnumerator<TVal>&);
     // -----------------------------------------------------------------------
     //  Enum interface
     // -----------------------------------------------------------------------
@@ -276,6 +298,11 @@ public :
     void* nextElementKey();
 
 private :
+    // -----------------------------------------------------------------------
+    //  Unimplemented constructors and operators
+    // -----------------------------------------------------------------------
+    RefHashTableOfEnumerator<TVal>& operator=(const RefHashTableOfEnumerator<TVal>&);
+
     // -----------------------------------------------------------------------
     //  Private methods
     // -----------------------------------------------------------------------
@@ -304,6 +331,7 @@ private :
     RefHashTableBucketElem<TVal>*         fCurElem;
     unsigned int                          fCurHash;
     RefHashTableOf<TVal>*                 fToEnum;
+    MemoryManager* const                  fMemoryManager;
 };
 
 XERCES_CPP_NAMESPACE_END
@@ -313,3 +341,4 @@ XERCES_CPP_NAMESPACE_END
 #endif
 
 #endif
+
