@@ -1,61 +1,39 @@
 /*
- * The Apache Software License, Version 1.1
- *
- * Copyright (c) 1999-2001 The Apache Software Foundation.  All rights
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xerces" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written
- *    permission, please contact apache\@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation, and was
- * originally based on software copyright (c) 1999, International
- * Business Machines, Inc., http://www.ibm.com .  For more information
- * on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Copyright 1999-2004 The Apache Software Foundation.
+ * 
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 /*
  * $Log: StringPool.hpp,v $
+ * Revision 1.8  2004/09/08 13:56:23  peiyongz
+ * Apache License Version 2.0
+ *
+ * Revision 1.7  2003/10/29 16:18:41  peiyongz
+ * Implement serialization/deserialization
+ *
+ * Revision 1.6  2003/10/09 13:49:30  neilg
+ * make StringPool functions virtual so that we can implement a synchronized version of StringPool for thread-safe updates.
+ *
+ * Revision 1.5  2003/05/16 06:01:52  knoaman
+ * Partial implementation of the configurable memory manager.
+ *
+ * Revision 1.4  2003/05/15 19:07:45  knoaman
+ * Partial implementation of the configurable memory manager.
+ *
+ * Revision 1.3  2003/03/07 18:11:54  tng
+ * Return a reference instead of void for operator=
+ *
  * Revision 1.2  2002/11/04 15:22:04  tng
  * C++ Namespace Support.
  *
@@ -87,6 +65,8 @@
 
 #include <xercesc/util/RefHashTableOf.hpp>
 
+#include <xercesc/internal/XSerializable.hpp>
+
 XERCES_CPP_NAMESPACE_BEGIN
 
 //
@@ -102,7 +82,7 @@ XERCES_CPP_NAMESPACE_BEGIN
 //  other than flushing it completely, and because ids are assigned
 //  sequentially from 1.
 //
-class XMLUTIL_EXPORT XMLStringPool
+class XMLUTIL_EXPORT XMLStringPool : public XSerializable, public XMemory
 {
 public :
     // -----------------------------------------------------------------------
@@ -110,30 +90,40 @@ public :
     // -----------------------------------------------------------------------
     XMLStringPool
     (
-        const   unsigned int    modulus = 109
+          const unsigned int   modulus = 109
+        , MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager
     );
-    ~XMLStringPool();
+    virtual ~XMLStringPool();
 
 
     // -----------------------------------------------------------------------
     //  Pool management methods
     // -----------------------------------------------------------------------
-    unsigned int addOrFind(const XMLCh* const newString);
-    bool exists(const XMLCh* const newString) const;
-    bool exists(const unsigned int id) const;
-    void flushAll();
-    unsigned int getId(const XMLCh* const toFind) const;
-    const XMLCh* getValueForId(const unsigned int id) const;
+    virtual unsigned int addOrFind(const XMLCh* const newString);
+    virtual bool exists(const XMLCh* const newString) const;
+    virtual bool exists(const unsigned int id) const;
+    virtual void flushAll();
+    virtual unsigned int getId(const XMLCh* const toFind) const;
+    virtual const XMLCh* getValueForId(const unsigned int id) const;
+    virtual unsigned int getStringCount() const;
 
+    /***
+     * Support for Serialization/De-serialization
+     ***/
+    DECL_XSERIALIZABLE(XMLStringPool)
+
+    XMLStringPool(MemoryManager* const manager);
 
 private :
     // -----------------------------------------------------------------------
     //  Private data types
     // -----------------------------------------------------------------------
-    class PoolElem
+    class PoolElem : public XMemory
     {
         public :
-            PoolElem(const XMLCh* const string, const unsigned int id);
+            PoolElem(const XMLCh* const string,
+                     const unsigned int id,
+                     MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager);
             ~PoolElem();
 
             inline const XMLCh* getKey() const { return fString; }
@@ -141,7 +131,7 @@ private :
 
             unsigned int    fId;
             XMLCh*          fString;
-
+            MemoryManager*  fMemoryManager;
     };
 
 
@@ -149,7 +139,7 @@ private :
     //  Unimplemented constructors and operators
     // -----------------------------------------------------------------------
     XMLStringPool(const XMLStringPool&);
-    void operator=(const XMLStringPool&);
+    XMLStringPool& operator=(const XMLStringPool&);
 
 
     // -----------------------------------------------------------------------
@@ -174,13 +164,17 @@ private :
     //      The current capacity of the id map. When the current id hits this
     //      value the map must must be expanded.
     //
-    //  fCurId
-    //      This is the counter used to assign unique ids. It is just bumped
-    //      up one for each new string added.
     // -----------------------------------------------------------------------
+    MemoryManager*              fMemoryManager;
     PoolElem**                  fIdMap;
     RefHashTableOf<PoolElem>*   fHashTable;
     unsigned int                fMapCapacity;
+
+protected:
+    // protected data members
+    //  fCurId
+    //      This is the counter used to assign unique ids. It is just bumped
+    //      up one for each new string added.
     unsigned int                fCurId;
 };
 
