@@ -1,61 +1,55 @@
 /*
- * The Apache Software License, Version 1.1
- *
- * Copyright (c) 2002 The Apache Software Foundation.  All rights
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xerces" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written
- *    permission, please contact apache\@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation, and was
- * originally based on software copyright (c) 2001, International
- * Business Machines, Inc., http://www.ibm.com .  For more information
- * on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Copyright 2002,2004 The Apache Software Foundation.
+ * 
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 /*
  * $Log: IconvGNUTransService.cpp,v $
+ * Revision 1.15  2004/09/08 13:56:45  peiyongz
+ * Apache License Version 2.0
+ *
+ * Revision 1.14  2004/07/23 15:29:09  amassari
+ * transcode was badly terminating the converted string (jira#1206)
+ *
+ * Revision 1.13  2004/07/23 14:35:03  amassari
+ * A global mutex was not cleaned up
+ *
+ * Revision 1.12  2004/02/25 14:53:24  peiyongz
+ * Bug#27209: Xerces 2.5.0 does not build with option -t IconvGNU because of syntax errors!
+ *
+ * Revision 1.11  2003/12/24 15:24:15  cargilld
+ * More updates to memory management so that the static memory manager.
+ *
+ * Revision 1.10  2003/08/19 14:01:41  neilg
+ * fix for bug 22537
+ *
+ * Revision 1.9  2003/05/17 16:32:18  knoaman
+ * Memory manager implementation : transcoder update.
+ *
+ * Revision 1.8  2003/05/16 21:37:00  knoaman
+ * Memory manager implementation: Modify constructors to pass in the memory manager.
+ *
+ * Revision 1.7  2003/05/15 18:47:05  knoaman
+ * Partial implementation of the configurable memory manager.
+ *
+ * Revision 1.6  2003/04/07 16:52:13  peiyongz
+ * Bug# 18672: IconvGNUTranscoder can't be build when namespaces is on.
+ *                       Patch from Bacek@yandex-team.ru (Vasily Tchekalkin)
+ *
+ * Revision 1.5  2003/03/09 17:03:25  peiyongz
+ * PanicHandler
+ *
  * Revision 1.4  2002/12/31 18:42:54  tng
  * [Bug 15608] IconvLCPTranscoder::transcode() is wrong at wcstombs() usage.
  *
@@ -80,8 +74,33 @@
 #include <errno.h>
 #include <endian.h>
 
+#include <xercesc/util/XMLString.hpp>
+#include <xercesc/util/XMLUniDefs.hpp>
+#include <xercesc/util/XMLUni.hpp>
+#include <xercesc/util/PlatformUtils.hpp>
+#include <xercesc/util/TranscodingException.hpp>
+#include "IconvGNUTransService.hpp"
+
+#if !defined(APP_NO_THREADS)
+#include <xercesc/util/Mutexes.hpp>
+#include <xercesc/util/XMLRegisterCleanup.hpp>
+#endif /* !APP_NO_THREADS */
+
 XERCES_CPP_NAMESPACE_BEGIN
 
+#if !defined(APP_NO_THREADS)
+
+// Iconv() access syncronization point
+static XMLMutex    *gIconvMutex = NULL;
+static XMLRegisterCleanup IconvGNUMutexCleanup;
+#  define ICONV_LOCK    XMLMutexLock lockConverter(gIconvMutex);
+
+#else /* APP_NO_THREADS */
+
+# define ICONV_LOCK
+
+#endif /* !APP_NO_THREADS */
+
 // ---------------------------------------------------------------------------
 // Description of encoding schemas, supported by iconv()
 // ---------------------------------------------------------------------------
@@ -161,13 +180,6 @@ static const IconvGNUEncoding    gIconvGNUEncodings[] = {
 #include <wchar.h>
 
 
-#include <xercesc/util/XMLString.hpp>
-#include <xercesc/util/XMLUniDefs.hpp>
-#include <xercesc/util/XMLUni.hpp>
-#include <xercesc/util/PlatformUtils.hpp>
-#include <xercesc/util/TranscodingException.hpp>
-#include "IconvGNUTransService.hpp"
-
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -198,19 +210,6 @@ static unsigned int getWideCharLength(const XMLCh* const src)
 }
 
 
-#if !defined(APP_NO_THREADS)
-
-#include <xercesc/util/Mutexes.hpp>
-// Iconv() access syncronization point
-static XMLMutex    *gIconvMutex = NULL;
-#  define ICONV_LOCK    XMLMutexLock lockConverter(gIconvMutex);
-
-#else /* APP_NO_THREADS */
-
-# define ICONV_LOCK
-
-#endif /* !APP_NO_THREADS */
-
 //----------------------------------------------------------------------------
 // There is implementation of the libiconv for FreeBSD (available through the
 // ports collection). The following is a wrapper around the iconv().
@@ -230,7 +229,7 @@ IconvGNUWrapper::IconvGNUWrapper ( iconv_t    cd_from,
       fCDTo(cd_to), fCDFrom(cd_from)
 {
     if (fCDFrom == (iconv_t) -1 || fCDTo == (iconv_t) -1) {
-    XMLPlatformUtils::panic (XMLPlatformUtils::Panic_NoTransService);
+    XMLPlatformUtils::panic (PanicHandler::Panic_NoTransService);
     }
 }
 
@@ -474,6 +473,12 @@ size_t    IconvGNUWrapper::iconvTo ( const char    *fromPtr,
 //  IconvGNUTransService: Constructors and Destructor
 // ---------------------------------------------------------------------------
 
+void reinitIconvGNUMutex()
+{
+    delete gIconvMutex;
+    gIconvMutex = 0;
+}
+
 IconvGNUTransService::IconvGNUTransService()
     : IconvGNUWrapper(), fUnicodeCP(0)
 {
@@ -482,7 +487,8 @@ IconvGNUTransService::IconvGNUTransService()
     if (gIconvMutex == NULL) {
         gIconvMutex = new XMLMutex;
         if (gIconvMutex == NULL)
-            XMLPlatformUtils::panic (XMLPlatformUtils::Panic_NoTransService);
+            XMLPlatformUtils::panic (PanicHandler::Panic_NoTransService);
+        IconvGNUMutexCleanup.registerCleanup(reinitIconvGNUMutex);
     }
 #endif
 
@@ -545,7 +551,7 @@ IconvGNUTransService::IconvGNUTransService()
         }
 
     if (fUnicodeCP == NULL || cdTo() == (iconv_t)-1 || cdFrom() == (iconv_t)-1)
-        XMLPlatformUtils::panic (XMLPlatformUtils::Panic_NoTransService);
+        XMLPlatformUtils::panic (PanicHandler::Panic_NoTransService);
 }
 
 IconvGNUTransService::~IconvGNUTransService()
@@ -648,12 +654,13 @@ IconvGNUTransService::makeNewXMLTranscoder
     const    XMLCh* const    encodingName
     ,    XMLTransService::Codes&    resValue
     , const    unsigned int    blockSize
+    ,        MemoryManager* const    manager
 )
 {
     resValue = XMLTransService::UnsupportedEncoding;
     IconvGNUTranscoder    *newTranscoder = NULL;
 
-    char    *encLocal = XMLString::transcode(encodingName);
+    char    *encLocal = XMLString::transcode(encodingName, manager);
     iconv_t    cd_from, cd_to;
 
     {
@@ -662,7 +669,7 @@ IconvGNUTransService::makeNewXMLTranscoder
         if (cd_from == (iconv_t)-1) {
             resValue = XMLTransService::SupportFilesNotFound;
             if (encLocal)
-                delete [] encLocal;
+                manager->deallocate(encLocal);//delete [] encLocal;
             return NULL;
         }
         cd_to = iconv_open (encLocal, fUnicodeCP);
@@ -670,18 +677,18 @@ IconvGNUTransService::makeNewXMLTranscoder
             resValue = XMLTransService::SupportFilesNotFound;
             iconv_close (cd_from);
             if (encLocal)
-                delete [] encLocal;
+                manager->deallocate(encLocal);//delete [] encLocal;
             return NULL;
         }
-        newTranscoder = new IconvGNUTranscoder (encodingName,
+        newTranscoder = new (manager) IconvGNUTranscoder (encodingName,
                              blockSize,
                              cd_from, cd_to,
-                             uChSize(), UBO());
+                             uChSize(), UBO(), manager);
     }
     if (newTranscoder)
         resValue = XMLTransService::Ok;
     if (encLocal)
-        delete [] encLocal;
+        manager->deallocate(encLocal);//delete [] encLocal;
     return newTranscoder;
 }
 
@@ -709,7 +716,8 @@ void IconvGNUTransService::lowerCase(XMLCh* const toLowerCase) const
 //  IconvGNULCPTranscoder: The virtual transcoder API
 // ---------------------------------------------------------------------------
 unsigned int
-IconvGNULCPTranscoder::calcRequiredSize (const char* const srcText)
+IconvGNULCPTranscoder::calcRequiredSize (const char* const srcText
+                                         , MemoryManager* const manager)
 {
     if (!srcText)
         return 0;
@@ -727,7 +735,7 @@ IconvGNULCPTranscoder::calcRequiredSize (const char* const srcText)
         const char    *ptr = srcText + srcLen - len;
         size_t    rc = iconvFrom(ptr, &len, &pTmpArr, gTempBuffArraySize);
         if (rc == (size_t) -1 && errno != E2BIG) {
-            ThrowXML(TranscodingException, XMLExcepts::Trans_BadSrcSeq);
+            ThrowXMLwithMemMgr(TranscodingException, XMLExcepts::Trans_BadSrcSeq, manager);
             /* return 0; */
         }
         rc = pTmpArr - (char *) tmpWideArr;
@@ -740,7 +748,8 @@ IconvGNULCPTranscoder::calcRequiredSize (const char* const srcText)
 
 
 unsigned int
-IconvGNULCPTranscoder::calcRequiredSize(const XMLCh* const srcText)
+IconvGNULCPTranscoder::calcRequiredSize(const XMLCh* const srcText
+                                        , MemoryManager* const manager)
 {
     if (!srcText)
         return 0;
@@ -754,7 +763,10 @@ IconvGNULCPTranscoder::calcRequiredSize(const XMLCh* const srcText)
     size_t      len = wLent * uChSize();
     if (uChSize() != sizeof(XMLCh) || UBO() != BYTE_ORDER) {
         if (len > gTempBuffArraySize) {
-            wBufPtr = new char[len];
+            wBufPtr = (char*) manager->allocate
+            (
+                len * sizeof(char)
+            );//new char[len];
             if (wBufPtr == NULL)
             return 0;
             wBuf = wBufPtr;
@@ -774,8 +786,8 @@ IconvGNULCPTranscoder::calcRequiredSize(const XMLCh* const srcText)
         size_t    rc = iconvTo(ptr, &len, &pTmpArr, gTempBuffArraySize);
         if (rc == (size_t) -1 && errno != E2BIG) {
             if (wBufPtr)
-                delete [] wBufPtr;
-            ThrowXML(TranscodingException, XMLExcepts::Trans_BadSrcSeq);
+                manager->deallocate(wBufPtr);//delete [] wBufPtr;
+            ThrowXMLwithMemMgr(TranscodingException, XMLExcepts::Trans_BadSrcSeq, manager);
             /* return 0; */
         }
         rc = pTmpArr - tmpBuff;
@@ -784,7 +796,7 @@ IconvGNULCPTranscoder::calcRequiredSize(const XMLCh* const srcText)
             break;
     }
     if (wBufPtr)
-        delete [] wBufPtr;
+        manager->deallocate(wBufPtr);//delete [] wBufPtr;
     return totalLen;
 }
 
@@ -846,10 +858,69 @@ char* IconvGNULCPTranscoder::transcode(const XMLCh* const toTranscode)
     return retVal;
 }
 
+char* IconvGNULCPTranscoder::transcode(const XMLCh* const toTranscode,
+                                       MemoryManager* const manager)
+{
+    if (!toTranscode)
+        return 0;
+
+    char* retVal = 0;
+    if (*toTranscode) {
+        unsigned int  wLent = getWideCharLength(toTranscode);
+
+        // Calc needed size.
+        const size_t neededLen = calcRequiredSize (toTranscode, manager);
+        if (neededLen == 0)
+            return 0;
+        // allocate output buffer
+        retVal = (char*) manager->allocate((neededLen + 1) * sizeof(char));//new char[neededLen + 1];
+        if (retVal == NULL)
+            return 0;
+        // prepare the original
+        char    tmpWBuff[gTempBuffArraySize];
+        char    *wideCharBuf = 0;
+        char    *wBufPtr = 0;
+        size_t  len = wLent * uChSize();
+
+        if (uChSize() != sizeof(XMLCh) || UBO() != BYTE_ORDER) {
+            if (len > gTempBuffArraySize) {
+                wBufPtr = (char*) manager->allocate(len * sizeof(char));//new char[len];
+                if (wBufPtr == NULL)
+                    return 0;
+                wideCharBuf = wBufPtr;
+            } else
+                wideCharBuf = tmpWBuff;
+            xmlToMbs (toTranscode, wLent, wideCharBuf, wLent);
+        } else
+            wideCharBuf = (char *) toTranscode;
+
+        // perform conversion
+        wLent *= uChSize();
+        char    *ptr = retVal;
+        size_t    rc = iconvTo(wideCharBuf, (size_t *) &wLent, &ptr, neededLen);
+        if (rc == (size_t)-1) {
+            if (wBufPtr)
+            manager->deallocate(wBufPtr);//delete [] wBufPtr;
+            return 0;
+        }
+        if (wBufPtr)
+            manager->deallocate(wBufPtr);//delete [] wBufPtr;
+        retVal[neededLen] = 0;
+
+    } else {
+        retVal = (char*) manager->allocate(sizeof(char));//new char[1];
+        if (retVal == NULL)
+            return 0;
+        retVal[0] = 0;
+    }
+    return retVal;
+}
+
 
 bool IconvGNULCPTranscoder::transcode( const   XMLCh* const    toTranscode
                     , char* const        toFill
-                    , const unsigned int    maxBytes)
+                    , const unsigned int    maxBytes
+                    , MemoryManager* const  manager)
 {
     // Watch for a couple of pyscho corner cases
     if (!toTranscode || !maxBytes) {
@@ -873,7 +944,10 @@ bool IconvGNULCPTranscoder::transcode( const   XMLCh* const    toTranscode
 
     if (uChSize() != sizeof(XMLCh) || UBO() != BYTE_ORDER) {
         if (len > gTempBuffArraySize) {
-            wBufPtr = new char[len];
+            wBufPtr = (char*) manager->allocate
+            (
+                len * sizeof(char)
+            );//new char[len];
             if (wBufPtr == NULL)
                 return 0;
         wideCharBuf = wBufPtr;
@@ -888,14 +962,14 @@ bool IconvGNULCPTranscoder::transcode( const   XMLCh* const    toTranscode
     size_t    rc = iconvTo(wideCharBuf, &len, &ptr, maxBytes);
     if (rc == (size_t)-1) {
         if (wBufPtr)
-            delete [] wBufPtr;
+            manager->deallocate(wBufPtr);//delete [] wBufPtr;
         return false;
     }
     if (wBufPtr)
-        delete [] wBufPtr;
+        manager->deallocate(wBufPtr);//delete [] wBufPtr;
 
-    // Cap it off just in case
-    toFill[rc] = 0;
+    // Cap it off
+    *ptr = 0;
     return true;
 }
 
@@ -957,10 +1031,68 @@ XMLCh* IconvGNULCPTranscoder::transcode(const char* const toTranscode)
     return retVal;
 }
 
+XMLCh* IconvGNULCPTranscoder::transcode(const char* const toTranscode,
+                                        MemoryManager* const manager)
+{
+    if (!toTranscode)
+        return 0;
+
+    XMLCh* retVal = 0;
+    if (*toTranscode) {
+        const unsigned int wLent = calcRequiredSize(toTranscode, manager);
+        if (wLent == 0) {
+            retVal = (XMLCh*) manager->allocate(sizeof(XMLCh));//new XMLCh[1];
+            retVal[0] = 0;
+            return retVal;
+        }
+
+        char    tmpWBuff[gTempBuffArraySize];
+        char    *wideCharBuf = 0;
+        char    *wBufPtr = 0;
+        size_t  len = wLent * uChSize();
+
+        retVal = (XMLCh*) manager->allocate((wLent + 1) * sizeof(XMLCh));//new XMLCh[wLent + 1];
+        if (retVal == NULL)
+            return NULL;
+        if (uChSize() != sizeof(XMLCh) || UBO() != BYTE_ORDER) {
+            if (len > gTempBuffArraySize) {
+                wBufPtr = (char*) manager->allocate(len * sizeof(char));//new char[len];
+                if (wBufPtr == NULL)
+                    return 0;
+                wideCharBuf = wBufPtr;
+            } else
+                wideCharBuf = tmpWBuff;
+        } else
+            wideCharBuf = (char *) retVal;
+
+        size_t    flen = strlen(toTranscode);
+        char    *ptr = wideCharBuf;
+        size_t    rc = iconvFrom(toTranscode, &flen, &ptr, len);
+        if (rc == (size_t) -1) {
+            if (wBufPtr)
+            manager->deallocate(wBufPtr);//delete [] wBufPtr;
+            return NULL;
+        }
+        if (uChSize() != sizeof(XMLCh) || UBO() != BYTE_ORDER)
+            mbsToXML (wideCharBuf, wLent, retVal, wLent);
+        if (wBufPtr)
+            manager->deallocate(wBufPtr);//delete [] wBufPtr;
+        retVal[wLent] = 0x00;
+    }
+    else {
+        retVal = (XMLCh*) manager->allocate(sizeof(XMLCh));//new XMLCh[1];
+        if (retVal == NULL )
+            return 0;
+        retVal[0] = 0;
+    }
+    return retVal;
+}
+
 
 bool IconvGNULCPTranscoder::transcode(const   char* const    toTranscode
                        ,       XMLCh* const    toFill
-                       , const unsigned int    maxChars)
+                       , const unsigned int    maxChars
+                       , MemoryManager* const  manager)
 {
     // Check for a couple of psycho corner cases
     if (!toTranscode || !maxChars)
@@ -986,7 +1118,10 @@ bool IconvGNULCPTranscoder::transcode(const   char* const    toTranscode
 
     if (uChSize() != sizeof(XMLCh) || UBO() != BYTE_ORDER) {
         if (len > gTempBuffArraySize) {
-            wBufPtr = new char[len];
+            wBufPtr = (char*) manager->allocate
+            (
+                len * sizeof(char)
+            );//new char[len];
             if (wBufPtr == NULL)
                 return 0;
             wideCharBuf = wBufPtr;
@@ -1000,14 +1135,14 @@ bool IconvGNULCPTranscoder::transcode(const   char* const    toTranscode
     size_t    rc = iconvFrom(toTranscode, &flen, &ptr, len);
     if (rc == (size_t)-1) {
         if (wBufPtr)
-            delete [] wBufPtr;
+            manager->deallocate(wBufPtr);//delete [] wBufPtr;
         return false;
     }
 
     if (uChSize() != sizeof(XMLCh) || UBO() != BYTE_ORDER)
         mbsToXML (wideCharBuf, wLent, toFill, wLent);
     if (wBufPtr)
-        delete [] wBufPtr;
+        manager->deallocate(wBufPtr);//delete [] wBufPtr;
 
     toFill[wLent] = 0x00;
     return true;
@@ -1042,8 +1177,9 @@ IconvGNUTranscoder::IconvGNUTranscoder (const    XMLCh* const    encodingName
                       ,    iconv_t        cd_to
                       ,    size_t        uchsize
                       ,    unsigned int    ubo
+                      , MemoryManager* const manager
     )
-    : XMLTranscoder(encodingName, blockSize)
+    : XMLTranscoder(encodingName, blockSize, manager)
     , IconvGNUWrapper (cd_from, cd_to, uchsize, ubo)
 {
 }
@@ -1084,7 +1220,10 @@ unsigned int    IconvGNUTranscoder::transcodeFrom
 
     if (uChSize() != sizeof(XMLCh) || UBO() != BYTE_ORDER) {
         if (len > gTempBuffArraySize) {
-            wBufPtr = new char[len];
+            wBufPtr = (char*) getMemoryManager()->allocate
+            (
+                len * sizeof(char)
+            );//new char[len];
             if (wBufPtr == NULL)
                 return 0;
             startTarget = wBufPtr;
@@ -1104,8 +1243,8 @@ unsigned int    IconvGNUTranscoder::transcodeFrom
         if (rc == (size_t)-1) {
             if (errno != E2BIG || prevSrcLen == srcLen) {
                 if (wBufPtr)
-                    delete [] wBufPtr;
-                ThrowXML(TranscodingException, XMLExcepts::Trans_BadSrcSeq);
+                    getMemoryManager()->deallocate(wBufPtr);//delete [] wBufPtr;
+                ThrowXMLwithMemMgr(TranscodingException, XMLExcepts::Trans_BadSrcSeq, getMemoryManager());
             }
         }
         charSizes[cnt] = prevSrcLen - srcLen;
@@ -1117,7 +1256,7 @@ unsigned int    IconvGNUTranscoder::transcodeFrom
     if (uChSize() != sizeof(XMLCh) || UBO() != BYTE_ORDER)
         mbsToXML (startTarget, toReturn, toFill, toReturn);
     if (wBufPtr)
-        delete [] wBufPtr;
+        getMemoryManager()->deallocate(wBufPtr);//delete [] wBufPtr;
     return toReturn;
 }
 
@@ -1138,7 +1277,10 @@ unsigned int    IconvGNUTranscoder::transcodeTo
 
     if (uChSize() != sizeof(XMLCh) || UBO() != BYTE_ORDER) {
         if (len > gTempBuffArraySize) {
-            wBufPtr = new char[len];
+            wBufPtr = (char*) getMemoryManager()->allocate
+            (
+                len * sizeof(char)
+            );//new char[len];
             if (wBufPtr == NULL)
                 return 0;
             startSrc = wBufPtr;
@@ -1153,12 +1295,12 @@ unsigned int    IconvGNUTranscoder::transcodeTo
     size_t    rc = iconvTo (startSrc, &srcLen, &startTarget, maxBytes);
     if (rc == (size_t)-1 && errno != E2BIG) {
         if (wBufPtr)
-            delete [] wBufPtr;
-        ThrowXML(TranscodingException, XMLExcepts::Trans_BadSrcSeq);
+            getMemoryManager()->deallocate(wBufPtr);//delete [] wBufPtr;
+        ThrowXMLwithMemMgr(TranscodingException, XMLExcepts::Trans_BadSrcSeq, getMemoryManager());
     }
     charsEaten = srcCount - srcLen / uChSize();
     if (wBufPtr)
-        delete [] wBufPtr;
+        getMemoryManager()->deallocate(wBufPtr);//delete [] wBufPtr;
     return startTarget - (char *)toFill;
 }
 
