@@ -1,62 +1,58 @@
 /*
- * The Apache Software License, Version 1.1
- *
- * Copyright (c) 2001 The Apache Software Foundation.  All rights
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xerces" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written
- *    permission, please contact apache\@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation, and was
- * originally based on software copyright (c) 2001, International
- * Business Machines, Inc., http://www.ibm.com .  For more information
- * on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Copyright 2001-2004 The Apache Software Foundation.
+ * 
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 /*
- * $Id: XMLDateTime.cpp,v 1.16 2003/09/25 22:24:28 peiyongz Exp $
+ * $Id: XMLDateTime.cpp,v 1.28 2004/09/08 13:56:24 peiyongz Exp $
  * $Log: XMLDateTime.cpp,v $
+ * Revision 1.28  2004/09/08 13:56:24  peiyongz
+ * Apache License Version 2.0
+ *
+ * Revision 1.27  2004/09/02 15:03:50  peiyongz
+ * To build on AIX
+ *
+ * Revision 1.26  2004/08/31 20:50:50  peiyongz
+ * Parse/keep milisecond as double to retain precision.
+ *
+ * Revision 1.25  2004/01/29 11:48:47  cargilld
+ * Code cleanup changes to get rid of various compiler diagnostic messages.
+ *
+ * Revision 1.24  2004/01/25 23:23:26  jberry
+ * Step around CodeWarrior compiler warning
+ *
+ * Revision 1.23  2004/01/13 19:50:56  peiyongz
+ * remove parseContent()
+ *
+ * Revision 1.22  2004/01/13 16:34:20  cargilld
+ * Misc memory management changes.
+ *
+ * Revision 1.21  2004/01/03 00:03:18  peiyongz
+ * parseContent
+ *
+ * Revision 1.20  2003/12/31 02:34:11  neilg
+ * enable production of canonical representations for dates with negative years, or years >9999
+ *
+ * Revision 1.19  2003/12/17 00:18:35  cargilld
+ * Update to memory management so that the static memory manager (one used to call Initialize) is only for static data.
+ *
+ * Revision 1.18  2003/12/16 22:48:52  peiyongz
+ * exception thrown upon invalid number, thanks Gareth Reakes.
+ *
+ * Revision 1.17  2003/12/11 21:38:12  peiyongz
+ * support for Canonical Representation for Datatype
+ *
  * Revision 1.16  2003/09/25 22:24:28  peiyongz
  * Using writeString/readString
  *
@@ -125,10 +121,14 @@
 //  Includes
 // ---------------------------------------------------------------------------
 #include <stdlib.h>
+#include <assert.h>
+#include <errno.h>
+
 #include <xercesc/util/XMLDateTime.hpp>
 #include <xercesc/util/XMLString.hpp>
 #include <xercesc/util/XMLUni.hpp>
 #include <xercesc/util/Janitor.hpp>
+#include <xercesc/util/NumberFormatException.hpp>
 
 XERCES_CPP_NAMESPACE_BEGIN
 
@@ -376,8 +376,8 @@ void XMLDateTime::addDuration(XMLDateTime*             fNewDate
     fNewDate->fValue[utc] = UTC_STD;
 }
 
-int XMLDateTime::compareResult(short resultA
-                             , short resultB
+int XMLDateTime::compareResult(int resultA
+                             , int resultB
                              , bool strict)
 {
 
@@ -420,7 +420,7 @@ int XMLDateTime::compare(const XMLDateTime* const pDate1
         return XMLDateTime::compareOrder(pDate1, pDate2);
     }
 
-    short c1, c2;
+    int c1, c2;
 
     if ( pDate1->isNormalized())
     {
@@ -456,6 +456,7 @@ int XMLDateTime::compareResult(const XMLDateTime* const pDate1
 
 int XMLDateTime::compareOrder(const XMLDateTime* const lValue
                             , const XMLDateTime* const rValue)
+                            //, MemoryManager* const memMgr)
 {
     //
     // If any of the them is not normalized() yet,
@@ -479,6 +480,18 @@ int XMLDateTime::compareOrder(const XMLDateTime* const lValue
         }
     }
 
+    if ( lTemp.fHasTime)
+    {
+        if ( lTemp.fMiliSecond < rTemp.fMiliSecond )
+        {
+            return LESS_THAN;
+        }
+        else if ( lTemp.fMiliSecond > rTemp.fMiliSecond )
+        {
+            return GREATER_THAN;
+        }
+    }
+
     return EQUAL;
 }
 
@@ -486,9 +499,9 @@ int XMLDateTime::compareOrder(const XMLDateTime* const lValue
 //  ctor and dtor
 // ---------------------------------------------------------------------------
 XMLDateTime::~XMLDateTime()
-{
+{    
     if (fBuffer)
-        fMemoryManager->deallocate(fBuffer);//delete[] fBuffer;
+        fMemoryManager->deallocate(fBuffer);//delete[] fBuffer;  
 }
 
 XMLDateTime::XMLDateTime(MemoryManager* const manager)
@@ -496,6 +509,8 @@ XMLDateTime::XMLDateTime(MemoryManager* const manager)
 , fEnd(0)
 , fBufferMaxLen(0)
 , fBuffer(0)
+, fMiliSecond(0)
+, fHasTime(false)
 , fMemoryManager(manager)
 {
     reset();
@@ -507,6 +522,8 @@ XMLDateTime::XMLDateTime(const XMLCh* const aString,
 , fEnd(0)
 , fBufferMaxLen(0)
 , fBuffer(0)
+, fMiliSecond(0)
+, fHasTime(false)
 , fMemoryManager(manager)
 {
     setBuffer(aString);
@@ -586,13 +603,15 @@ void XMLDateTime::parseDateTime()
 
     //fStart is supposed to point to 'T'
     if (fBuffer[fStart++] != DATETIME_SEPARATOR)
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_dt_missingT
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
 
     getTime();
     validateDateTime();
     normalize();
+    fHasTime = true;
 }
 
 //
@@ -620,6 +639,7 @@ void XMLDateTime::parseTime()
 
     validateDateTime();
     normalize();
+    fHasTime = true;
 }
 
 //
@@ -634,9 +654,10 @@ void XMLDateTime::parseDay()
         fBuffer[1] != DATE_SEPARATOR ||
         fBuffer[2] != DATE_SEPARATOR  )
     {
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_gDay_invalid
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
     }
 
     //initialize values
@@ -649,9 +670,10 @@ void XMLDateTime::parseDay()
         int sign = findUTCSign(DAY_SIZE);
         if ( sign < 0 )
         {
-            ThrowXML1(SchemaDateTimeException
+            ThrowXMLwithMemMgr1(SchemaDateTimeException
                     , XMLExcepts::DateTime_gDay_invalid
-                    , fBuffer);
+                    , fBuffer
+                    , fMemoryManager);
         }
         else
         {
@@ -675,9 +697,10 @@ void XMLDateTime::parseMonth()
     if (fBuffer[0] != DATE_SEPARATOR ||
         fBuffer[1] != DATE_SEPARATOR  )
     {
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_gMth_invalid
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
     }
 
     //set constants
@@ -702,9 +725,10 @@ void XMLDateTime::parseMonth()
         int sign = findUTCSign(fStart);
         if ( sign < 0 )
         {
-            ThrowXML1(SchemaDateTimeException
+            ThrowXMLwithMemMgr1(SchemaDateTimeException
                     , XMLExcepts::DateTime_gMth_invalid
-                    , fBuffer);
+                    , fBuffer
+                    , fMemoryManager);
         }
         else
         {
@@ -758,9 +782,10 @@ void XMLDateTime::parseMonthDay()
         fBuffer[1] != DATE_SEPARATOR ||
         fBuffer[4] != DATE_SEPARATOR )
     {
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_gMthDay_invalid
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
     }
 
 
@@ -774,9 +799,10 @@ void XMLDateTime::parseMonthDay()
         int sign = findUTCSign(MONTHDAY_SIZE);
         if ( sign<0 )
         {
-            ThrowXML1(SchemaDateTimeException
+            ThrowXMLwithMemMgr1(SchemaDateTimeException
                     , XMLExcepts::DateTime_gMthDay_invalid
-                    , fBuffer);
+                    , fBuffer
+                    , fMemoryManager);
         }
         else
         {
@@ -819,18 +845,20 @@ void XMLDateTime::parseDuration()
     if ( (c != DURATION_STARTER) &&
          (c != chDash)            )
     {
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_dur_Start_dashP
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
     }
 
     // 'P' must ALWAYS be present in either case
     if ( (c == chDash) &&
          (fBuffer[fStart++]!= DURATION_STARTER ))
     {
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_dur_noP
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
     }
 
     // java code
@@ -847,9 +875,10 @@ void XMLDateTime::parseDuration()
     //
     if (indexOf(fStart, fEnd, chDash) != NOT_FOUND)
     {
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_dur_DashNotFirst
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
     }
 
     //at least one number and designator must be seen after P
@@ -892,9 +921,10 @@ void XMLDateTime::parseDuration()
     if ( (fEnd == endDate) &&   // 'T' absent
          (fStart != fEnd)   )   // something after Day
     {
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_dur_inv_b4T
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
     }
 
     if ( fEnd != endDate ) // 'T' present
@@ -940,13 +970,14 @@ void XMLDateTime::parseDuration()
                  */
                 if ( mlsec+1 == end )
                 {
-                    ThrowXML1(SchemaDateTimeException
+                    ThrowXMLwithMemMgr1(SchemaDateTimeException
                             , XMLExcepts::DateTime_dur_inv_seconds
-                            ,fBuffer);
+                            , fBuffer
+                            , fMemoryManager);
                 }
 
                 fValue[Second]     = negate * parseInt(fStart, mlsec);
-                fValue[MiliSecond] = negate * parseInt(mlsec+1, end);
+                fMiliSecond        = negate * parseMiliSecond(mlsec+1, end);
             }
             else
             {
@@ -962,17 +993,19 @@ void XMLDateTime::parseDuration()
         if ( (fStart != fEnd) ||
               fBuffer[--fStart] == DATETIME_SEPARATOR )
         {
-            ThrowXML1(SchemaDateTimeException
+            ThrowXMLwithMemMgr1(SchemaDateTimeException
                     , XMLExcepts::DateTime_dur_NoTimeAfterT
-                    ,fBuffer);
+                    , fBuffer
+                    , fMemoryManager);
         }
     }
 
     if ( !designator )
     {
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_dur_NoElementAtAll
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
     }
 
 }
@@ -994,18 +1027,20 @@ void XMLDateTime::getDate()
 
     // Ensure enough chars in buffer
     if ( (fStart+YMD_MIN_SIZE) > fEnd)
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_date_incomplete
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
 
     getYearMonth();    // Scan YearMonth and
                        // fStart point to the next '-'
 
     if (fBuffer[fStart++] != DATE_SEPARATOR)
     {
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_date_invalid
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
         //("CCYY-MM must be followed by '-' sign");
     }
 
@@ -1029,18 +1064,20 @@ void XMLDateTime::getTime()
 
     // Ensure enough chars in buffer
     if ( (fStart+TIME_MIN_SIZE) > fEnd)
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_time_incomplete
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
         //"Imcomplete Time Format"
 
     // check (fixed) format first
     if ((fBuffer[fStart + 2] != TIME_SEPARATOR) ||
         (fBuffer[fStart + 5] != TIME_SEPARATOR)  )
     {
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_time_invalid
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
         //("Error in parsing time" );
     }
 
@@ -1067,28 +1104,30 @@ void XMLDateTime::getTime()
         // make sure we have some thing between the '.' and fEnd
         if (fStart >= fEnd)
         {
-            ThrowXML1(SchemaDateTimeException
+            ThrowXMLwithMemMgr1(SchemaDateTimeException
                     , XMLExcepts::DateTime_ms_noDigit
-                    , fBuffer);
+                    , fBuffer
+                    , fMemoryManager);
             //("ms shall be present once '.' is present" );
         }
 
         if ( sign == NOT_FOUND )
         {
-            fValue[MiliSecond] = parseInt(fStart, fEnd);  //get ms between '.' and fEnd
+            fMiliSecond = parseMiliSecond(fStart, fEnd);  //get ms between '.' and fEnd
             fStart = fEnd;
         }
         else
         {
-            fValue[MiliSecond] = parseInt(fStart, sign);  //get ms between UTC sign and fEnd
+            fMiliSecond = parseMiliSecond(fStart, sign);  //get ms between UTC sign and fEnd
         }
 	}
     else if(sign == 0 || sign != fStart)
     {
         // seconds has more than 2 digits
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_min_invalid
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
     }
 
     //parse UTC time zone (hh:mm)
@@ -1109,9 +1148,10 @@ void XMLDateTime::getYearMonth()
 
     // Ensure enough chars in buffer
     if ( (fStart+YMONTH_MIN_SIZE) > fEnd)
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_ym_incomplete
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
         //"Imcomplete YearMonth Format";
 
     // skip the first leading '-'
@@ -1122,9 +1162,10 @@ void XMLDateTime::getYearMonth()
     //
     int yearSeparator = indexOf(start, fEnd, DATE_SEPARATOR);
     if ( yearSeparator == NOT_FOUND)
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_ym_invalid
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
         //("Year separator is missing or misplaced");
 
     fValue[CentYear] = parseIntYear(yearSeparator);
@@ -1134,9 +1175,10 @@ void XMLDateTime::getYearMonth()
     //gonna check we have enough byte for month
     //
     if ((fStart + 2) > fEnd )
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_ym_noMonth
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
         //"no month in buffer"
 
     fValue[Month] = parseInt(fStart, yearSeparator + 3);
@@ -1152,9 +1194,10 @@ void XMLDateTime::parseTimeZone()
         int sign = findUTCSign(fStart);
         if ( sign < 0 )
         {
-            ThrowXML1(SchemaDateTimeException
+            ThrowXMLwithMemMgr1(SchemaDateTimeException
                     , XMLExcepts::DateTime_tz_noUTCsign
-                    , fBuffer);
+                    , fBuffer
+                    , fMemoryManager);
             //("Error in month parsing");
         }
         else
@@ -1180,9 +1223,10 @@ void XMLDateTime::getTimeZone(const int sign)
     {
         if ((sign + 1) != fEnd )
         {
-            ThrowXML1(SchemaDateTimeException
+            ThrowXMLwithMemMgr1(SchemaDateTimeException
                     , XMLExcepts::DateTime_tz_stuffAfterZ
-                    , fBuffer);
+                    , fBuffer
+                    , fMemoryManager);
             //"Error in parsing time zone");
         }		
 
@@ -1198,9 +1242,10 @@ void XMLDateTime::getTimeZone(const int sign)
     if ( ( ( sign + TIMEZONE_SIZE + 1) != fEnd )      ||
          ( fBuffer[sign + 3] != TIMEZONE_SEPARATOR ) )
     {
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_tz_invalid
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
         //("Error in parsing time zone");
     }
 
@@ -1277,18 +1322,20 @@ void XMLDateTime::validateDateTime() const
     //          or reporting an error message should be sufficient?
     if ( fValue[CentYear] == 0 )
     {
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_year_zero
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
         //"The year \"0000\" is an illegal year value");
     }
 
     if ( fValue[Month] < 1  ||
          fValue[Month] > 12  )
     {
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_mth_invalid
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
 		//"The month must have values 1 to 12");
     }
 
@@ -1296,9 +1343,10 @@ void XMLDateTime::validateDateTime() const
     if ( fValue[Day] > maxDayInMonthFor( fValue[CentYear], fValue[Month]) ||
          fValue[Day] == 0 )
     {
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_day_invalid
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
         //"The day must have values 1 to 31");
     }
 
@@ -1307,11 +1355,12 @@ void XMLDateTime::validateDateTime() const
         (fValue[Hour] > 24) ||
         ((fValue[Hour] == 24) && ((fValue[Minute] !=0) ||
                                   (fValue[Second] !=0) ||
-                                  (fValue[MiliSecond] !=0))))
+                                  (fMiliSecond    !=0))))
     {
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_hour_invalid
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
         //("Hour must have values 0-23");
     }
 
@@ -1319,9 +1368,10 @@ void XMLDateTime::validateDateTime() const
     if ( fValue[Minute] < 0 ||
          fValue[Minute] > 59 )
     {
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_min_invalid
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
         //"Minute must have values 0-59");
     }
 
@@ -1329,9 +1379,10 @@ void XMLDateTime::validateDateTime() const
     if ( fValue[Second] < 0 ||
          fValue[Second] > 60 )
     {
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_second_invalid
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
         //"Second must have values 0-60");
     }
 
@@ -1339,18 +1390,20 @@ void XMLDateTime::validateDateTime() const
     if ( (abs(fTimeZone[hh]) > 14) ||
          ((abs(fTimeZone[hh]) == 14) && (fTimeZone[mm] != 0)) )
     {
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_tz_hh_invalid
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
         //"Time zone should have range -14..+14");
     }
 
     //validate time-zone minutes
     if ( abs(fTimeZone[mm]) > 59 )
     {
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_min_invalid
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
         //("Minute must have values 0-59");
     }
 	
@@ -1397,12 +1450,46 @@ int XMLDateTime::parseInt(const int start, const int end) const
     for (int i=start; i < end; i++) {
 
         if (fBuffer[i] < chDigit_0 || fBuffer[i] > chDigit_9)
-            break;
+            ThrowXMLwithMemMgr(NumberFormatException, XMLExcepts::XMLNUM_Inv_chars, fMemoryManager);
 
         retVal = (retVal * 10) + (unsigned int) (fBuffer[i] - chDigit_0);
     }
 
-    return (int) retVal;;
+    return (int) retVal;
+}
+
+//
+// Note:
+//    start: pointing to the first digit after the '.'
+//    end:   pointing to one position after the last digit
+//    fStart NOT updated
+//
+double XMLDateTime::parseMiliSecond(const int start, const int end) const
+{
+
+    unsigned int  miliSecLen = (end-1) - (start-1) + 1; //to include the '.'
+    XMLCh* miliSecData = (XMLCh*) fMemoryManager->allocate( (miliSecLen + 1) * sizeof(XMLCh));
+    ArrayJanitor<XMLCh> janMili(miliSecData, fMemoryManager);
+    XMLString::copyNString(miliSecData, &(fBuffer[start-1]), miliSecLen);
+    *(miliSecData + miliSecLen) = chNull;
+
+    char *nptr = XMLString::transcode(miliSecData, fMemoryManager);
+    ArrayJanitor<char> jan(nptr, fMemoryManager);
+    int   strLen = strlen(nptr);
+    char *endptr = 0;
+    errno = 0;
+
+    //printf("milisec=<%s>\n", nptr);
+
+    double retVal = strtod(nptr, &endptr);
+
+    // check if all chars are valid char
+    if ( (endptr - nptr) != strLen)
+        ThrowXMLwithMemMgr(NumberFormatException, XMLExcepts::XMLNUM_Inv_chars, fMemoryManager);
+
+    // we don't check underflow occurs since
+    // nothing we can do about it.
+    return retVal;
 }
 
 //
@@ -1420,17 +1507,19 @@ int XMLDateTime::parseIntYear(const int end) const
     int length = end - start;
     if (length < 4)
     {
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_year_tooShort
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
         //"Year must have 'CCYY' format");
     }
     else if (length > 4 &&
              fBuffer[start] == chDigit_0)
     {
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_year_leadingZero
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
         //"Leading zeros are required if the year value would otherwise have fewer than four digits;
         // otherwise they are forbidden");
     }
@@ -1440,6 +1529,205 @@ int XMLDateTime::parseIntYear(const int end) const
     return ( negative ? (-1) * yearVal : yearVal );
 }
 
+/***
+ * E2-41
+ *
+ *  3.2.7.2 Canonical representation
+ * 
+ *  Except for trailing fractional zero digits in the seconds representation, 
+ *  '24:00:00' time representations, and timezone (for timezoned values), 
+ *  the mapping from literals to values is one-to-one. Where there is more 
+ *  than one possible representation, the canonical representation is as follows: 
+ *  redundant trailing zero digits in fractional-second literals are prohibited. 
+ *  An hour representation of '24' is prohibited. Timezoned values are canonically
+ *  represented by appending 'Z' to the nontimezoned representation. (All 
+ *  timezoned dateTime values are UTC.) 
+ *
+ *  .'24:00:00' -> '00:00:00'
+ *  .milisecond: trailing zeros removed
+ *  .'Z'
+ *
+ ***/
+XMLCh* XMLDateTime::getDateTimeCanonicalRepresentation(MemoryManager* const memMgr) const
+{
+    XMLCh *miliStartPtr, *miliEndPtr;
+    searchMiliSeconds(miliStartPtr, miliEndPtr);
+    int miliSecondsLen = miliEndPtr - miliStartPtr;
+
+    MemoryManager* toUse = memMgr? memMgr : fMemoryManager;
+    XMLCh* retBuf = (XMLCh*) toUse->allocate( (21 + miliSecondsLen + 2) * sizeof(XMLCh));
+    XMLCh* retPtr = retBuf;
+
+    // (-?) cc+yy-mm-dd'T'hh:mm:ss'Z'    ('.'s+)?
+    //      2+  8       1      8   1
+    //
+    int additionalLen = fillYearString(retPtr, CentYear);
+    if(additionalLen != 0)
+    {
+        // very bad luck; have to resize the buffer...
+        XMLCh *tmpBuf = (XMLCh*) toUse->allocate( (additionalLen+21+miliSecondsLen +2) * sizeof(XMLCh));
+        XMLString::moveChars(tmpBuf, retBuf, 4+additionalLen);
+        retPtr = tmpBuf+(retPtr-retBuf);
+        toUse->deallocate(retBuf);
+        retBuf = tmpBuf;
+    }
+    *retPtr++ = DATE_SEPARATOR;
+    fillString(retPtr, Month, 2);
+    *retPtr++ = DATE_SEPARATOR;
+    fillString(retPtr, Day, 2);
+    *retPtr++ = DATETIME_SEPARATOR;
+
+    fillString(retPtr, Hour, 2);
+    if (fValue[Hour] == 24)
+    {
+        *(retPtr - 2) = chDigit_0;
+        *(retPtr - 1) = chDigit_0;
+    }
+    *retPtr++ = TIME_SEPARATOR;
+    fillString(retPtr, Minute, 2);
+    *retPtr++ = TIME_SEPARATOR;
+    fillString(retPtr, Second, 2);
+
+    if (miliSecondsLen)
+    {
+        *retPtr++ = chPeriod;
+        XMLString::copyNString(retPtr, miliStartPtr, miliSecondsLen);
+        retPtr += miliSecondsLen;
+    }
+
+    *retPtr++ = UTC_STD_CHAR;
+    *retPtr = chNull;
+
+    return retBuf;
+}
+
+/***
+ * 3.2.8 time
+ *
+ *  . either the time zone must be omitted or, 
+ *    if present, the time zone must be Coordinated Universal Time (UTC) indicated by a "Z".   
+ *
+ *  . Additionally, the canonical representation for midnight is 00:00:00.
+ *
+***/
+XMLCh* XMLDateTime::getTimeCanonicalRepresentation(MemoryManager* const memMgr) const
+{
+    XMLCh *miliStartPtr, *miliEndPtr;
+    searchMiliSeconds(miliStartPtr, miliEndPtr);
+    int miliSecondsLen = miliEndPtr - miliStartPtr;
+
+    MemoryManager* toUse = memMgr? memMgr : fMemoryManager;
+    XMLCh* retBuf = (XMLCh*) toUse->allocate( (10 + miliSecondsLen + 2) * sizeof(XMLCh));
+    XMLCh* retPtr = retBuf;
+
+    // 'hh:mm:ss'Z'    ('.'s+)?
+    //      8    1
+    //
+
+    fillString(retPtr, Hour, 2);
+    if (fValue[Hour] == 24)
+    {
+        *(retPtr - 2) = chDigit_0;
+        *(retPtr - 1) = chDigit_0;
+    }
+    *retPtr++ = TIME_SEPARATOR;
+    fillString(retPtr, Minute, 2);
+    *retPtr++ = TIME_SEPARATOR;
+    fillString(retPtr, Second, 2);
+
+    if (miliSecondsLen)
+    {
+        *retPtr++ = chPeriod;
+        XMLString::copyNString(retPtr, miliStartPtr, miliSecondsLen);
+        retPtr += miliSecondsLen;
+    }
+
+    *retPtr++ = UTC_STD_CHAR;
+    *retPtr = chNull;
+
+    return retBuf;
+}
+
+void XMLDateTime::fillString(XMLCh*& ptr, valueIndex ind, int expLen) const
+{
+    XMLCh strBuffer[16];
+    assert(expLen < 16);
+    XMLString::binToText(fValue[ind], strBuffer, expLen, 10, fMemoryManager);
+    int   actualLen = XMLString::stringLen(strBuffer);
+    int   i;
+    //append leading zeros
+    for (i = 0; i < expLen - actualLen; i++)
+    {
+        *ptr++ = chDigit_0;
+    }
+
+    for (i = 0; i < actualLen; i++)
+    {
+        *ptr++ = strBuffer[i];
+    }
+
+}
+
+int XMLDateTime::fillYearString(XMLCh*& ptr, valueIndex ind) const
+{
+    XMLCh strBuffer[16];
+    // let's hope we get no years of 15 digits...
+    XMLString::binToText(fValue[ind], strBuffer, 15, 10, fMemoryManager);
+    int   actualLen = XMLString::stringLen(strBuffer);
+    // don't forget that years can be negative...
+    int negativeYear = 0;
+    if(strBuffer[0] == chDash)
+    {
+        *ptr++ = strBuffer[0];
+        negativeYear = 1;
+    }
+    int   i;
+    //append leading zeros
+    for (i = 0; i < 4 - actualLen+negativeYear; i++)
+    {
+        *ptr++ = chDigit_0;
+    }
+
+    for (i = negativeYear; i < actualLen; i++)
+    {
+        *ptr++ = strBuffer[i];
+    }
+    if(actualLen > 4)
+        return actualLen-4;
+    return 0;
+}
+
+/***
+ *
+ *   .check if the rawData has the mili second component
+ *   .capture the substring
+ *
+ ***/
+void XMLDateTime::searchMiliSeconds(XMLCh*& miliStartPtr, XMLCh*& miliEndPtr) const
+{
+    miliStartPtr = miliEndPtr = 0;
+
+    int milisec = XMLString::indexOf(fBuffer, MILISECOND_SEPARATOR);
+    if (milisec == -1)
+        return;
+
+    miliStartPtr = fBuffer + milisec + 1;
+    miliEndPtr   = miliStartPtr;
+    while (*miliEndPtr)
+    {
+        if ((*miliEndPtr < chDigit_0) || (*miliEndPtr > chDigit_9))
+            break;
+
+        miliEndPtr++;
+    }
+
+    //remove trailing zeros
+    while( *(miliEndPtr - 1) == chDigit_0)
+        miliEndPtr--;
+
+    return;
+}
+
 /***
  * Support for Serialization/De-serialization
  ***/
