@@ -1,61 +1,21 @@
 /*
- * The Apache Software License, Version 1.1
- *
- * Copyright (c) 1999-2000 The Apache Software Foundation.  All rights
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xerces" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written
- *    permission, please contact apache\@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation, and was
- * originally based on software copyright (c) 1999, International
- * Business Machines, Inc., http://www.ibm.com .  For more information
- * on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Copyright 1999-2004 The Apache Software Foundation.
+ * 
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 /**
- * $Id: XMLUTF8Transcoder.cpp,v 1.3 2002/11/28 17:39:02 tng Exp $
+ * $Id: XMLUTF8Transcoder.cpp,v 1.10 2004/09/08 13:56:24 peiyongz Exp $
  */
 
 // ---------------------------------------------------------------------------
@@ -65,8 +25,6 @@
 #include <xercesc/util/XMLString.hpp>
 #include <xercesc/util/XMLUniDefs.hpp>
 #include <xercesc/util/XMLUTF8Transcoder.hpp>
-#include <xercesc/util/UTFDataFormatException.hpp>
-
 
 XERCES_CPP_NAMESPACE_BEGIN
 
@@ -76,6 +34,16 @@ XERCES_CPP_NAMESPACE_BEGIN
 //  gUTFBytes
 //      A list of counts of trailing bytes for each initial byte in the input.
 //
+//  gUTFByteIndicator
+//      For a UTF8 sequence of n bytes, n>=2, the first byte of the
+//      sequence must contain n 1's followed by precisely 1 0 with the
+//      rest of the byte containing arbitrary bits.  This array stores
+//      the required bit pattern for validity checking.
+//  gUTFByteIndicatorTest
+//      When bitwise and'd with the observed value, if the observed
+//      value is correct then a result matching gUTFByteIndicator will
+//      be produced.
+//
 //  gUTFOffsets
 //      A list of values to offset each result char type, according to how
 //      many source bytes when into making it.
@@ -98,12 +66,21 @@ static const XMLByte gUTFBytes[256] =
     ,   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
     ,   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
     ,   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
-    ,   1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1
+    ,   0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1
     ,   1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1
     ,   2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2
     ,   3, 3, 3, 3, 3, 3, 3, 3, 4, 4, 4, 4, 5, 5, 5, 5
 };
 
+static const XMLByte gUTFByteIndicator[6] =
+{
+    0x00, 0xC0, 0xE0, 0xF0, 0xF8, 0xFC
+};
+static const XMLByte gUTFByteIndicatorTest[6] =
+{
+    0x80, 0xE0, 0xF0, 0xF8, 0xFC, 0xFE
+};
+
 static const XMLUInt32 gUTFOffsets[6] =
 {
     0, 0x3080, 0xE2080, 0x3C82080, 0xFA082080, 0x82082080
@@ -120,9 +97,9 @@ static const XMLByte gFirstByteMark[7] =
 //  XMLUTF8Transcoder: Constructors and Destructor
 // ---------------------------------------------------------------------------
 XMLUTF8Transcoder::XMLUTF8Transcoder(const  XMLCh* const    encodingName
-                                    , const unsigned int    blockSize) :
-
-    XMLTranscoder(encodingName, blockSize)
+                                    , const unsigned int    blockSize
+                                    , MemoryManager* const  manager)
+:XMLTranscoder(encodingName, blockSize, manager)
 {
 }
 
@@ -192,20 +169,195 @@ XMLUTF8Transcoder::transcodeFrom(const  XMLByte* const          srcData
             break;
 
         // Looks ok, so lets build up the value
+        // or at least let's try to do so--remembering that
+        // we cannot assume the encoding to be valid:
+
+        // first, test first byte
+        if((gUTFByteIndicatorTest[trailingBytes] & *srcPtr) != gUTFByteIndicator[trailingBytes]) {
+            char pos[2] = {(char)0x31, 0}; 
+            char len[2] = {(char)(trailingBytes+0x31), 0};
+            char byte[2] = {*srcPtr,0};
+            ThrowXMLwithMemMgr3(UTFDataFormatException, XMLExcepts::UTF8_FormatError, pos, byte, len, getMemoryManager());
+        }
+
+        /***
+         * http://www.unicode.org/reports/tr27/
+         *
+         * Table 3.1B. lists all of the byte sequences that are legal in UTF-8. 
+         * A range of byte values such as A0..BF indicates that any byte from A0 to BF (inclusive) 
+         * is legal in that position. 
+         * Any byte value outside of the ranges listed is illegal. 
+         * For example, 
+         * the byte sequence <C0 AF> is illegal  since C0 is not legal in the 1st Byte column. 
+         * The byte sequence <E0 9F 80> is illegal since in the row 
+         *    where E0 is legal as a first byte, 
+         *    9F is not legal as a second byte. 
+         * The byte sequence <F4 80 83 92> is legal, since every byte in that sequence matches 
+         * a byte range in a row of the table (the last row). 
+         *
+         *
+         * Table 3.1B. Legal UTF-8 Byte Sequences  
+         * Code Points              1st Byte    2nd Byte    3rd Byte    4th Byte 
+         * =========================================================================
+         * U+0000..U+007F            00..7F       
+         * -------------------------------------------------------------------------
+         * U+0080..U+07FF            C2..DF      80..BF      
+         *
+         * -------------------------------------------------------------------------
+         * U+0800..U+0FFF            E0          A0..BF     80..BF   
+         *                                       -- 
+         *                          
+         * U+1000..U+FFFF            E1..EF      80..BF     80..BF    
+         *
+         * --------------------------------------------------------------------------
+         * U+10000..U+3FFFF          F0          90..BF     80..BF       80..BF 
+         *                                       --
+         * U+40000..U+FFFFF          F1..F3      80..BF     80..BF       80..BF 
+         * U+100000..U+10FFFF        F4          80..8F     80..BF       80..BF 
+         *                                           --
+         * ==========================================================================
+         *
+         *  Cases where a trailing byte range is not 80..BF are underlined in the table to 
+         *  draw attention to them. These occur only in the second byte of a sequence.
+         *
+         ***/
+
         XMLUInt32 tmpVal = 0;
+
         switch(trailingBytes)
         {
-            case 5 : tmpVal += *srcPtr++; tmpVal <<= 6;
-            case 4 : tmpVal += *srcPtr++; tmpVal <<= 6;
-            case 3 : tmpVal += *srcPtr++; tmpVal <<= 6;
-            case 2 : tmpVal += *srcPtr++; tmpVal <<= 6;
-            case 1 : tmpVal += *srcPtr++; tmpVal <<= 6;
-            case 0 : tmpVal += *srcPtr++;
-                     break;
+            case 1 :
+                // UTF-8:   [110y yyyy] [10xx xxxx]
+                // Unicode: [0000 0yyy] [yyxx xxxx]
+                //
+                // 0xC0, 0xC1 has been filtered out             
+                checkTrailingBytes(*(srcPtr+1), 1, 1);
 
-            default :
-                ThrowXML(TranscodingException, XMLExcepts::Trans_BadSrcSeq);
+                tmpVal = *srcPtr++;
+                tmpVal <<= 6;
+                tmpVal += *srcPtr++;
+
+                break;
+            case 2 :
+                // UTF-8:   [1110 zzzz] [10yy yyyy] [10xx xxxx]
+                // Unicode: [zzzz yyyy] [yyxx xxxx]
+                //
+                if (( *srcPtr == 0xE0) && ( *(srcPtr+1) < 0xA0)) 
+                {
+                    char byte0[2] = {*srcPtr    ,0};
+                    char byte1[2] = {*(srcPtr+1),0};
+
+                    ThrowXMLwithMemMgr2(UTFDataFormatException
+                                      , XMLExcepts::UTF8_Invalid_3BytesSeq
+                                      , byte0
+                                      , byte1
+                                      , getMemoryManager());
+                }
+
+                checkTrailingBytes(*(srcPtr+1), 2, 1);
+                checkTrailingBytes(*(srcPtr+2), 2, 2);
+
+                //
+                // D36 (a) UTF-8 is the Unicode Transformation Format that serializes 
+                //         a Unicode code point as a sequence of one to four bytes, 
+                //         as specified in Table 3.1, UTF-8 Bit Distribution.
+                //     (b) An illegal UTF-8 code unit sequence is any byte sequence that 
+                //         does not match the patterns listed in Table 3.1B, Legal UTF-8 
+                //         Byte Sequences.
+                //     (c) An irregular UTF-8 code unit sequence is a six-byte sequence 
+                //         where the first three bytes correspond to a high surrogate, 
+                //         and the next three bytes correspond to a low surrogate. 
+                //         As a consequence of C12, these irregular UTF-8 sequences shall 
+                //         not be generated by a conformant process. 
+                //
+                //irregular three bytes sequence
+                // that is zzzzyy matches leading surrogate tag 110110 or 
+                //                       trailing surrogate tag 110111
+                // *srcPtr=1110 1101 
+                // *(srcPtr+1)=1010 yyyy or 
+                // *(srcPtr+1)=1011 yyyy
+                //
+                // 0xED 1110 1101
+                // 0xA0 1010 0000
+
+                if ((*srcPtr == 0xED) && (*(srcPtr+1) >= 0xA0))
+                {
+                    char byte0[2] = {*srcPtr,    0};
+                    char byte1[2] = {*(srcPtr+1),0};
+
+                     ThrowXMLwithMemMgr2(UTFDataFormatException
+                              , XMLExcepts::UTF8_Irregular_3BytesSeq
+                              , byte0
+                              , byte1
+                              , getMemoryManager());
+                }
+
+                tmpVal = *srcPtr++;
+                tmpVal <<= 6;
+                tmpVal += *srcPtr++;
+                tmpVal <<= 6;
+                tmpVal += *srcPtr++;
+
+                break;
+            case 3 : 
+                // UTF-8:   [1111 0uuu] [10uu zzzz] [10yy yyyy] [10xx xxxx]*
+                // Unicode: [1101 10ww] [wwzz zzyy] (high surrogate)
+                //          [1101 11yy] [yyxx xxxx] (low surrogate)
+                //          * uuuuu = wwww + 1
+                //
+                if (((*srcPtr == 0xF0) && (*(srcPtr+1) < 0x90)) ||
+                    ((*srcPtr == 0xF4) && (*(srcPtr+1) > 0x8F))  )
+                {
+                    char byte0[2] = {*srcPtr    ,0};
+                    char byte1[2] = {*(srcPtr+1),0};
+
+                    ThrowXMLwithMemMgr2(UTFDataFormatException
+                                      , XMLExcepts::UTF8_Invalid_4BytesSeq
+                                      , byte0
+                                      , byte1
+                                      , getMemoryManager());
+                }
+
+                checkTrailingBytes(*(srcPtr+1), 3, 1);
+                checkTrailingBytes(*(srcPtr+2), 3, 2);
+                checkTrailingBytes(*(srcPtr+3), 3, 3);
+                
+                tmpVal = *srcPtr++;
+                tmpVal <<= 6;
+                tmpVal += *srcPtr++;
+                tmpVal <<= 6;
+                tmpVal += *srcPtr++;
+                tmpVal <<= 6;
+                tmpVal += *srcPtr++;
+
+                break;
+            default: // trailingBytes > 3
+
+                /***
+                 * The definition of UTF-8 in Annex D of ISO/IEC 10646-1:2000 also allows 
+                 * for the use of five- and six-byte sequences to encode characters that 
+                 * are outside the range of the Unicode character set; those five- and 
+                 * six-byte sequences are illegal for the use of UTF-8 as a transformation 
+                 * of Unicode characters. ISO/IEC 10646 does not allow mapping of unpaired 
+                 * surrogates, nor U+FFFE and U+FFFF (but it does allow other noncharacters).
+                 ***/
+                char len[2]  = {(char)(trailingBytes+0x31), 0};
+                char byte[2] = {*srcPtr,0};
+
+                ThrowXMLwithMemMgr2(UTFDataFormatException
+                                  , XMLExcepts::UTF8_Exceede_BytesLimit
+                                  , byte
+                                  , len
+                                  , getMemoryManager());
+
+                break;
         }
+
+
+        // since trailingBytes comes from an array, this logic is redundant
+        //  default :
+        //      ThrowXMLwithMemMgr(TranscodingException, XMLExcepts::Trans_BadSrcSeq);
+        //}
         tmpVal -= gUTFOffsets[trailingBytes];
 
         //
@@ -230,7 +382,7 @@ XMLUTF8Transcoder::transcodeFrom(const  XMLByte* const          srcData
             if ((outPtr - toFill) > 32)
                 break;
 
-            ThrowXML(TranscodingException, XMLExcepts::Trans_BadSrcSeq);
+            ThrowXMLwithMemMgr(TranscodingException, XMLExcepts::Trans_BadSrcSeq, getMemoryManager());
         }
          else
         {
@@ -250,12 +402,12 @@ XMLUTF8Transcoder::transcodeFrom(const  XMLByte* const          srcData
             *outPtr++ = XMLCh((tmpVal >> 10) + 0xD800);
 
             //
-            //  And then the treailing char. This one accounts for no
+            //  And then the trailing char. This one accounts for no
             //  bytes eaten from the source, so set the char size for this
             //  one to be zero.
             //
             *sizePtr++ = 0;
-            *outPtr++ = XMLCh(tmpVal & 0x3FF) + 0xDC00;
+            *outPtr++ = XMLCh((tmpVal & 0x3FF) + 0xDC00);
         }
     }
 
@@ -322,25 +474,22 @@ XMLUTF8Transcoder::transcodeTo( const   XMLCh* const    srcData
             encodedBytes = 2;
         else if (curVal < 0x10000)
             encodedBytes = 3;
-        else if (curVal < 0x200000)
+        else if (curVal < 0x110000)
             encodedBytes = 4;
-        else if (curVal < 0x4000000)
-            encodedBytes = 5;
-        else if (curVal <= 0x7FFFFFFF)
-            encodedBytes = 6;
         else
         {
             // If the options say to throw, then throw
             if (options == UnRep_Throw)
             {
-                XMLCh tmpBuf[16];
-                XMLString::binToText(curVal, tmpBuf, 16, 16);
-                ThrowXML2
+                XMLCh tmpBuf[17];
+                XMLString::binToText(curVal, tmpBuf, 16, 16, getMemoryManager());
+                ThrowXMLwithMemMgr2
                 (
                     TranscodingException
                     , XMLExcepts::Trans_Unrepresentable
                     , tmpBuf
                     , getEncodingName()
+                    , getMemoryManager()
                 );
             }
 
