@@ -1,61 +1,33 @@
 /*
- * The Apache Software License, Version 1.1
- *
- * Copyright (c) 1999-2000 The Apache Software Foundation.  All rights
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xerces" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written
- *    permission, please contact apache\@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation, and was
- * originally based on software copyright (c) 1999, International
- * Business Machines, Inc., http://www.ibm.com .  For more information
- * on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Copyright 1999-2000,2004 The Apache Software Foundation.
+ * 
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 /*
  * $Log: DTDEntityDecl.hpp,v $
+ * Revision 1.6  2004/09/08 13:56:50  peiyongz
+ * Apache License Version 2.0
+ *
+ * Revision 1.5  2003/10/10 16:24:51  peiyongz
+ * Implementation of Serialization/Deserialization
+ *
+ * Revision 1.4  2003/05/16 21:43:19  knoaman
+ * Memory manager implementation: Modify constructors to pass in the memory manager.
+ *
+ * Revision 1.3  2003/03/07 18:17:12  tng
+ * Return a reference instead of void for operator=
+ *
  * Revision 1.2  2002/11/04 14:50:40  tng
  * C++ Namespace Support.
  *
@@ -99,17 +71,19 @@ public :
     // -----------------------------------------------------------------------
     //  Constructors and Destructor
     // -----------------------------------------------------------------------
-    DTDEntityDecl();
+    DTDEntityDecl(MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager);
     DTDEntityDecl
     (
-        const   XMLCh* const    entName
-        , const bool            fromIntSubset = false
+        const   XMLCh* const   entName
+        , const bool           fromIntSubset = false
+        , MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager
     );
     DTDEntityDecl
     (
-        const   XMLCh* const    entName
-        , const XMLCh* const    value
-        , const bool            fromIntSubset = false
+        const   XMLCh* const   entName
+        , const XMLCh* const   value
+        , const bool           fromIntSubset = false
+        , MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager
     );
     DTDEntityDecl
     (
@@ -136,13 +110,17 @@ public :
     void setIsParameter(const bool newValue);
     void setIsSpecialChar(const bool newValue);
 
+    /***
+     * Support for Serialization/De-serialization
+     ***/
+    DECL_XSERIALIZABLE(DTDEntityDecl)
 
 private :
     // -----------------------------------------------------------------------
     //  Unimplemented constructors and operators
     // -----------------------------------------------------------------------
     DTDEntityDecl(const DTDEntityDecl&);
-    void operator=(DTDEntityDecl&);
+    DTDEntityDecl& operator=(DTDEntityDecl&);
 
 
     // -----------------------------------------------------------------------
@@ -171,28 +149,31 @@ private :
 // ---------------------------------------------------------------------------
 //  DTDEntityDecl: Constructors and Destructor
 // ---------------------------------------------------------------------------
-inline DTDEntityDecl::DTDEntityDecl() :
+inline DTDEntityDecl::DTDEntityDecl(MemoryManager* const manager) :
 
-    fDeclaredInIntSubset(false)
+    XMLEntityDecl(manager)
+    , fDeclaredInIntSubset(false)
     , fIsParameter(false)
     , fIsSpecialChar(false)
 {
 }
 
-inline DTDEntityDecl::DTDEntityDecl(const   XMLCh* const    entName
-                                    , const bool            fromIntSubset) :
+inline DTDEntityDecl::DTDEntityDecl( const XMLCh* const   entName
+                                   , const bool           fromIntSubset
+                                   , MemoryManager* const manager) :
 
-    XMLEntityDecl(entName)
+    XMLEntityDecl(entName, manager)
     , fDeclaredInIntSubset(fromIntSubset)
     , fIsParameter(false)
     , fIsSpecialChar(false)
 {
 }
 
-inline DTDEntityDecl::DTDEntityDecl(const   XMLCh* const    entName
-                                    , const XMLCh* const    value
-                                    , const bool            fromIntSubset) :
-    XMLEntityDecl(entName, value)
+inline DTDEntityDecl::DTDEntityDecl( const XMLCh* const   entName
+                                   , const XMLCh* const   value
+                                   , const bool           fromIntSubset
+                                   , MemoryManager* const manager) :
+    XMLEntityDecl(entName, value, manager)
     , fDeclaredInIntSubset(fromIntSubset)
     , fIsParameter(false)
     , fIsSpecialChar(false)
@@ -203,7 +184,7 @@ inline DTDEntityDecl::DTDEntityDecl(const   XMLCh* const    entName
                                     , const XMLCh           value
                                     , const bool            fromIntSubset
                                     , const bool            specialChar) :
-    XMLEntityDecl(entName, value)
+    XMLEntityDecl(entName, value, XMLPlatformUtils::fgMemoryManager)
     , fDeclaredInIntSubset(fromIntSubset)
     , fIsParameter(false)
     , fIsSpecialChar(specialChar)
