@@ -1,61 +1,48 @@
 /*
- * The Apache Software License, Version 1.1
- *
- * Copyright (c) 1999-2001 The Apache Software Foundation.  All rights
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xerces" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written
- *    permission, please contact apache\@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation, and was
- * originally based on software copyright (c) 1999, International
- * Business Machines, Inc., http://www.ibm.com .  For more information
- * on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Copyright 1999-2001,2004 The Apache Software Foundation.
+ * 
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 /*
  * $Log: MixedContentModel.cpp,v $
+ * Revision 1.12  2004/09/08 13:56:51  peiyongz
+ * Apache License Version 2.0
+ *
+ * Revision 1.11  2004/08/11 21:08:31  knoaman
+ * Fixing Xerces-C crash when creating a MixedContentModel. Patch by  Andrew Fang.
+ *
+ * Revision 1.10  2004/01/29 11:51:21  cargilld
+ * Code cleanup changes to get rid of various compiler diagnostic messages.
+ *
+ * Revision 1.9  2003/12/17 00:18:38  cargilld
+ * Update to memory management so that the static memory manager (one used to call Initialize) is only for static data.
+ *
+ * Revision 1.8  2003/11/20 18:12:20  knoaman
+ * Use a bitwise operation to check the node type.
+ *
+ * Revision 1.7  2003/11/07 17:08:11  knoaman
+ * For PSVI support, distinguish wildcard elements with namespace lists.
+ *
+ * Revision 1.6  2003/05/18 14:02:06  knoaman
+ * Memory manager implementation: pass per instance manager.
+ *
+ * Revision 1.5  2003/05/16 21:43:20  knoaman
+ * Memory manager implementation: Modify constructors to pass in the memory manager.
+ *
+ * Revision 1.4  2003/05/15 18:48:27  knoaman
+ * Partial implementation of the configurable memory manager.
+ *
  * Revision 1.3  2002/11/04 14:54:58  tng
  * C++ Namespace Support.
  *
@@ -148,12 +135,14 @@ XERCES_CPP_NAMESPACE_BEGIN
 // ---------------------------------------------------------------------------
 MixedContentModel::MixedContentModel(const bool             dtd
                                    , ContentSpecNode* const parentContentSpec
-                                   , const bool             ordered) :
+                                   , const bool             ordered
+                                   , MemoryManager* const   manager) :
    fCount(0)
  , fChildren(0)
  , fChildTypes(0)
  , fOrdered(ordered)
  , fDTD(dtd)
+ , fMemoryManager(manager)
 {
     //
     //  Create a vector of unsigned ints that will be filled in with the
@@ -161,9 +150,8 @@ MixedContentModel::MixedContentModel(const bool             dtd
     //  it an initial capacity of 64 which should be more than enough for
     //  99% of the scenarios.
     //
-
-    ValueVectorOf<QName*> children(64);
-    ValueVectorOf<ContentSpecNode::NodeTypes> childTypes(64);
+    ValueVectorOf<QName*> children(64, fMemoryManager);
+    ValueVectorOf<ContentSpecNode::NodeTypes> childTypes(64, fMemoryManager);
 
     //
     //  Get the parent element's content spec. This is the head of the tree
@@ -172,7 +160,7 @@ MixedContentModel::MixedContentModel(const bool             dtd
     //
     ContentSpecNode* curNode = parentContentSpec;
     if (!curNode)
-        ThrowXML(RuntimeException, XMLExcepts::CM_NoParentCSN);
+        ThrowXMLwithMemMgr(RuntimeException, XMLExcepts::CM_NoParentCSN, fMemoryManager);
 
     // And now call the private recursive method that iterates the tree
     buildChildList(curNode, children, childTypes);
@@ -182,18 +170,24 @@ MixedContentModel::MixedContentModel(const bool             dtd
     //  fill them in.
     //
     fCount = children.size();
-    fChildren = new QName*[fCount];
-    fChildTypes = new ContentSpecNode::NodeTypes[fCount];
+    fChildren = (QName**) fMemoryManager->allocate(fCount * sizeof(QName*)); //new QName*[fCount];
+    fChildTypes = (ContentSpecNode::NodeTypes*) fMemoryManager->allocate
+    (
+        fCount * sizeof(ContentSpecNode::NodeTypes)
+    ); //new ContentSpecNode::NodeTypes[fCount];
     for (unsigned int index = 0; index < fCount; index++) {
-        fChildren[index] = children.elementAt(index);
+        fChildren[index] = new (fMemoryManager) QName(*children.elementAt(index));
         fChildTypes[index] = childTypes.elementAt(index);
     }
 }
 
 MixedContentModel::~MixedContentModel()
 {
-    delete [] fChildren;
-    delete [] fChildTypes;
+    for (unsigned int index = 0; index < fCount; index++) {
+        delete fChildren[index];
+    }
+    fMemoryManager->deallocate(fChildren); //delete [] fChildren;
+    fMemoryManager->deallocate(fChildTypes); //delete [] fChildTypes;
 }
 
 
@@ -243,7 +237,7 @@ bool MixedContentModel::hasDups() const
 int
 MixedContentModel::validateContent( QName** const         children
                                   , const unsigned int    childCount
-                                  , const unsigned int    emptyNamespaceId) const
+                                  , const unsigned int) const
 {
     // must match order
     if (fOrdered) {
@@ -350,7 +344,7 @@ MixedContentModel::validateContent( QName** const         children
 
 int MixedContentModel::validateContentSpecial(QName** const           children
                                             , const unsigned int      childCount
-                                            , const unsigned int      emptyNamespaceId
+                                            , const unsigned int
                                             , GrammarResolver*  const pGrammarResolver
                                             , XMLStringPool*    const pStringPool) const
 {
@@ -468,8 +462,8 @@ MixedContentModel::buildChildList(  ContentSpecNode* const       curNode
     ContentSpecNode* rightNode = curNode->getSecond();
 
     // And recurse according to the type of node
-    if ((curType == ContentSpecNode::Choice)
-    ||  (curType == ContentSpecNode::Sequence))
+    if (((curType & 0x0f) == ContentSpecNode::Choice)
+    ||  ((curType & 0x0f) == ContentSpecNode::Sequence))
     {
         // Recurse on the left and right nodes
         buildChildList(leftNode, toFill, toType);
