@@ -1,61 +1,39 @@
 /*
- * The Apache Software License, Version 1.1
- *
- * Copyright (c) 1999-2001 The Apache Software Foundation.  All rights
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xerces" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written
- *    permission, please contact apache\@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation, and was
- * originally based on software copyright (c) 1999, International
- * Business Machines, Inc., http://www.ibm.com .  For more information
- * on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Copyright 1999-2001,2004 The Apache Software Foundation.
+ * 
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 /*
  * $Log: MixedContentModel.hpp,v $
+ * Revision 1.9  2004/09/16 13:32:04  amassari
+ * Updated error message for UPA to also state the complex type that is failing the test
+ *
+ * Revision 1.8  2004/09/08 13:56:51  peiyongz
+ * Apache License Version 2.0
+ *
+ * Revision 1.7  2004/01/29 11:51:21  cargilld
+ * Code cleanup changes to get rid of various compiler diagnostic messages.
+ *
+ * Revision 1.6  2003/05/16 21:43:20  knoaman
+ * Memory manager implementation: Modify constructors to pass in the memory manager.
+ *
+ * Revision 1.5  2003/05/15 18:48:27  knoaman
+ * Partial implementation of the configurable memory manager.
+ *
+ * Revision 1.4  2003/03/07 18:16:57  tng
+ * Return a reference instead of void for operator=
+ *
  * Revision 1.3  2002/11/04 14:54:58  tng
  * C++ Namespace Support.
  *
@@ -149,9 +127,10 @@ public :
     // -----------------------------------------------------------------------
     MixedContentModel
     (
-        const bool              dtd
-      , ContentSpecNode* const  parentContentSpec
+        const bool                dtd
+        , ContentSpecNode* const  parentContentSpec
 		, const bool              ordered = false
+        , MemoryManager* const    manager = XMLPlatformUtils::fgMemoryManager
     );
 
     ~MixedContentModel();
@@ -193,6 +172,7 @@ public :
       , XMLStringPool*    const pStringPool
       , XMLValidator*     const pValidator
       , unsigned int*     const pContentSpecOrgURI
+      , const XMLCh*            pComplexTypeName = 0
     ) ;
 
 private :
@@ -211,7 +191,7 @@ private :
     // -----------------------------------------------------------------------
     MixedContentModel();
     MixedContentModel(const MixedContentModel&);
-    void operator=(const MixedContentModel&);
+    MixedContentModel& operator=(const MixedContentModel&);
 
 
     // -----------------------------------------------------------------------
@@ -235,12 +215,12 @@ private :
     //      Boolean to allow DTDs to validate even with namespace support.
     //
     // -----------------------------------------------------------------------
-    unsigned int    fCount;
-    QName**         fChildren;
-    ContentSpecNode::NodeTypes*  fChildTypes;
-    bool            fOrdered;
-    bool            fDTD;
-
+    unsigned int                fCount;
+    QName**                     fChildren;
+    ContentSpecNode::NodeTypes* fChildTypes;
+    bool                        fOrdered;
+    bool                        fDTD;
+    MemoryManager*              fMemoryManager;
 };
 
 inline ContentLeafNameTypeVector* MixedContentModel::getContentLeafNameTypeVector() const
@@ -249,19 +229,20 @@ inline ContentLeafNameTypeVector* MixedContentModel::getContentLeafNameTypeVecto
 }
 
 inline unsigned int
-MixedContentModel::getNextState(const unsigned int currentState,
-                                const unsigned int elementIndex) const {
+MixedContentModel::getNextState(const unsigned int,
+                                const unsigned int) const {
 
     return XMLContentModel::gInvalidTrans;
 }
 
 inline void MixedContentModel::checkUniqueParticleAttribution
     (
-        SchemaGrammar*    const pGrammar
-      , GrammarResolver*  const pGrammarResolver
-      , XMLStringPool*    const pStringPool
-      , XMLValidator*     const pValidator
+        SchemaGrammar*    const 
+      , GrammarResolver*  const 
+      , XMLStringPool*    const 
+      , XMLValidator*     const
       , unsigned int*     const pContentSpecOrgURI
+      , const XMLCh*            pComplexTypeName /*= 0*/
     )
 {
     // rename back
