@@ -1,61 +1,45 @@
 /*
- * The Apache Software License, Version 1.1
- *
- * Copyright (c) 1999-2001 The Apache Software Foundation.  All rights
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xerces" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written
- *    permission, please contact apache\@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation, and was
- * originally based on software copyright (c) 1999, International
- * Business Machines, Inc., http://www.ibm.com .  For more information
- * on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Copyright 1999-2001,2004 The Apache Software Foundation.
+ * 
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 /*
  * $Log: SimpleContentModel.cpp,v $
+ * Revision 1.10  2004/09/16 13:32:04  amassari
+ * Updated error message for UPA to also state the complex type that is failing the test
+ *
+ * Revision 1.9  2004/09/08 13:56:51  peiyongz
+ * Apache License Version 2.0
+ *
+ * Revision 1.8  2004/01/29 11:51:21  cargilld
+ * Code cleanup changes to get rid of various compiler diagnostic messages.
+ *
+ * Revision 1.7  2003/12/17 00:18:38  cargilld
+ * Update to memory management so that the static memory manager (one used to call Initialize) is only for static data.
+ *
+ * Revision 1.6  2003/11/20 18:12:20  knoaman
+ * Use a bitwise operation to check the node type.
+ *
+ * Revision 1.5  2003/11/07 17:08:11  knoaman
+ * For PSVI support, distinguish wildcard elements with namespace lists.
+ *
+ * Revision 1.4  2002/11/04 14:54:58  tng
+ * C++ Namespace Support.
+ *
+ * Revision 1.3  2002/09/24 19:48:39  tng
+ * Performance: use XMLString::equals instead of XMLString::compareString
+ *
  * Revision 1.2  2002/02/25 21:18:53  tng
  * Schema Fix: Ensure no invalid uri index for UPA checking.
  *
@@ -127,6 +111,7 @@
 #include <xercesc/validators/schema/SubstitutionGroupComparator.hpp>
 #include <xercesc/validators/schema/XercesElementWildcard.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
 
 // ---------------------------------------------------------------------------
 //  SimpleContentModel: Implementation of the ContentModel virtual interface
@@ -139,14 +124,14 @@
 int
 SimpleContentModel::validateContent(QName** const       children
                                   , const unsigned int  childCount
-                                  , const unsigned int  emptyNamespaceId) const
+                                  , const unsigned int) const
 {
     //
     //  According to the type of operation, we do the correct type of
     //  content check.
     //
     unsigned int index;
-    switch(fOp)
+    switch(fOp & 0x0f)
     {
         case ContentSpecNode::Leaf :
             //
@@ -158,13 +143,13 @@ SimpleContentModel::validateContent(QName** const       children
 
             // If the 0th child is not the right kind, report an error at 0
             if (fDTD) {
-                if (XMLString::compareString(children[0]->getRawName(), fFirstChild->getRawName())!=0) {
+                if (!XMLString::equals(children[0]->getRawName(), fFirstChild->getRawName())) {
                     return 0;
                 }
             }
             else {
                 if ((children[0]->getURI() != fFirstChild->getURI()) ||
-                    (XMLString::compareString(children[0]->getLocalPart(), fFirstChild->getLocalPart())!=0)) {
+                    !XMLString::equals(children[0]->getLocalPart(), fFirstChild->getLocalPart())) {
                     return 0;
                 }
             }
@@ -181,13 +166,13 @@ SimpleContentModel::validateContent(QName** const       children
             //
             if (childCount == 1) {
                 if (fDTD) {
-                    if (XMLString::compareString(children[0]->getRawName(), fFirstChild->getRawName())!=0) {
+                    if (!XMLString::equals(children[0]->getRawName(), fFirstChild->getRawName())) {
                             return 0;
                     }
                 }
                 else {
                     if ((children[0]->getURI() != fFirstChild->getURI()) ||
-                        (XMLString::compareString(children[0]->getLocalPart(), fFirstChild->getLocalPart())!=0)) {
+                        (!XMLString::equals(children[0]->getLocalPart(), fFirstChild->getLocalPart()))) {
                         return 0;
                     }
                 }
@@ -208,7 +193,7 @@ SimpleContentModel::validateContent(QName** const       children
             {
                 if (fDTD) {
                     for (index = 0; index < childCount; index++) {
-                        if (XMLString::compareString(children[index]->getRawName(), fFirstChild->getRawName())!=0) {
+                        if (!XMLString::equals(children[index]->getRawName(), fFirstChild->getRawName())) {
                             return index;
                         }
                     }
@@ -216,7 +201,7 @@ SimpleContentModel::validateContent(QName** const       children
                 else {
                     for (index = 0; index < childCount; index++) {
                         if ((children[index]->getURI() != fFirstChild->getURI()) ||
-                            (XMLString::compareString(children[index]->getLocalPart(), fFirstChild->getLocalPart())!=0)) {
+                            !XMLString::equals(children[index]->getLocalPart(), fFirstChild->getLocalPart())) {
                             return index;
                         }
                     }
@@ -235,7 +220,7 @@ SimpleContentModel::validateContent(QName** const       children
 
             if (fDTD) {
                 for (index = 0; index < childCount; index++) {
-                    if (XMLString::compareString(children[index]->getRawName(), fFirstChild->getRawName())!=0) {
+                    if (!XMLString::equals(children[index]->getRawName(), fFirstChild->getRawName())) {
                         return index;
                     }
                 }
@@ -243,7 +228,7 @@ SimpleContentModel::validateContent(QName** const       children
             else {
                 for (index = 0; index < childCount; index++) {
                     if ((children[index]->getURI() != fFirstChild->getURI()) ||
-                        (XMLString::compareString(children[index]->getLocalPart(), fFirstChild->getLocalPart())!=0)) {
+                        !XMLString::equals(children[index]->getLocalPart(), fFirstChild->getLocalPart())) {
                         return index;
                     }
                 }
@@ -259,16 +244,16 @@ SimpleContentModel::validateContent(QName** const       children
                 return 0;
 
             if (fDTD) {
-                if ((XMLString::compareString(children[0]->getRawName(), fFirstChild->getRawName())!=0) &&
-                    (XMLString::compareString(children[0]->getRawName(), fSecondChild->getRawName())!=0)) {
+                if (!XMLString::equals(children[0]->getRawName(), fFirstChild->getRawName()) &&
+                    !XMLString::equals(children[0]->getRawName(), fSecondChild->getRawName())) {
                     return 0;
                 }
             }
             else {
                 if (((children[0]->getURI() != fFirstChild->getURI()) ||
-                     (XMLString::compareString(children[0]->getLocalPart(), fFirstChild->getLocalPart())!=0)) &&
+                     !XMLString::equals(children[0]->getLocalPart(), fFirstChild->getLocalPart())) &&
                     ((children[0]->getURI() != fSecondChild->getURI()) ||
-                     (XMLString::compareString(children[0]->getLocalPart(), fSecondChild->getLocalPart())!=0))) {
+                     !XMLString::equals(children[0]->getLocalPart(), fSecondChild->getLocalPart()))) {
                     return 0;
                 }
             }
@@ -289,21 +274,21 @@ SimpleContentModel::validateContent(QName** const       children
 
             if (childCount == 2) {
                 if (fDTD) {
-                    if (XMLString::compareString(children[0]->getRawName(), fFirstChild->getRawName())!=0) {
+                    if (!XMLString::equals(children[0]->getRawName(), fFirstChild->getRawName())) {
                         return 0;
                     }
-                    if (XMLString::compareString(children[1]->getRawName(), fSecondChild->getRawName())!=0) {
+                    if (!XMLString::equals(children[1]->getRawName(), fSecondChild->getRawName())) {
                         return 1;
                     }
                 }
                 else {
                     if ((children[0]->getURI() != fFirstChild->getURI()) ||
-                        (XMLString::compareString(children[0]->getLocalPart(), fFirstChild->getLocalPart())!=0)) {
+                        !XMLString::equals(children[0]->getLocalPart(), fFirstChild->getLocalPart())) {
                         return 0;
                     }
 
                     if ((children[1]->getURI() != fSecondChild->getURI()) ||
-                        (XMLString::compareString(children[1]->getLocalPart(), fSecondChild->getLocalPart())!=0)) {
+                        !XMLString::equals(children[1]->getLocalPart(), fSecondChild->getLocalPart())) {
                         return 1;
                     }
                 }
@@ -318,7 +303,7 @@ SimpleContentModel::validateContent(QName** const       children
             break;
 
         default :
-            ThrowXML(RuntimeException, XMLExcepts::CM_UnknownCMSpecType);
+            ThrowXMLwithMemMgr(RuntimeException, XMLExcepts::CM_UnknownCMSpecType, fMemoryManager);
             break;
     }
     return -1;
@@ -326,7 +311,7 @@ SimpleContentModel::validateContent(QName** const       children
 
 int SimpleContentModel::validateContentSpecial(QName** const          children
                                             , const unsigned int      childCount
-                                            , const unsigned int      emptyNamespaceId
+                                            , const unsigned int
                                             , GrammarResolver*  const pGrammarResolver
                                             , XMLStringPool*    const pStringPool) const
 {
@@ -338,7 +323,7 @@ int SimpleContentModel::validateContentSpecial(QName** const          children
     //  content check.
     //
     unsigned int index;
-    switch(fOp)
+    switch(fOp & 0x0f)
     {
         case ContentSpecNode::Leaf :
             //
@@ -349,7 +334,7 @@ int SimpleContentModel::validateContentSpecial(QName** const          children
                 return 0;
 
             if ((children[0]->getURI() != fFirstChild->getURI()) ||
-                (XMLString::compareString(children[0]->getLocalPart(), fFirstChild->getLocalPart())!=0))
+                !XMLString::equals(children[0]->getLocalPart(), fFirstChild->getLocalPart()))
             {
                 if (!comparator.isEquivalentTo(children[0], fFirstChild))
                    return 0;
@@ -367,7 +352,7 @@ int SimpleContentModel::validateContentSpecial(QName** const          children
             //
             if ((childCount == 1) &&
                ((children[0]->getURI() != fFirstChild->getURI()) ||
-                (XMLString::compareString(children[0]->getLocalPart(), fFirstChild->getLocalPart())!=0)))
+                !XMLString::equals(children[0]->getLocalPart(), fFirstChild->getLocalPart())))
             {
                 if(!comparator.isEquivalentTo(children[0], fFirstChild))
                     return 0;
@@ -388,7 +373,7 @@ int SimpleContentModel::validateContentSpecial(QName** const          children
                 for (index = 0; index < childCount; index++)
                 {
                     if ((children[index]->getURI() != fFirstChild->getURI()) ||
-                        (XMLString::compareString(children[index]->getLocalPart(), fFirstChild->getLocalPart())!=0))
+                        !XMLString::equals(children[index]->getLocalPart(), fFirstChild->getLocalPart()))
                     {
     				    if (!comparator.isEquivalentTo(children[index], fFirstChild))
                             return index;
@@ -409,7 +394,7 @@ int SimpleContentModel::validateContentSpecial(QName** const          children
             for (index = 0; index < childCount; index++)
             {
                 if ((children[index]->getURI() != fFirstChild->getURI()) ||
-                    (XMLString::compareString(children[index]->getLocalPart(), fFirstChild->getLocalPart())!=0))
+                    !XMLString::equals(children[index]->getLocalPart(), fFirstChild->getLocalPart()))
                 {
     			    if (!comparator.isEquivalentTo(children[index], fFirstChild))
                         return index;
@@ -426,9 +411,9 @@ int SimpleContentModel::validateContentSpecial(QName** const          children
                 return 0;
 
             if (((children[0]->getURI() != fFirstChild->getURI()) ||
-                 (XMLString::compareString(children[0]->getLocalPart(), fFirstChild->getLocalPart())!=0)) &&
+                 !XMLString::equals(children[0]->getLocalPart(), fFirstChild->getLocalPart())) &&
                 ((children[0]->getURI() != fSecondChild->getURI()) ||
-                 (XMLString::compareString(children[0]->getLocalPart(), fSecondChild->getLocalPart())!=0)))
+                 !XMLString::equals(children[0]->getLocalPart(), fSecondChild->getLocalPart())))
             {
 
                  if (!comparator.isEquivalentTo(children[0], fFirstChild) &&
@@ -453,14 +438,14 @@ int SimpleContentModel::validateContentSpecial(QName** const          children
             if (childCount == 2)
             {
                 if ((children[0]->getURI() != fFirstChild->getURI()) ||
-                    (XMLString::compareString(children[0]->getLocalPart(), fFirstChild->getLocalPart())!=0))
+                    !XMLString::equals(children[0]->getLocalPart(), fFirstChild->getLocalPart()))
                 {
                     if(!comparator.isEquivalentTo(children[0], fFirstChild))
                         return 0;
                 }
 
                 if ((children[1]->getURI() != fSecondChild->getURI()) ||
-                    (XMLString::compareString(children[1]->getLocalPart(), fSecondChild->getLocalPart())!=0))
+                    !XMLString::equals(children[1]->getLocalPart(), fSecondChild->getLocalPart()))
                 {
                     if (!comparator.isEquivalentTo(children[1], fSecondChild))
                         return 1;
@@ -478,7 +463,7 @@ int SimpleContentModel::validateContentSpecial(QName** const          children
             break;
 
         default :
-            ThrowXML(RuntimeException, XMLExcepts::CM_UnknownCMSpecType);
+            ThrowXMLwithMemMgr(RuntimeException, XMLExcepts::CM_UnknownCMSpecType, fMemoryManager);
             break;
     }
     return -1;
@@ -496,6 +481,7 @@ void SimpleContentModel::checkUniqueParticleAttribution
       , XMLStringPool*    const pStringPool
       , XMLValidator*     const pValidator
       , unsigned int*     const pContentSpecOrgURI
+      , const XMLCh*            pComplexTypeName /*= 0*/
     )
 {
     // rename back
@@ -514,7 +500,7 @@ void SimpleContentModel::checkUniqueParticleAttribution
         fSecondChild->setURI(pContentSpecOrgURI[orgURIIndex]);
 
     // only possible violation is when it's a choice
-    if (fOp == ContentSpecNode::Choice) {
+    if ((fOp & 0x0f) == ContentSpecNode::Choice) {
 
         SubstitutionGroupComparator comparator(pGrammarResolver, pStringPool);
 
@@ -526,8 +512,11 @@ void SimpleContentModel::checkUniqueParticleAttribution
                                             &comparator))
 
             pValidator->emitError(XMLValid::UniqueParticleAttributionFail,
+                                  pComplexTypeName,
                                   fFirstChild->getRawName(),
                                   fSecondChild->getRawName());
     }
 }
 
+XERCES_CPP_NAMESPACE_END
+
