@@ -1,61 +1,93 @@
 /*
- * The Apache Software License, Version 1.1
- *
- * Copyright (c) 2001 The Apache Software Foundation.  All rights
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xerces" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written
- *    permission, please contact apache\@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation, and was
- * originally based on software copyright (c) 1999, International
- * Business Machines, Inc., http://www.ibm.com .  For more information
- * on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Copyright 2001,2004 The Apache Software Foundation.
+ * 
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 /*
  * $Log: ComplexTypeInfo.cpp,v $
+ * Revision 1.30  2004/09/23 00:37:24  cargilld
+ * Remove unused variable and data member.
+ *
+ * Revision 1.29  2004/09/16 13:32:04  amassari
+ * Updated error message for UPA to also state the complex type that is failing the test
+ *
+ * Revision 1.28  2004/09/08 13:56:56  peiyongz
+ * Apache License Version 2.0
+ *
+ * Revision 1.27  2004/07/22 15:00:54  knoaman
+ * Eliminate the need to create a temporary content model when performing UPA checking
+ *
+ * Revision 1.26  2004/01/29 11:52:31  cargilld
+ * Code cleanup changes to get rid of various compiler diagnostic messages.
+ *
+ * Revision 1.25  2004/01/13 16:17:09  knoaman
+ * Fo sanity, use class name to qualify method
+ *
+ * Revision 1.24  2004/01/09 22:41:58  knoaman
+ * Use a global static mutex for locking when creating local static mutexes instead of compareAndSwap
+ *
+ * Revision 1.23  2003/12/24 17:42:03  knoaman
+ * Misc. PSVI updates
+ *
+ * Revision 1.22  2003/12/17 00:18:40  cargilld
+ * Update to memory management so that the static memory manager (one used to call Initialize) is only for static data.
+ *
+ * Revision 1.21  2003/12/16 17:18:50  peiyongz
+ * don't expand ContextSpecNode when deserilized
+ *
+ * Revision 1.20  2003/11/20 18:05:16  knoaman
+ * PSVI: Use a copy of the content spec node when creating the content model.
+ *
+ * Revision 1.19  2003/11/13 23:20:47  peiyongz
+ * initSize
+ *
+ * Revision 1.18  2003/11/10 21:54:51  neilg
+ * implementation for new stateless means of traversing attribute definition lists
+ *
+ * Revision 1.17  2003/11/07 17:08:12  knoaman
+ * For PSVI support, distinguish wildcard elements with namespace lists.
+ *
+ * Revision 1.16  2003/10/29 16:22:58  peiyongz
+ * allow proper recreation of ContentModel from deserialization
+ *
+ * Revision 1.15  2003/10/21 10:23:53  amassari
+ * When creating a new list of attributes, use the assigned memory manager
+ *
+ * Revision 1.14  2003/10/17 21:17:12  peiyongz
+ * using XTemplateSerializer
+ *
+ * Revision 1.13  2003/10/14 15:22:28  peiyongz
+ * Implementation of Serialization/Deserialization
+ *
+ * Revision 1.12  2003/10/10 16:25:40  peiyongz
+ * Implementation of Serialization/Deserialization
+ *
+ * Revision 1.11  2003/10/08 21:33:48  peiyongz
+ * Synchronize ContentSpec/ContentModel/FormattedModel
+ *
+ * Revision 1.10  2003/09/26 18:31:05  peiyongz
+ * Synchronize ContentSpecNode and formattedModel
+ *
+ * Revision 1.9  2003/05/18 14:02:07  knoaman
+ * Memory manager implementation: pass per instance manager.
+ *
+ * Revision 1.8  2003/05/16 21:43:21  knoaman
+ * Memory manager implementation: Modify constructors to pass in the memory manager.
+ *
+ * Revision 1.7  2003/05/15 18:57:27  knoaman
+ * Partial implementation of the configurable memory manager.
+ *
  * Revision 1.6  2003/01/29 19:57:30  gareth
  * API to retrive local and uri part of the type name
  *
@@ -170,14 +202,125 @@
 #include <xercesc/validators/common/MixedContentModel.hpp>
 #include <xercesc/validators/common/SimpleContentModel.hpp>
 #include <xercesc/validators/schema/XSDLocator.hpp>
+#include <xercesc/internal/XTemplateSerializer.hpp>
+#include <xercesc/util/XMLRegisterCleanup.hpp>
 
 XERCES_CPP_NAMESPACE_BEGIN
 
+// ---------------------------------------------------------------------------
+//  Local static data
+// ---------------------------------------------------------------------------
+static bool               sAnyTypeMutexRegistered = false;
+static XMLMutex*          sAnyTypeMutex = 0;
+static XMLRegisterCleanup anyTypeCleanup;
+
+
+// ---------------------------------------------------------------------------
+//  ComplexTypeInfo: Static member data
+// ---------------------------------------------------------------------------
+ComplexTypeInfo* ComplexTypeInfo::fAnyType = 0;
+
+
+// ---------------------------------------------------------------------------
+//  ComplexTypeInfo: Static meber methods
+// ---------------------------------------------------------------------------
+void ComplexTypeInfo::reinitAnyType() {
+
+    delete fAnyType;
+    fAnyType = 0;
+
+    // delete local static data
+    delete sAnyTypeMutex;
+    sAnyTypeMutex = 0;
+    sAnyTypeMutexRegistered = false;
+}
+
+ComplexTypeInfo* ComplexTypeInfo::getAnyType(unsigned int emptyNSId)
+{
+    if (!sAnyTypeMutexRegistered)
+    {
+        if (!sAnyTypeMutex)
+        {
+            XMLMutexLock lock(XMLPlatformUtils::fgAtomicMutex);
+            if (!sAnyTypeMutex)
+                sAnyTypeMutex = new XMLMutex;
+        }
+
+        // Use a faux scope to synchronize while we do this
+        {
+            XMLMutexLock lock(sAnyTypeMutex);
+
+            // If we got here first, then register it and set the registered flag
+            if (!sAnyTypeMutexRegistered)
+            {
+                // create type name
+                XMLCh typeName[128];
+                unsigned int nsLen = XMLString::stringLen(
+                    SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+
+			    XMLString::copyString(
+                    typeName, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+                typeName[nsLen] = chComma;
+                XMLString::copyString(
+                    typeName + nsLen + 1, SchemaSymbols::fgATTVAL_ANYTYPE);
+
+                // Create and initialize 'anyType'
+                fAnyType = new ComplexTypeInfo();
+
+                ContentSpecNode* term = new ContentSpecNode
+                (
+                    new QName
+                    (
+                        XMLUni::fgZeroLenString
+                        , XMLUni::fgZeroLenString
+                        , emptyNSId
+                    )
+                    , false
+                );
+                term->setType(ContentSpecNode::Any_Lax);
+                term->setMinOccurs(0);
+                term->setMaxOccurs(SchemaSymbols::XSD_UNBOUNDED);
+
+                ContentSpecNode* particle = new ContentSpecNode
+                (
+                    ContentSpecNode::ModelGroupSequence
+                    , term
+                    , 0
+                );
+
+                SchemaAttDef* attWildCard = new SchemaAttDef
+                (
+                    XMLUni::fgZeroLenString
+                    , XMLUni::fgZeroLenString
+                    , emptyNSId
+                    , XMLAttDef::Any_Any
+                    , XMLAttDef::ProcessContents_Lax
+                );
+
+                fAnyType->setTypeName(typeName);
+                fAnyType->setBaseComplexTypeInfo(fAnyType);
+                fAnyType->setDerivedBy(SchemaSymbols::XSD_RESTRICTION);
+                fAnyType->setContentType(SchemaElementDecl::Mixed_Complex);
+                fAnyType->setContentSpec(particle);
+                fAnyType->setAttWildCard(attWildCard);
+
+                // register cleanup method
+                anyTypeCleanup.registerCleanup(ComplexTypeInfo::reinitAnyType);
+                sAnyTypeMutexRegistered = true;
+            }
+        }
+    }
+
+    return fAnyType;
+}
+
+
 // ---------------------------------------------------------------------------
 //  ComplexTypeInfo: Constructors and Destructor
 // ---------------------------------------------------------------------------
-ComplexTypeInfo::ComplexTypeInfo()
-    : fAbstract(false)
+ComplexTypeInfo::ComplexTypeInfo(MemoryManager* const manager)
+    : fAnonymous(false)
+    , fAbstract(false)
     , fAdoptContentSpec(true)
     , fAttWithTypeId(false)
     , fPreprocessed(false)
@@ -194,18 +337,17 @@ ComplexTypeInfo::ComplexTypeInfo()
     , fDatatypeValidator(0)
     , fBaseComplexTypeInfo(0)
     , fContentSpec(0)
-    , fAttWildCard(0)
-    , fAttDefs(0)
+    , fAttWildCard(0)    
     , fAttList(0)
-    , fElements(0)
+    , fElements(0)    
+    , fAttDefs(0)
     , fContentModel(0)
     , fFormattedModel(0)
     , fContentSpecOrgURI(0)
     , fUniqueURI(0)
-    , fContentSpecOrgURISize(16)
-    , fSpecNodesToDelete(0)
+    , fContentSpecOrgURISize(16)    
     , fLocator(0)
-    , fAnonymous(false)
+    , fMemoryManager(manager)
 {
 
 }
@@ -213,9 +355,9 @@ ComplexTypeInfo::ComplexTypeInfo()
 
 ComplexTypeInfo::~ComplexTypeInfo()
 {
-    delete [] fTypeName;
-    delete [] fTypeLocalName;
-    delete [] fTypeUri;
+    fMemoryManager->deallocate(fTypeName); //delete [] fTypeName;
+    fMemoryManager->deallocate(fTypeLocalName); //delete [] fTypeLocalName;
+    fMemoryManager->deallocate(fTypeUri); //delete [] fTypeUri;
 
     if (fAdoptContentSpec) {
         delete fContentSpec;
@@ -224,13 +366,13 @@ ComplexTypeInfo::~ComplexTypeInfo()
     delete fAttWildCard;
     delete fAttDefs;
     delete fAttList;
-    delete fElements;
-    delete fSpecNodesToDelete;
+    delete fElements;    
     delete fLocator;
 
     delete fContentModel;
-    delete [] fFormattedModel;
-    delete [] fContentSpecOrgURI;
+    fMemoryManager->deallocate(fFormattedModel); //delete [] fFormattedModel;
+    fMemoryManager->deallocate(fContentSpecOrgURI); //delete [] fContentSpecOrgURI;
+
 }
 
 // ---------------------------------------------------------------------------
@@ -247,6 +389,10 @@ void ComplexTypeInfo::addAttDef(SchemaAttDef* const toAdd) {
 
     fAttDefs->put((void*)(toAdd->getAttName()->getLocalPart()),
                           toAdd->getAttName()->getURI(), toAdd);
+    // update and/or create fAttList
+    if(!fAttList)
+        ((ComplexTypeInfo*)this)->fAttList = new (fMemoryManager) SchemaAttDefList(fAttDefs,fMemoryManager);
+    fAttList->addAttDef(toAdd);
 }
 
 void ComplexTypeInfo::setContentSpec(ContentSpecNode* const toAdopt) {
@@ -256,6 +402,9 @@ void ComplexTypeInfo::setContentSpec(ContentSpecNode* const toAdopt) {
     }
 
     fContentSpec = toAdopt;
+
+    //reset Content Model
+    setContentModel(0);
 }
 
 void ComplexTypeInfo::setLocator(XSDLocator* const aLocator) {
@@ -277,7 +426,7 @@ XMLAttDefList& ComplexTypeInfo::getAttDefList() const
         if (!fAttDefs)
             faultInAttDefList();
 
-        ((ComplexTypeInfo*)this)->fAttList = new SchemaAttDefList(fAttDefs);
+        ((ComplexTypeInfo*)this)->fAttList = new (fMemoryManager) SchemaAttDefList(fAttDefs, fMemoryManager);
     }
 
     // Reset it before we return it
@@ -305,7 +454,7 @@ ComplexTypeInfo::getFormattedContentModel() const
 // ---------------------------------------------------------------------------
 //  ComplexTypeInfo: Helper methods
 // ---------------------------------------------------------------------------
-XMLAttDef* ComplexTypeInfo::findAttr(const XMLCh* const qName
+XMLAttDef* ComplexTypeInfo::findAttr(const XMLCh* const
                                      , const unsigned int uriId
                                      , const XMLCh* const baseName
                                      , const XMLCh* const prefix
@@ -326,13 +475,25 @@ XMLAttDef* ComplexTypeInfo::findAttr(const XMLCh* const qName
             faultInAttDefList();
 
         // And add a default attribute for this name
-        retVal = new SchemaAttDef(prefix, baseName, uriId);
+        retVal = new (fMemoryManager) SchemaAttDef
+        (
+            prefix
+            , baseName
+            , uriId
+            , XMLAttDef::CData
+            , XMLAttDef::Implied
+            , fMemoryManager
+        );
         retVal->setElemId(getElementId());
         fAttDefs->put((void*)retVal->getAttName()->getLocalPart(), uriId, retVal);
 
+        // update and/or create fAttList
+        if(!fAttList)
+            ((ComplexTypeInfo*)this)->fAttList = new (fMemoryManager) SchemaAttDefList(fAttDefs,fMemoryManager);
+        fAttList->addAttDef(retVal);
         wasAdded = true;
     }
-     else
+    else
     {
         wasAdded = false;
     }
@@ -350,7 +511,7 @@ bool ComplexTypeInfo::resetDefs() {
     //  This lets the scanner use them to track which has been provided and
     //  which have not.
     //
-    RefHash2KeysTableOfEnumerator<SchemaAttDef> enumDefs(fAttDefs);
+    RefHash2KeysTableOfEnumerator<SchemaAttDef> enumDefs(fAttDefs, false, fMemoryManager);
     while (enumDefs.hasMoreElements())
         enumDefs.nextElement().setProvided(false);
 
@@ -363,16 +524,12 @@ void ComplexTypeInfo::checkUniqueParticleAttribution (SchemaGrammar*    const pG
                                                       XMLStringPool*    const pStringPool,
                                                       XMLValidator*     const pValidator)
 {
-    if (fContentSpec) {
-        ContentSpecNode* specNode = new ContentSpecNode(*fContentSpec);
-        XMLContentModel* cm = makeContentModel(true, specNode);
-
-        if (cm) {
-            cm->checkUniqueParticleAttribution(pGrammar, pGrammarResolver, pStringPool, pValidator, fContentSpecOrgURI);
-            delete cm;
+    if (fContentSpec && !fContentModel)
+    {
+        fContentModel = makeContentModel(true);
+        if (fContentModel) {
+            fContentModel->checkUniqueParticleAttribution(pGrammar, pGrammarResolver, pStringPool, pValidator, fContentSpecOrgURI, fTypeLocalName);
         }
-
-        fSpecNodesToDelete->removeAllElements();
     }
 }
 
@@ -383,7 +540,7 @@ void ComplexTypeInfo::faultInAttDefList() const
 {
     // Use a hash modulus of 29 and tell it owns its elements
     ((ComplexTypeInfo*)this)->fAttDefs =
-                    new RefHash2KeysTableOf<SchemaAttDef>(29, true);
+        new (fMemoryManager) RefHash2KeysTableOf<SchemaAttDef>(29, true, fMemoryManager);
 }
 
 XMLCh* ComplexTypeInfo::formatContentModel() const
@@ -391,13 +548,13 @@ XMLCh* ComplexTypeInfo::formatContentModel() const
     XMLCh* newValue = 0;
     if (fContentType == SchemaElementDecl::Any)
     {
-        newValue = XMLString::replicate(XMLUni::fgAnyString);
+        newValue = XMLString::replicate(XMLUni::fgAnyString, fMemoryManager);
     }
-     else if (fContentType == SchemaElementDecl::Empty)
+    else if (fContentType == SchemaElementDecl::Empty)
     {
-        newValue = XMLString::replicate(XMLUni::fgEmptyString);
+        newValue = XMLString::replicate(XMLUni::fgEmptyString, fMemoryManager);
     }
-     else
+    else
     {
         //
         //  Use a temp XML buffer to format into. Content models could be
@@ -407,43 +564,40 @@ XMLCh* ComplexTypeInfo::formatContentModel() const
         const ContentSpecNode* specNode = fContentSpec;
 
         if (specNode) {
-            XMLBuffer bufFmt;
-
+            XMLBuffer bufFmt(1023, fMemoryManager);
 
             specNode->formatSpec(bufFmt);
-            newValue = XMLString::replicate(bufFmt.getRawBuffer());
+            newValue = XMLString::replicate
+            (
+                bufFmt.getRawBuffer()
+                , fMemoryManager
+            );
         }
     }
     return newValue;
 }
 
-XMLContentModel* ComplexTypeInfo::makeContentModel(const bool checkUPA, ContentSpecNode* const specNode)
+XMLContentModel* ComplexTypeInfo::makeContentModel(const bool checkUPA)
 {
-    if ((specNode || fContentSpec) && !fSpecNodesToDelete) {
-        fSpecNodesToDelete = new RefVectorOf<ContentSpecNode>(8);
+    ContentSpecNode* aSpecNode = new (fMemoryManager) ContentSpecNode(*fContentSpec);
+    XMLContentModel* retModel = 0;
+
+    if (checkUPA) {
+        fContentSpecOrgURI = (unsigned int*) fMemoryManager->allocate
+        (
+            fContentSpecOrgURISize * sizeof(unsigned int)
+        ); //new unsigned int[fContentSpecOrgURISize];
     }
 
-    // expand the content spec first
-    ContentSpecNode* aSpecNode = specNode;
-    if (aSpecNode) {
-
-        fContentSpecOrgURI = new unsigned int[fContentSpecOrgURISize];
-        aSpecNode = convertContentSpecTree(aSpecNode, checkUPA);
-        fSpecNodesToDelete->addElement(aSpecNode);
-    }
-    else {
-        aSpecNode = convertContentSpecTree(fContentSpec, checkUPA);
-        if (aSpecNode != fContentSpec) {
-            if (!fAdoptContentSpec && (aSpecNode == fContentSpec->getFirst()))
-                fAdoptContentSpec = false;
-            else
-                fAdoptContentSpec = true;
-
-            fContentSpec = aSpecNode;
-        }
-    }
+    aSpecNode = convertContentSpecTree(aSpecNode, checkUPA);
+    retModel = buildContentModel(aSpecNode);
 
+    delete aSpecNode;
+    return retModel;
+}
 
+XMLContentModel* ComplexTypeInfo::buildContentModel(ContentSpecNode* const aSpecNode)
+{
     XMLContentModel* cmRet = 0;
     if (fContentType == SchemaElementDecl::Simple) {
        // just return nothing
@@ -454,7 +608,7 @@ XMLContentModel* ComplexTypeInfo::makeContentModel(const bool checkUPA, ContentS
         //  Just create a mixel content model object. This type of
         //  content model is optimized for mixed content validation.
         //
-        cmRet = new MixedContentModel(false, aSpecNode);
+        cmRet = new (fMemoryManager) MixedContentModel(false, aSpecNode, false, fMemoryManager);
     }
     else if (fContentType == SchemaElementDecl::Mixed_Complex) {
 
@@ -473,21 +627,19 @@ XMLContentModel* ComplexTypeInfo::makeContentModel(const bool checkUPA, ContentS
     }
      else
     {
-        ThrowXML(RuntimeException, XMLExcepts::CM_MustBeMixedOrChildren);
+        ThrowXMLwithMemMgr(RuntimeException, XMLExcepts::CM_MustBeMixedOrChildren, fMemoryManager);
     }
 
     return cmRet;
 }
 
-
-
 // ---------------------------------------------------------------------------
 //  SchemaElementDecl: Private helper methods
 // ---------------------------------------------------------------------------
 XMLContentModel* ComplexTypeInfo::createChildModel(ContentSpecNode* specNode, const bool isMixed)
 {
     if(!specNode)
-        ThrowXML(RuntimeException, XMLExcepts::CM_UnknownCMSpecType);
+        ThrowXMLwithMemMgr(RuntimeException, XMLExcepts::CM_UnknownCMSpecType, fMemoryManager);
 
     ContentSpecNode::NodeTypes specType = specNode->getType();
     //
@@ -496,7 +648,7 @@ XMLContentModel* ComplexTypeInfo::createChildModel(ContentSpecNode* specNode, co
     //
     if (specNode->getElement()) {
         if (specNode->getElement()->getURI() == XMLElementDecl::fgPCDataElemId)
-            ThrowXML(RuntimeException, XMLExcepts::CM_NoPCDATAHere);
+            ThrowXMLwithMemMgr(RuntimeException, XMLExcepts::CM_NoPCDATAHere, fMemoryManager);
     }
 
     //
@@ -514,12 +666,12 @@ XMLContentModel* ComplexTypeInfo::createChildModel(ContentSpecNode* specNode, co
             // All the nodes under an ALL must be additional ALL nodes and
             // ELEMENTs (or ELEMENTs under ZERO_OR_ONE nodes.)
             // We collapse the ELEMENTs into a single vector.
-            return new AllContentModel(specNode, true);
+            return new (fMemoryManager) AllContentModel(specNode, true, fMemoryManager);
         }
         else if (specType == ContentSpecNode::ZeroOrOne) {
             // An ALL node can appear under a ZERO_OR_ONE node.
             if (specNode->getFirst()->getType() == ContentSpecNode::All) {
-                return new AllContentModel(specNode->getFirst(), true);
+                return new (fMemoryManager) AllContentModel(specNode->getFirst(), true, fMemoryManager);
             }
         }
 
@@ -528,16 +680,17 @@ XMLContentModel* ComplexTypeInfo::createChildModel(ContentSpecNode* specNode, co
      else if (specType == ContentSpecNode::Leaf)
     {
         // Create a simple content model
-        return new SimpleContentModel
+        return new (fMemoryManager) SimpleContentModel
         (
             false
             , specNode->getElement()
             , 0
             , ContentSpecNode::Leaf
+            , fMemoryManager
         );
     }
-     else if ((specType == ContentSpecNode::Choice)
-          ||  (specType == ContentSpecNode::Sequence))
+     else if (((specType & 0x0f) == ContentSpecNode::Choice)
+          ||  ((specType & 0x0f) == ContentSpecNode::Sequence))
     {
         //
         //  Lets see if both of the children are leafs. If so, then it has to
@@ -547,12 +700,13 @@ XMLContentModel* ComplexTypeInfo::createChildModel(ContentSpecNode* specNode, co
         &&  (specNode->getSecond())
         &&  (specNode->getSecond()->getType() == ContentSpecNode::Leaf))
         {
-            return new SimpleContentModel
+            return new (fMemoryManager) SimpleContentModel
             (
                 false
                 , specNode->getFirst()->getElement()
                 , specNode->getSecond()->getElement()
                 , specType
+                , fMemoryManager
             );
         }
     }
@@ -567,28 +721,29 @@ XMLContentModel* ComplexTypeInfo::createChildModel(ContentSpecNode* specNode, co
         //
         if (specNode->getFirst()->getType() == ContentSpecNode::Leaf)
         {
-            return new SimpleContentModel
+            return new (fMemoryManager) SimpleContentModel
             (
                 false
                 , specNode->getFirst()->getElement()
                 , 0
                 , specType
+                , fMemoryManager
             );
         }
         else if (specNode->getFirst()->getType() == ContentSpecNode::All)
-            return new AllContentModel(specNode->getFirst(), false);
+            return new (fMemoryManager) AllContentModel(specNode->getFirst(), false, fMemoryManager);
 
     }
     else if (specType == ContentSpecNode::All)
-        return new AllContentModel(specNode, false);
+        return new (fMemoryManager) AllContentModel(specNode, false, fMemoryManager);
 
     else
     {
-        ThrowXML(RuntimeException, XMLExcepts::CM_UnknownCMSpecType);
+        ThrowXMLwithMemMgr(RuntimeException, XMLExcepts::CM_UnknownCMSpecType, fMemoryManager);
     }
 
     // Its not any simple type of content, so create a DFA based content model
-    return new DFAContentModel(false, specNode, isMixed);
+    return new (fMemoryManager) DFAContentModel(false, specNode, isMixed, fMemoryManager);
 }
 
 ContentSpecNode*
@@ -603,12 +758,14 @@ ComplexTypeInfo::convertContentSpecTree(ContentSpecNode* const curNode,
     // When checking Unique Particle Attribution, rename leaf elements
     if (checkUPA) {
         if (curNode->getElement()) {
+            if (fUniqueURI == fContentSpecOrgURISize) {
+                resizeContentSpecOrgURI();
+            }
+
             fContentSpecOrgURI[fUniqueURI] = curNode->getElement()->getURI();
             curNode->getElement()->setURI(fUniqueURI);
             fUniqueURI++;
         }
-        if (fUniqueURI == fContentSpecOrgURISize)
-            resizeContentSpecOrgURI();
     }
 
     // Get the spec type of the passed node
@@ -623,9 +780,9 @@ ComplexTypeInfo::convertContentSpecTree(ContentSpecNode* const curNode,
     {
         retNode =  expandContentModel(curNode, minOccurs, maxOccurs);
     }
-    else if ((curType == ContentSpecNode::Choice)
+    else if (((curType & 0x0f) == ContentSpecNode::Choice)
         ||   (curType == ContentSpecNode::All)
-        ||   (curType == ContentSpecNode::Sequence))
+        ||   ((curType & 0x0f) == ContentSpecNode::Sequence))
     {
         ContentSpecNode* childNode = curNode->getFirst();
         ContentSpecNode* leftNode = convertContentSpecTree(childNode, checkUPA);
@@ -677,47 +834,114 @@ ContentSpecNode* ComplexTypeInfo::expandContentModel(ContentSpecNode* const spec
     }
     else if (minOccurs == 0 && maxOccurs == 1) {
 
-        retNode = new ContentSpecNode(ContentSpecNode::ZeroOrOne, retNode, 0);
+        retNode = new (fMemoryManager) ContentSpecNode
+        (
+            ContentSpecNode::ZeroOrOne
+            , retNode
+            , 0
+            , true
+            , true
+            , fMemoryManager
+        );
     }
     else if (minOccurs == 0 && maxOccurs == -1) {
-        retNode = new ContentSpecNode(ContentSpecNode::ZeroOrMore, retNode, 0);
+        retNode = new (fMemoryManager) ContentSpecNode
+        (
+            ContentSpecNode::ZeroOrMore
+            , retNode
+            , 0
+            , true
+            , true
+            , fMemoryManager
+        );
     }
     else if (minOccurs == 1 && maxOccurs == -1) {
-        retNode = new ContentSpecNode(ContentSpecNode::OneOrMore, retNode, 0);
+        retNode = new (fMemoryManager) ContentSpecNode
+        (
+            ContentSpecNode::OneOrMore
+            , retNode
+            , 0
+            , true
+            , true
+            , fMemoryManager
+        );
     }
     else if (maxOccurs == -1) {
 
-        retNode = new ContentSpecNode(ContentSpecNode::OneOrMore, retNode, 0);
+        retNode = new (fMemoryManager) ContentSpecNode
+        (
+            ContentSpecNode::OneOrMore
+            , retNode
+            , 0
+            , true
+            , true
+            , fMemoryManager
+        );
 
         for (int i=0; i < (int)(minOccurs-1); i++) {
-            retNode = new ContentSpecNode(ContentSpecNode::Sequence,
-                                          saveNode, retNode, false);
+            retNode = new (fMemoryManager) ContentSpecNode
+            (
+                ContentSpecNode::Sequence
+                , saveNode
+                , retNode
+                , false
+                , true
+                , fMemoryManager
+            );
         }
     }
     else {
 
         if (minOccurs == 0) {
 
-            ContentSpecNode* optional =
-                new ContentSpecNode(ContentSpecNode::ZeroOrOne, saveNode, 0);
+            ContentSpecNode* optional = new (fMemoryManager) ContentSpecNode
+            (
+                ContentSpecNode::ZeroOrOne
+                , saveNode
+                , 0
+                , true
+                , true
+                , fMemoryManager
+            );
 
             retNode = optional;
 
             for (int i=0; i < (int)(maxOccurs-minOccurs-1); i++) {
-                retNode = new ContentSpecNode(ContentSpecNode::Sequence,
-                                              retNode, optional, true, false);
+                retNode = new (fMemoryManager) ContentSpecNode
+                (
+                    ContentSpecNode::Sequence
+                    , retNode
+                    , optional
+                    , true
+                    , false
+                    , fMemoryManager
+                );
             }
         }
         else {
 
             if (minOccurs > 1) {
 
-                retNode = new ContentSpecNode(ContentSpecNode::Sequence,
-                                              retNode, saveNode, true, false);
+                retNode = new (fMemoryManager) ContentSpecNode
+                (
+                    ContentSpecNode::Sequence
+                    , retNode
+                    , saveNode
+                    , true
+                    , false
+                    , fMemoryManager
+                );
 
                 for (int i=1; i < (int)(minOccurs-1); i++) {
-                    retNode = new ContentSpecNode(ContentSpecNode::Sequence,
-                                                  retNode, saveNode, true, false);
+                    retNode = new (fMemoryManager) ContentSpecNode
+                    (
+                        ContentSpecNode::Sequence
+                        , retNode
+                        , saveNode
+                        , true
+                        , false
+                        , fMemoryManager
+                    );
                 }
             }
 
@@ -725,14 +949,37 @@ ContentSpecNode* ComplexTypeInfo::expandContentModel(ContentSpecNode* const spec
 
             if (counter > 0) {
 
-                ContentSpecNode* optional = new ContentSpecNode(ContentSpecNode::ZeroOrOne, saveNode, 0, false);
+                ContentSpecNode* optional = new (fMemoryManager) ContentSpecNode
+                (
+                    ContentSpecNode::ZeroOrOne
+                    , saveNode
+                    , 0
+                    , false
+                    , true
+                    , fMemoryManager
+                );
 
-                retNode = new ContentSpecNode(ContentSpecNode::Sequence, retNode, optional);
+                retNode = new (fMemoryManager) ContentSpecNode
+                (
+                    ContentSpecNode::Sequence
+                    , retNode
+                    , optional
+                    , true
+                    , true
+                    , fMemoryManager
+                );
 
                 for (int j=1; j < counter; j++) {
 
-                    retNode = new ContentSpecNode(ContentSpecNode::Sequence,
-					                              retNode, optional, true, false);
+                    retNode = new (fMemoryManager) ContentSpecNode
+                    (
+                        ContentSpecNode::Sequence
+					    , retNode
+                        , optional
+                        , true
+                        , false
+                        , fMemoryManager
+                    );
                 }
             }
         }
@@ -744,7 +991,10 @@ ContentSpecNode* ComplexTypeInfo::expandContentModel(ContentSpecNode* const spec
 void ComplexTypeInfo::resizeContentSpecOrgURI() {
 
     unsigned int newSize = fContentSpecOrgURISize * 2;
-    unsigned int* newContentSpecOrgURI = new unsigned int[newSize];
+    unsigned int* newContentSpecOrgURI = (unsigned int*) fMemoryManager->allocate
+    (
+        newSize * sizeof(unsigned int)
+    ); //new unsigned int[newSize];
 
     // Copy the existing values
     unsigned int index = 0;
@@ -755,11 +1005,120 @@ void ComplexTypeInfo::resizeContentSpecOrgURI() {
         newContentSpecOrgURI[index] = 0;
 
     // Delete the old array and udpate our members
-    delete [] fContentSpecOrgURI;
+    fMemoryManager->deallocate(fContentSpecOrgURI); //delete [] fContentSpecOrgURI;
     fContentSpecOrgURI = newContentSpecOrgURI;
     fContentSpecOrgURISize = newSize;
 }
 
+/***
+ * Support for Serialization/De-serialization
+ ***/
+
+IMPL_XSERIALIZABLE_TOCREATE(ComplexTypeInfo)
+
+void ComplexTypeInfo::serialize(XSerializeEngine& serEng)
+{
+   
+    if (serEng.isStoring())
+    {    
+        serEng<<fAnonymous;
+        serEng<<fAbstract;
+        serEng<<fAdoptContentSpec;
+        serEng<<fAttWithTypeId;
+        serEng<<fPreprocessed;
+        serEng<<fDerivedBy;
+        serEng<<fBlockSet;
+        serEng<<fFinalSet;
+        serEng<<fScopeDefined;
+        serEng<<fElementId;
+        serEng<<fContentType;
+
+        serEng.writeString(fTypeName);
+        serEng.writeString(fTypeLocalName);
+        serEng.writeString(fTypeUri);
+
+        DatatypeValidator::storeDV(serEng, fBaseDatatypeValidator);
+        DatatypeValidator::storeDV(serEng, fDatatypeValidator);
+
+        serEng<<fBaseComplexTypeInfo;
+        serEng<<fContentSpec;
+        serEng<<fAttWildCard;
+        serEng<<fAttList;
+
+        /***
+         * 
+         * Serialize RefVectorOf<SchemaElementDecl>*    fElements;
+         * Serialize RefHash2KeysTableOf<SchemaAttDef>* fAttDefs;
+         ***/
+        XTemplateSerializer::storeObject(fElements, serEng);
+        XTemplateSerializer::storeObject(fAttDefs, serEng);
+
+         /***
+          *   Don't serialize 
+          *
+          *   fContentModel;
+          *   fFormattedModel;
+          *   fLocator;          
+          * 
+          *   fContentSpecOrgURI:     start of the array
+          *   fContentSpecOrgURISize: size of the array
+          *   fUniqueURI:             the current last element in the array
+          ***/
+    }
+    else
+    {
+        serEng>>fAnonymous;
+        serEng>>fAbstract;
+        serEng>>fAdoptContentSpec;
+        serEng>>fAttWithTypeId;
+        serEng>>fPreprocessed;
+        serEng>>fDerivedBy;
+        serEng>>fBlockSet;
+        serEng>>fFinalSet;
+        serEng>>fScopeDefined;
+        serEng>>fElementId;
+        serEng>>fContentType;
+
+        serEng.readString(fTypeName);
+        serEng.readString(fTypeLocalName);
+        serEng.readString(fTypeUri);
+
+        fBaseDatatypeValidator = DatatypeValidator::loadDV(serEng);
+        fDatatypeValidator     = DatatypeValidator::loadDV(serEng);
+
+        serEng>>fBaseComplexTypeInfo;
+        serEng>>fContentSpec;
+        serEng>>fAttWildCard;
+        serEng>>fAttList;
+
+        /***
+         * 
+         * Deserialize RefVectorOf<SchemaElementDecl>*    fElements;
+         * Deserialize RefHash2KeysTableOf<SchemaAttDef>* fAttDefs;
+         ***/
+        XTemplateSerializer::loadObject(&fElements, 8, false, serEng);
+        XTemplateSerializer::loadObject(&fAttDefs, 29, true, serEng);
+
+         /***
+          *   Don't deserialize 
+          *
+          *   fFormattedModel;
+          *   fLocator;          
+          * 
+          *   fContentSpecOrgURI:     start of the array
+          *   fContentSpecOrgURISize: size of the array
+          *   fUniqueURI:             the current last element in the array
+          ***/
+
+         fFormattedModel = 0;
+         fLocator = 0;         
+         fContentSpecOrgURI = 0;
+         fContentSpecOrgURISize = 0;
+         fUniqueURI = 0;
+    }
+}
+
+
 XERCES_CPP_NAMESPACE_END
 
 /**
