@@ -1,61 +1,53 @@
 /*
- * The Apache Software License, Version 1.1
- *
- * Copyright (c) 2001 The Apache Software Foundation.  All rights
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xerces" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written
- *    permission, please contact apache\@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation, and was
- * originally based on software copyright (c) 2001, International
- * Business Machines, Inc., http://www.ibm.com .  For more information
- * on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Copyright 2001,2004 The Apache Software Foundation.
+ * 
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 /*
  * $Log: SchemaAttDef.hpp,v $
+ * Revision 1.14  2004/09/20 14:47:13  amassari
+ * Mark some methods as deprecated
+ *
+ * Revision 1.13  2004/09/08 13:56:56  peiyongz
+ * Apache License Version 2.0
+ *
+ * Revision 1.12  2004/01/29 11:52:31  cargilld
+ * Code cleanup changes to get rid of various compiler diagnostic messages.
+ *
+ * Revision 1.11  2003/12/24 17:42:03  knoaman
+ * Misc. PSVI updates
+ *
+ * Revision 1.10  2003/12/17 20:50:35  knoaman
+ * PSVI: fix for annotation of attributes in attributeGroup/derived types
+ *
+ * Revision 1.9  2003/11/24 05:13:41  neilg
+ * update method documentation
+ *
+ * Revision 1.8  2003/11/21 22:34:46  neilg
+ * More schema component model implementation, thanks to David Cargill.
+ * In particular, this cleans up and completes the XSModel, XSNamespaceItem,
+ * XSAttributeDeclaration and XSAttributeGroup implementations.
+ *
+ * Revision 1.7  2003/10/10 16:25:40  peiyongz
+ * Implementation of Serialization/Deserialization
+ *
+ * Revision 1.6  2003/05/18 14:02:07  knoaman
+ * Memory manager implementation: pass per instance manager.
+ *
+ * Revision 1.5  2003/05/16 21:43:21  knoaman
+ * Memory manager implementation: Modify constructors to pass in the memory manager.
+ *
  * Revision 1.4  2003/02/06 13:51:55  gareth
  * fixed bug with multiple attributes being validated by the same union type.
  *
@@ -96,11 +88,12 @@
 #include <xercesc/validators/datatype/DatatypeValidator.hpp>
 #include <xercesc/validators/datatype/UnionDatatypeValidator.hpp>
 #include <xercesc/validators/schema/PSVIDefs.hpp>
+
 XERCES_CPP_NAMESPACE_BEGIN
 
 class DatatypeValidator;
 class QName;
-
+class ComplexTypeInfo;
 //
 //  This class is a derivative of the core XMLAttDef class. This class adds
 //  any Schema specific data members and provides Schema specific implementations
@@ -112,24 +105,26 @@ public :
     // -----------------------------------------------------------------------
     //  Constructors and Destructors
     // -----------------------------------------------------------------------
-    SchemaAttDef();
+    SchemaAttDef(MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager);
     SchemaAttDef
     (
-          const XMLCh* const                     prefix
-        , const XMLCh* const                     localPart
-        , const int                              uriId
-        , const XMLAttDef::AttTypes              type = CData
-        , const XMLAttDef::DefAttTypes           defType = Implied
+          const XMLCh* const           prefix
+        , const XMLCh* const           localPart
+        , const int                    uriId
+        , const XMLAttDef::AttTypes    type = CData
+        , const XMLAttDef::DefAttTypes defType = Implied
+        , MemoryManager* const         manager = XMLPlatformUtils::fgMemoryManager
     );
     SchemaAttDef
     (
-          const XMLCh* const                     prefix
-        , const XMLCh* const                     localPart
-        , const int                              uriId
-        , const XMLCh* const                     attValue
-        , const XMLAttDef::AttTypes              type
-        , const XMLAttDef::DefAttTypes           defType
-        , const XMLCh* const                     enumValues = 0
+          const XMLCh* const           prefix
+        , const XMLCh* const           localPart
+        , const int                    uriId
+        , const XMLCh* const           attValue
+        , const XMLAttDef::AttTypes    type
+        , const XMLAttDef::DefAttTypes defType
+        , const XMLCh* const           enumValues = 0
+        , MemoryManager* const         manager = XMLPlatformUtils::fgMemoryManager
     );
     SchemaAttDef
     (
@@ -147,7 +142,7 @@ public :
     // ----------------------------------------------------------------------
     // Partial implementation of PSVI
     // The values these methods return are only accurate until the DOMAttr
-    // is created that uses the values. After this a celan up method is called
+    // is created that uses the values. After this a clean up method is called
     // and the SchemaAttDef may be used again.
     // note that some of this information has dependancies. For example,
     // if something is not valid then the information returned by the other 
@@ -161,6 +156,7 @@ public :
      * 1.1 If it was valid as defined by Attribute Locally Valid (3.2.4), then valid;
      * 1.2 otherwise invalid.
      * 2 otherwise notKnown.
+     * @deprecated
      */
     PSVIDefs::Validity getValidity() const;
 
@@ -168,26 +164,31 @@ public :
      * The appropriate case among the following:
      * 1 If it was strictly assessed, then full;
      * 2 otherwise none.
+     * @deprecated
      */
     PSVIDefs::Validation getValidationAttempted() const;
 
     /**
      * @return the complexity. Always simple for attrs
+     * @deprecated
      */
     PSVIDefs::Complexity getTypeType() const;
 
     /**
      * The target namespace of the type definition.
+     * @deprecated
      */
     const XMLCh* getTypeUri() const;
 
     /**
      * The {name} of the type definition, if it is not absent. 
+     * @deprecated
      */
     const XMLCh* getTypeName() const;
 
     /**
      * true if the {name} of the type definition is absent, otherwise false.
+     * @deprecated
      */
     bool getTypeAnonymous() const;
 
@@ -195,25 +196,35 @@ public :
      * If this method returns true and validity is VALID then the next three 
      * produce accurate results
      * @return true if the element is validated using a union type
+     * @deprecated
      */
     bool isTypeDefinitionUnion() const;
 
     /**
      * The {target namespace} of the actual member type definition.
+     * @deprecated
      */
     const XMLCh* getMemberTypeUri() const;
 
     /**
      * @return true if the {name} of the actual member type definition is absent, otherwise false.
+     * @deprecated
      */
     bool getMemberTypeAnonymous() const;
 
     /**
      * @return the {name} of the actual member type definition, if it is not absent. 
+     * @deprecated
      */
     const XMLCh* getMemberTypeName() const;
 
+    /*
+     * @deprecated
+     */
     virtual const XMLCh* getDOMTypeInfoUri() const;
+    /*
+     * @deprecated
+     */
     virtual const XMLCh* getDOMTypeInfoName() const;
     
     
@@ -225,6 +236,14 @@ public :
     QName* getAttName() const;
     DatatypeValidator* getDatatypeValidator() const;
     ValueVectorOf<unsigned int>* getNamespaceList() const;
+    const SchemaAttDef* getBaseAttDecl() const;
+    SchemaAttDef* getBaseAttDecl();
+    PSVIDefs::PSVIScope getPSVIScope() const;
+
+    /*
+     * @deprecated
+     */
+    ComplexTypeInfo* getEnclosingCT() const;
 
     // -----------------------------------------------------------------------
     //  Setter methods
@@ -237,14 +256,40 @@ public :
        ,const int                 uriId = -1
     );
     void setDatatypeValidator(DatatypeValidator* newDatatypeValidator);
+    /*
+     * @deprecated
+     */
     void setAnyDatatypeValidator(DatatypeValidator* newDatatypeValidator);
+    void setBaseAttDecl(SchemaAttDef* const attDef);
+    void setPSVIScope(const PSVIDefs::PSVIScope toSet);
+
+    /*
+     * @deprecated
+     */
     void setMembertypeValidator(const DatatypeValidator* newDatatypeValidator);
     void setNamespaceList(const ValueVectorOf<unsigned int>* const toSet);
     void resetNamespaceList();
+    /*
+     * @deprecated
+     */
     void setValidity(PSVIDefs::Validity valid);
+    /*
+     * @deprecated
+     */
     void setValidationAttempted(PSVIDefs::Validation validation);
+    void setEnclosingCT(ComplexTypeInfo* complexTypeInfo);
+    /***
+     * Support for Serialization/De-serialization
+     ***/
+    DECL_XSERIALIZABLE(SchemaAttDef)
 
 private :
+    // -----------------------------------------------------------------------
+    //  Unimplemented constructors and operators
+    // -----------------------------------------------------------------------
+    SchemaAttDef(const SchemaAttDef&);
+    SchemaAttDef& operator=(const SchemaAttDef&);
+
     // -----------------------------------------------------------------------
     //  Private data members
     //
@@ -276,6 +321,13 @@ private :
     //  fValidation
     //      The type of validation that happened to this attr
     //
+    //  fBaseAttDecl
+    //      The base attribute declaration that this attribute is based on
+    //      NOTE: we do not have a notion of attribute use, so in the case
+    //      of ref'd attributes and inherited attributes, we make a copy
+    //      of the actual attribute declaration. The fBaseAttDecl stores that
+    //      declaration, and will be helpful when we build the XSModel (i.e
+    //      easy access the XSAnnotation object).
     // -----------------------------------------------------------------------
     unsigned int                 fElemId;
     QName*                       fAttName;
@@ -285,6 +337,8 @@ private :
     ValueVectorOf<unsigned int>* fNamespaceList;
     PSVIDefs::Validity           fValidity;
     PSVIDefs::Validation         fValidation;
+    PSVIDefs::PSVIScope          fPSVIScope;
+    SchemaAttDef*                fBaseAttDecl;
 };
 
 
@@ -417,6 +471,25 @@ SchemaAttDef::getNamespaceList() const {
     return fNamespaceList;
 }
 
+inline ComplexTypeInfo* SchemaAttDef::getEnclosingCT() const
+{
+    return 0;
+}
+
+inline SchemaAttDef* SchemaAttDef::getBaseAttDecl()
+{
+    return fBaseAttDecl;
+}
+
+inline const SchemaAttDef* SchemaAttDef::getBaseAttDecl() const
+{
+    return fBaseAttDecl;
+}
+
+inline PSVIDefs::PSVIScope SchemaAttDef::getPSVIScope() const
+{
+    return fPSVIScope;
+}
 
 // ---------------------------------------------------------------------------
 //  SchemaAttDef: Setter methods
@@ -456,7 +529,7 @@ inline void SchemaAttDef::setNamespaceList(const ValueVectorOf<unsigned int>* co
             *fNamespaceList = *toSet;
         }
         else {
-            fNamespaceList = new ValueVectorOf<unsigned int>(*toSet);
+            fNamespaceList = new (getMemoryManager()) ValueVectorOf<unsigned int>(*toSet);
         }
     }
     else  {
@@ -475,6 +548,20 @@ inline void SchemaAttDef::reset() {
     fValidation = PSVIDefs::NONE;    
 }
 
+inline void SchemaAttDef::setEnclosingCT(ComplexTypeInfo*)
+{
+}
+
+inline void SchemaAttDef::setBaseAttDecl(SchemaAttDef* const attDef)
+{
+    fBaseAttDecl = attDef;
+}
+
+inline void SchemaAttDef::setPSVIScope(const PSVIDefs::PSVIScope toSet)
+{
+    fPSVIScope = toSet;
+}
+
 XERCES_CPP_NAMESPACE_END
 
 #endif
