@@ -1,61 +1,21 @@
 /*
- * The Apache Software License, Version 1.1
- *
- * Copyright (c) 2001-2002 The Apache Software Foundation.  All rights
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xerces" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written
- *    permission, please contact apache\@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation, and was
- * originally based on software copyright (c) 2001, International
- * Business Machines, Inc., http://www.ibm.com .  For more information
- * on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Copyright 2001-2002,2004 The Apache Software Foundation.
+ * 
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 /*
- * $Id: TraverseSchema.hpp,v 1.21 2003/01/15 15:32:15 knoaman Exp $
+ * $Id: TraverseSchema.hpp,v 1.37 2004/09/28 02:14:14 cargilld Exp $
  */
 
 #if !defined(TRAVERSESCHEMA_HPP)
@@ -82,6 +42,7 @@
 #include <xercesc/validators/schema/SchemaInfo.hpp>
 #include <xercesc/validators/schema/GeneralAttributeCheck.hpp>
 #include <xercesc/validators/schema/XSDErrorReporter.hpp>
+#include <xercesc/util/XMLResourceIdentifier.hpp>
 
 XERCES_CPP_NAMESPACE_BEGIN
 
@@ -107,7 +68,7 @@ class XSDDOMParser;
 class XMLErrorReporter;
 
 
-class VALIDATORS_EXPORT TraverseSchema
+class VALIDATORS_EXPORT TraverseSchema : public XMemory
 {
 public:
     // -----------------------------------------------------------------------
@@ -123,6 +84,7 @@ public:
         , const XMLCh* const      schemaURL
         , XMLEntityHandler* const entityHandler
         , XMLErrorReporter* const errorReporter
+        , MemoryManager* const    manager = XMLPlatformUtils::fgMemoryManager
     );
 
     ~TraverseSchema();
@@ -151,7 +113,7 @@ private:
     //  Unimplemented constructors and operators
     // -----------------------------------------------------------------------
     TraverseSchema(const TraverseSchema&);
-    void operator=(const TraverseSchema&);
+    TraverseSchema& operator=(const TraverseSchema&);
 
     // -----------------------------------------------------------------------
     //  Init/CleanUp methods
@@ -169,7 +131,8 @@ private:
     void                preprocessSchema(DOMElement* const schemaRoot,
                                          const XMLCh* const schemaURL);
     void                traverseSchemaHeader(const DOMElement* const schemaRoot);
-    void                traverseAnnotationDecl(const DOMElement* const childElem,
+    XSAnnotation*       traverseAnnotationDecl(const DOMElement* const childElem,
+                                               ValueVectorOf<DOMNode*>* const nonXSAttList,
                                                const bool topLevel = false);
     void                traverseInclude(const DOMElement* const childElem);
     void                traverseImport(const DOMElement* const childElem);
@@ -180,11 +143,13 @@ private:
     void                traverseSimpleContentDecl(const XMLCh* const typeName,
                                                   const XMLCh* const qualifiedName,
                                                   const DOMElement* const contentDecl,
-                                                  ComplexTypeInfo* const typeInfo);
+                                                  ComplexTypeInfo* const typeInfo,
+                                                  Janitor<XSAnnotation>* const janAnnot);
     void                traverseComplexContentDecl(const XMLCh* const typeName,
                                                   const DOMElement* const contentDecl,
                                                   ComplexTypeInfo* const typeInfo,
-                                                  const bool isMixed);
+                                                  const bool isMixed,
+                                                  Janitor<XSAnnotation>* const janAnnot);
     DatatypeValidator*  traverseSimpleTypeDecl(const DOMElement* const childElem,
                                                const bool topLevel = true,
                                                int baseRefContext = SchemaSymbols::XSD_EMPTYSET);
@@ -195,19 +160,22 @@ private:
                                        const DOMElement* const contentElem,
                                        const XMLCh* const typeName,
                                        const XMLCh* const qualifiedName,
-                                       const int finalSet);
+                                       const int finalSet,
+                                       Janitor<XSAnnotation>* const janAnnot);
     DatatypeValidator*  traverseByRestriction(const DOMElement* const rootElem,
                                               const DOMElement* const contentElem,
                                               const XMLCh* const typeName,
                                               const XMLCh* const qualifiedName,
-                                              const int finalSet);
+                                              const int finalSet,
+                                              Janitor<XSAnnotation>* const janAnnot);
     DatatypeValidator*  traverseByUnion(const DOMElement* const rootElem,
                                         const DOMElement* const contentElem,
                                         const XMLCh* const typeName,
                                         const XMLCh* const qualifiedName,
                                         const int finalSet,
-                                        int baseRefContext);
-    QName*              traverseElementDecl(const DOMElement* const childElem,
+                                        int baseRefContext,
+                                        Janitor<XSAnnotation>* const janAnnot);
+    SchemaElementDecl*    traverseElementDecl(const DOMElement* const childElem,
                                             const bool topLevel = false);
     const XMLCh*        traverseNotationDecl(const DOMElement* const childElem);
     const XMLCh*        traverseNotationDecl(const DOMElement* const childElem,
@@ -349,8 +317,19 @@ private:
     /**
       * Process a 'ref' of an Element declaration
       */
-    QName* processElementDeclRef(const DOMElement* const elem,
-                                 const XMLCh* const refName);
+    SchemaElementDecl* processElementDeclRef(const DOMElement* const elem,
+                                             const XMLCh* const refName);
+    void processElemDeclAttrs(const DOMElement* const elem,
+                              SchemaElementDecl* const elemDecl,
+                              const XMLCh*& valConstraint,
+                              bool isTopLevel = false);
+    void processElemDeclIC(DOMElement* const elem,
+                           SchemaElementDecl* const elemDecl);
+    bool checkElemDeclValueConstraint(const DOMElement* const elem,
+                                      SchemaElementDecl* const elemDecl,
+                                      const XMLCh* const valConstraint,
+                                      ComplexTypeInfo* const typeInfo,
+                                      DatatypeValidator* const validator);
 
     /**
       * Process a 'ref' of an Attribute declaration
@@ -386,11 +365,6 @@ private:
       */
     bool isIdentityConstraintName(const XMLCh* const constraintName);
 
-    /**
-      * Check a 'ref' declaration representation constraint
-      */
-    bool isValidRefDeclaration(const DOMElement* const elem);
-
     /**
       * If 'typeStr' belongs to a different schema, return that schema URI,
       * otherwise return 0;
@@ -412,17 +386,14 @@ private:
       * the type is a complex type
       */
     ComplexTypeInfo* getElementComplexTypeInfo(const DOMElement* const elem,
-                                               const XMLCh* const typeStr,
-                                               bool& noErrorDetected,
+                                               const XMLCh* const typeStr,                                               
                                                const XMLCh* const otherSchemaURI);
 
     /**
-      * Return schema element declaration for a given substituteGroup element
-      * name
+      * Return global schema element declaration for a given element name
       */
-    SchemaElementDecl* getSubstituteGroupElemDecl(const DOMElement* const elem,
-                                                  const XMLCh* const name,
-                                                  bool& noErrorDetected);
+    SchemaElementDecl* getGlobalElemDecl(const DOMElement* const elem,
+                                         const XMLCh* const name);
 
     /**
       * Check validity constraint of a substitutionGroup attribute in
@@ -438,14 +409,20 @@ private:
     bool isSubstitutionGroupCircular(SchemaElementDecl* const elemDecl,
                                      SchemaElementDecl* const subsElemDecl);
 
+    void processSubstitutionGroup(const DOMElement* const elem,
+                                  SchemaElementDecl* elemDecl,
+                                  ComplexTypeInfo*& typeInfo,
+                                  DatatypeValidator*& validator,
+                                  const XMLCh* const subsElemQName);
+
     /**
       * Create a 'SchemaElementDecl' object and add it to SchemaGrammar
       */
     SchemaElementDecl* createSchemaElementDecl(const DOMElement* const elem,
-                                               const bool topLevel,
-                                               const unsigned short elemType,
+                                               const XMLCh* const name,
                                                bool& isDuplicate,
-                                               const bool isFixedValue);
+                                               const XMLCh*& valConstraint,
+                                               const bool topLevel);
 
     /**
       * Return the value of a given attribute name from an element node
@@ -464,10 +441,8 @@ private:
     void processComplexContent(const DOMElement* const elem,
                                const XMLCh* const typeName,
                                const DOMElement* const childElem,
-                               ComplexTypeInfo* const typeInfo,
-                               const XMLCh* const baseRawName,
-                               const XMLCh* const baseLocalPart,
-                               const XMLCh* const baseURI,
+                               ComplexTypeInfo* const typeInfo,                               
+                               const XMLCh* const baseLocalPart,                               
                                const bool isMixed,
                                const bool isBaseAnyType = false);
 
@@ -492,6 +467,11 @@ private:
                                        const XMLCh* const uriStr,
                                        const XMLCh* const localPart);
 
+    DatatypeValidator*
+    getAttrDatatypeValidatorNS(const DOMElement* const elem,
+                               const XMLCh* localPart,
+                               const XMLCh* typeURI);
+
     /**
       * Returns true if a DOM Element is an attribute or attribute group
       */
@@ -501,10 +481,7 @@ private:
       * Process attributes of a complex type
       */
     void processAttributes(const DOMElement* const elem,
-                           const DOMElement* const attElem,
-                           const XMLCh* const baseRawName,
-                           const XMLCh* const baseLocalPart,
-                           const XMLCh* const baseURI,
+                           const DOMElement* const attElem,                           
                            ComplexTypeInfo* const typeInfo,
                            const bool isBaseAnyType = false);
 
@@ -519,7 +496,12 @@ private:
       * Resolve a schema location attribute value to an input source.
       * Caller to delete the returned object.
       */
-    InputSource* resolveSchemaLocation(const XMLCh* const loc);
+    InputSource* resolveSchemaLocation
+    (
+        const XMLCh* const loc
+        , const XMLResourceIdentifier::ResourceIdentifierType resourceIdentitiferType
+        , const XMLCh* const nameSpace=0
+    );
 
     void restoreSchemaInfo(SchemaInfo* const toRestore,
                            SchemaInfo::ListType const aListType = SchemaInfo::INCLUDE,
@@ -549,6 +531,10 @@ private:
                          ComplexTypeInfo* const baseTypeInfo,
                          ComplexTypeInfo* const newTypeInfo);
 
+    void processElements(const DOMElement* const elem,
+                         XercesGroupInfo* const fromGroup,
+                         ComplexTypeInfo* const typeInfo);
+
     void copyGroupElements(const DOMElement* const elem,
                            XercesGroupInfo* const fromGroup,
                            XercesGroupInfo* const toGroup,
@@ -695,9 +681,14 @@ private:
     void processKeyRefFor(SchemaInfo* const aSchemaInfo,
                           ValueVectorOf<SchemaInfo*>* const infoList);
 
-    // Spaces are not allowed in URI, so %20 is used instead.
-    // Convert %20 to spaces before resolving the URI
-    void normalizeURI(const XMLCh* const systemURI, XMLBuffer& normalizedURI);
+    void processAttValue(const XMLCh* const attVal, XMLBuffer& aBuf);
+
+    // routine to generate synthetic annotations
+    XSAnnotation* generateSyntheticAnnotation(const DOMElement* const elem
+                                             , ValueVectorOf<DOMNode*>* nonXSAttList);
+
+    // routine to validate annotations
+    void validateAnnotations();
 
     // -----------------------------------------------------------------------
     //  Private constants
@@ -761,6 +752,7 @@ private:
     ValueVectorOf<SchemaElementDecl*>*             fIC_Elements;
     ValueVectorOf<const DOMElement*>*              fDeclStack;
     ValueVectorOf<unsigned int>**                  fGlobalDeclarations;
+    ValueVectorOf<DOMNode*>*                       fNonXSAttList;
     RefHashTableOf<ValueVectorOf<DOMElement*> >*   fIC_NodeListNS;
     RefHashTableOf<ValueVectorOf<unsigned int> >*  fIC_NamespaceDepthNS;
     RefHash2KeysTableOf<XMLCh>*                    fNotationRegistry;
@@ -768,10 +760,13 @@ private:
     RefHash2KeysTableOf<IdentityConstraint>*       fIdentityConstraintNames;
     RefHash2KeysTableOf<ElemVector>*               fValidSubstitutionGroups;
     RefHash2KeysTableOf<SchemaInfo>*               fSchemaInfoList;
-    GeneralAttributeCheck                          fAttributeCheck;
     XSDDOMParser*                                  fParser;
     XSDErrorReporter                               fXSDErrorReporter;
     XSDLocator*                                    fLocator;
+    MemoryManager*                                 fMemoryManager;
+    MemoryManager*                                 fGrammarPoolMemoryManager;
+    XSAnnotation*                                  fAnnotation;
+    GeneralAttributeCheck                          fAttributeCheck;
 
     friend class GeneralAttributeCheck;
 };
@@ -813,19 +808,6 @@ inline const XMLCh* TraverseSchema::getLocalPart(const XMLCh* const rawName) {
     return fStringPool->getValueForId(fStringPool->addOrFind(fBuffer.getRawBuffer()));
 }
 
-inline bool
-TraverseSchema::isValidRefDeclaration(const DOMElement* const elem) {
-
-    return !(XMLString::stringLen(elem->getAttribute(SchemaSymbols::fgATT_ABSTRACT)) != 0
-             || XMLString::stringLen(elem->getAttribute(SchemaSymbols::fgATT_NILLABLE)) != 0
-             || XMLString::stringLen(elem->getAttribute(SchemaSymbols::fgATT_BLOCK)) != 0
-             || XMLString::stringLen(elem->getAttribute(SchemaSymbols::fgATT_FINAL)) != 0
-             || XMLString::stringLen(elem->getAttribute(SchemaSymbols::fgATT_TYPE)) != 0
-             || XMLString::stringLen(elem->getAttribute(SchemaSymbols::fgATT_DEFAULT)) != 0
-             || XMLString::stringLen(elem->getAttribute(SchemaSymbols::fgATT_FIXED)) != 0
-             || XMLString::stringLen(elem->getAttribute(SchemaSymbols::fgATT_SUBSTITUTIONGROUP)) != 0);
-}
-
 inline
 const XMLCh* TraverseSchema::getElementAttValue(const DOMElement* const elem,
                                                 const XMLCh* const attName,
@@ -895,7 +877,7 @@ inline const XMLCh* TraverseSchema::genAnonTypeName(const XMLCh* const prefix) {
 
     XMLCh anonCountStr[16]; // a count of 15 digits should be enough
 
-    XMLString::binToText(fAnonXSTypeCount++, anonCountStr, 15, 10);
+    XMLString::binToText(fAnonXSTypeCount++, anonCountStr, 15, 10, fMemoryManager);
     fBuffer.set(prefix);
     fBuffer.append(anonCountStr);
 
@@ -931,32 +913,6 @@ inline void TraverseSchema::getRedefineNewTypeName(const XMLCh* const oldTypeNam
     }
 }
 
-inline void TraverseSchema::normalizeURI(const XMLCh* const systemURI,
-                                         XMLBuffer& normalizedURI)
-{
-    const XMLCh* pszSrc = systemURI;
-
-    normalizedURI.reset();
-
-    while (*pszSrc) {
-
-        if ((*(pszSrc) == chPercent)
-        &&  (*(pszSrc+1) == chDigit_2)
-        &&  (*(pszSrc+2) == chDigit_0))
-        {
-            pszSrc += 3;
-            normalizedURI.append(chSpace);
-        }
-        else if (*pszSrc == 0xFFFF) { //escaped character
-            pszSrc++;
-        }
-        else {
-            normalizedURI.append(*pszSrc);
-            pszSrc++;
-        }
-    }
-}
-
 XERCES_CPP_NAMESPACE_END
 
 #endif
