@@ -1,61 +1,21 @@
 /*
- * The Apache Software License, Version 1.1
- *
- * Copyright (c) 2001 The Apache Software Foundation.  All rights
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xerces" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written
- *    permission, please contact apache\@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation, and was
- * originally based on software copyright (c) 2001, International
- * Business Machines, Inc., http://www.ibm.com .  For more information
- * on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Copyright 2001-2004 The Apache Software Foundation.
+ * 
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 /*
- * $Id: XercesXPath.hpp,v 1.2 2002/11/04 14:47:42 tng Exp $
+ * $Id: XercesXPath.hpp,v 1.9 2004/09/08 13:57:00 peiyongz Exp $
  */
 
 #if !defined(XERCESXPATH_HPP)
@@ -69,6 +29,8 @@
 #include <xercesc/util/RefVectorOf.hpp>
 #include <xercesc/util/ValueVectorOf.hpp>
 
+#include <xercesc/internal/XSerializable.hpp>
+
 XERCES_CPP_NAMESPACE_BEGIN
 
 // ---------------------------------------------------------------------------
@@ -78,7 +40,7 @@ class XMLStringPool;
 class NamespaceScope;
 
 
-class VALIDATORS_EXPORT XercesNodeTest
+class VALIDATORS_EXPORT XercesNodeTest : public XSerializable, public XMemory
 {
 public:
     // -----------------------------------------------------------------------
@@ -88,15 +50,18 @@ public:
         QNAME = 1,
         WILDCARD = 2,
         NODE = 3,
-        NAMESPACE= 4
+        NAMESPACE= 4,
+        UNKNOWN
     };
 
     // -----------------------------------------------------------------------
     //  Constructors/Destructor
     // -----------------------------------------------------------------------
-    XercesNodeTest(const short type);
+    XercesNodeTest(const short type,
+                   MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager);
     XercesNodeTest(const QName* const qName);
-    XercesNodeTest(const XMLCh* const prefix, const unsigned int uriId);
+    XercesNodeTest(const XMLCh* const prefix, const unsigned int uriId,
+                   MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager);
     XercesNodeTest(const XercesNodeTest& other);
     ~XercesNodeTest() { delete fName; }
 
@@ -113,6 +78,13 @@ public:
     short getType() const { return fType; }
     QName* getName() const { return fName; }
 
+    /***
+     * Support for Serialization/De-serialization
+     ***/
+    DECL_XSERIALIZABLE(XercesNodeTest)
+
+    XercesNodeTest(MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager);
+
 private:
     // -----------------------------------------------------------------------
     //  Data members
@@ -125,7 +97,8 @@ private:
 /**
   * A location path step comprised of an axis and node test.
   */
-class VALIDATORS_EXPORT XercesStep {
+class VALIDATORS_EXPORT XercesStep : public XSerializable, public XMemory
+{
 public:
     // -----------------------------------------------------------------------
     //  Constants
@@ -134,7 +107,8 @@ public:
         CHILD = 1,
         ATTRIBUTE = 2,
         SELF = 3,
-        DESCENDANT = 4
+        DESCENDANT = 4,
+        UNKNOWN
     };
 
     // -----------------------------------------------------------------------
@@ -157,6 +131,13 @@ public:
     unsigned short getAxisType() const { return fAxisType; }
     XercesNodeTest* getNodeTest() const { return fNodeTest; }
 
+    /***
+     * Support for Serialization/De-serialization
+     ***/
+    DECL_XSERIALIZABLE(XercesStep)
+
+    XercesStep(MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager);
+
 private:
     // -----------------------------------------------------------------------
     //  Data members
@@ -169,13 +150,12 @@ private:
 /**
   * A location path representation for an XPath expression.
   */
-class VALIDATORS_EXPORT XercesLocationPath
+class VALIDATORS_EXPORT XercesLocationPath : public XSerializable, public XMemory
 {
 public:
     // -----------------------------------------------------------------------
     //  Constructors/Destructor
     // -----------------------------------------------------------------------
-    XercesLocationPath();
     XercesLocationPath(RefVectorOf<XercesStep>* const steps);
     ~XercesLocationPath() { delete fSteps; }
 
@@ -192,6 +172,13 @@ public:
     void addStep(XercesStep* const aStep);
     XercesStep* getStep(const unsigned int index) const;
 
+    /***
+     * Support for Serialization/De-serialization
+     ***/
+    DECL_XSERIALIZABLE(XercesLocationPath)
+
+    XercesLocationPath(MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager);
+
 private:
     // -----------------------------------------------------------------------
     //  Unimplemented contstructors and operators
@@ -206,7 +193,7 @@ private:
 };
 
 
-class VALIDATORS_EXPORT XercesXPath
+class VALIDATORS_EXPORT XercesXPath : public XSerializable, public XMemory
 {
 public:
     // -----------------------------------------------------------------------
@@ -276,7 +263,8 @@ public:
                 XMLStringPool* const stringPool,
                 NamespaceScope* const scopeContext,
                 const unsigned int emptyNamespaceId,
-                const bool isSelector = false);
+                const bool isSelector = false,
+                MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager);
 	~XercesXPath();
 
     // -----------------------------------------------------------------------
@@ -290,6 +278,15 @@ public:
     // -----------------------------------------------------------------------
     RefVectorOf<XercesLocationPath>* getLocationPaths() const;
 
+    /***
+     * Support for Serialization/De-serialization
+     ***/
+    DECL_XSERIALIZABLE(XercesXPath)
+
+    XercesXPath(MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager);
+
+    XMLCh* getExpression();
+
 private:
     // -----------------------------------------------------------------------
     //  Unimplemented contstructors and operators
@@ -311,10 +308,11 @@ private:
     unsigned int                     fEmptyNamespaceId;
     XMLCh*                           fExpression;
     RefVectorOf<XercesLocationPath>* fLocationPaths;
+    MemoryManager*                   fMemoryManager;
 };
 
 
-class VALIDATORS_EXPORT XPathScanner
+class VALIDATORS_EXPORT XPathScanner : public XMemory
 {
 public:
     // -----------------------------------------------------------------------
@@ -460,10 +458,6 @@ inline unsigned int XercesLocationPath::getStepSize() const {
 
 inline void XercesLocationPath::addStep(XercesStep* const aStep) {
 
-    if (!fSteps) {
-        fSteps = new RefVectorOf<XercesStep>(16);
-    }
-
     fSteps->addElement(aStep);
 }
 
@@ -492,6 +486,10 @@ inline RefVectorOf<XercesLocationPath>* XercesXPath::getLocationPaths() const {
     return fLocationPaths;
 }
 
+inline XMLCh* XercesXPath::getExpression() {
+    return fExpression;
+}
+
 XERCES_CPP_NAMESPACE_END
 
 #endif
